clear 
format long



a = intval('1.4')
b = intval('0.2')


%Parameterization Controls:
M = 20
N = 30

%Parameter Domains:
tau = intval('0.12');
domRad0 = intval('0.2');


scale_eigs = intval(1)


%Fixed Point (floating point formula)

x_1 = ((b-1)/(2*a)) + (sqrt((1-b)^2 +4*a))/(2*a);
y_1 = b*x_1;

x_2 = ((b-1)/(2*a)) - (sqrt((1-b)^2 +4*a))/(2*a);
y_2 = b*x_2;



f1_error = henonMap(x_1, y_1, a, b) - [x_1;y_1];
f2_error = henonMap(x_2, y_2, a, b) - [x_2;y_2];


%Fixed Point Parameter Expansion:==========================================

[x_M1, y_M1] = henon_FP_expansion_intval(x_1, y_1, a, b, M);
[x_M2, y_M2] = henon_FP_expansion_intval(x_2, y_2, a, b, M);




%==========================================================================
%Compute Eigenvalue and Eigenvector Series:
%==========================================================================

[lambda1, lambda2, xi1, xi2, ...
 lambda_series1, lambda_series2, ...
 xi_1_series1, xi_2_series1, xi_1_series2, xi_2_series2, ...
 delta_lambda1, delta_lambda2, delta_xi1, delta_xi2] = ...
        henon_linearSeriesAtFP_intval(...
    M, tau, scale_eigs, x_M1, y_M1, a, b);




[lambda3, lambda4, xi3, xi4, ...
 lambda_series3, lambda_series4, ...
 xi_1_series3, xi_2_series3, xi_1_series4, xi_2_series4, ...
 delta_lambda3, delta_lambda4, delta_xi3, delta_xi4] = ...
        henon_linearSeriesAtFP_intval(...
    M, tau, scale_eigs, x_M2, y_M2, a, b);



%==========================================================================
%Q and Q^-1 matrix series computations:
%==========================================================================


Q = [xi1, xi2];
Q_inv = inv(Q);

            
%==========================================================================
%Build the Q_omega matrix series:             
%==========================================================================
Q_0 = [xi_1_series1(1), xi_1_series2(1);
       xi_2_series1(1), xi_2_series2(1)]



%delta_Q = (delta_xi1 + delta_xi2)
delta_Q = 10^(-10)
Q_omega = intval(zeros(2, 2, M+1));
Q_omega(1:2, 1:2, 1) = Q_0;

for m = 1:M
    Q_m = [xi_1_series1(m+1), xi_1_series2(m+1);
           xi_2_series1(m+1), xi_2_series2(m+1)];
    Q_omega(1:2, 1:2, m+1) = Q_m;
end


%==========================================================================
%Invert and validate the Q series and compute matrix norms:
%==========================================================================

Q_omegaInv = matrixSeries_formalInverse_intval(Q_omega, M);
delta_Qinv = matrixSeries_validateInverse_intval(...
                   Q_omega, Q_omegaInv, delta_Q, tau, M)



Q_omega_sigmaNorm = 0;
Q_omegaInv_sigmaNorm = 0;

for m = 0:M
    Q_omega_sigmaNorm = Q_omega_sigmaNorm + ...
                        norm(Q_omega(1:2, 1:2, m+1), inf)*tau^m;
    Q_omegaInv_sigmaNorm = Q_omegaInv_sigmaNorm + ...
                        norm(Q_omegaInv(1:2, 1:2, m+1), inf)*tau^m;;
end


%==========================================================================
%Compute 1D manifold coefficients at omega = 0:
%==========================================================================
                                                 

 
[RN1, RN2] = henon_1D_manifold_intval([x_2, y_2], lambda3, xi3, ...
                                 a, b, N, intval(1));

                           
                                  
                             
                             
                             
%==========================================================================
%Compute rho partial of 1D PN at omega = 0 (i.e. first order term in omega).
%==========================================================================

[RN1_rho, RN2_rho] = ...
    henon_rhoPartial_intval([x_2, y_2], lambda3, xi3, ...
                            RN1, RN2, a, b, N);
                     


%==========================================================================
%Compute 1D manifold parameter expansion in omega:
%==========================================================================


Lambda3_powers = compute_seriesPowers_intval(lambda_series3, N, M);
 
[R_MN1, R_MN2] = henon_invariantBranch_intval(...
                    x_M2, y_M2, ...
                    RN1, RN2, RN1_rho, RN2_rho, ...
                    lambda_series3, Lambda3_powers, ...
                    xi_1_series3, xi_2_series3, M, N, a, b); 


 [Rmn1, Rmn2] = convertToPoly_2Dbranch_2Dspace(R_MN1, R_MN2, M, N);               
                
               
%==========================================================================
%Plotting:
%==========================================================================
                
 num_x = 100
 num_y = 20
 numParm = num_x*num_y
 parmDomain = zeros(numParm, 2);
 theXs = linspace(-2.5, 2.5, num_x);
 theYs = linspace(-0.12, 0.12, num_y);
 
 pointNum = 0;
 for n = 1:num_y
     for m = 1:num_x
        pointNum = pointNum + 1;
        parmDomain(pointNum, 1:2) = [theXs(m), theYs(n)];
     end
 end

branchManifold3 = zeros(numParm, 3);

pointNum = 0;
for n = 1:num_y
    for m = 1:num_x
        pointNum = pointNum + 1;
        thisParameter = parmDomain(pointNum, 1:2);
        thisPoint1 = polyval(Rmn1, thisParameter(2), thisParameter(1));
        thisPoint2 = polyval(Rmn2, thisParameter(2), thisParameter(1));
        branchManifold3(pointNum, 1:3) = ...
            [thisPoint1.mid, thisPoint2.mid, b.mid+theYs(n)];
    end
end
 
%figure
%hold on
%plot3(branchManifold(:, 1), branchManifold(:, 2), branchManifold(:, 3), 'b.')

%stopNow = stopHere

tauDom = 0.12;
numPoints = 100;

tauDom = linspace(-tauDom, tauDom, numPoints)               
fp_branch1 = zeros(numPoints, 2);
fp_branch2 = zeros(numPoints, 2);


for m = 1:numPoints
    fp_branch1(m, 1:2) = [horner(tauDom(m), x_M1.mid, M),
                         horner(tauDom(m), y_M1.mid, M)];
    fp_branch2(m, 1:2) = [horner(tauDom(m), x_M2.mid, M),
                         horner(tauDom(m), y_M2.mid, M)];
end


orbitPoints = 5000
orbit0 = zeros(orbitPoints, 2);
orbit1 = zeros(orbitPoints, 2);
orbit2 = zeros(orbitPoints, 2);
parm = 0.2;
parm1 = 0.3;
parm2 = 0.1;

p = henonMap(1,1,a.mid, parm);
p1 = henonMap(1,1,a.mid, parm1);
p2 = henonMap(1,1,a.mid, parm2);

for m = 1:500
    p = henonMap(p(1), p(2), a.mid, parm);
    p1 = henonMap(p1(1), p1(2), a.mid, parm1);
    p2 = henonMap(p2(1), p2(2), a.mid, parm2);
end

for m = 1:orbitPoints
    orbit0(m, 1:2) = p';
    p = henonMap(p(1), p(2), a.mid, parm);
    orbit1(m, 1:2) = p1';
    p1 = henonMap(p1(1), p1(2), a.mid, parm1);
    orbit2(m, 1:2) = p2';
    p2 = henonMap(p2(1), p2(2), a.mid, parm2);
end




figure 
hold on
plot3(branchManifold3(:, 1), branchManifold3(:, 2), branchManifold3(:, 3), 'g.')
plot3(orbit0(:,1), orbit0(:,2), parm*ones(orbitPoints, 1), 'g.');
plot3(orbit1(:,1), orbit1(:,2), parm1*ones(orbitPoints, 1), 'g.');
plot3(orbit2(:,1), orbit2(:,2), parm2*ones(orbitPoints, 1), 'g.');
plot3(fp_branch2(:, 1), fp_branch2(:, 2), b.mid+tauDom(:), 'k.') 
plot3(fp_branch1(:, 1), fp_branch1(:, 2), b.mid+tauDom(:), 'k.') 
plot3(x_2.mid, y_2.mid, parm, 'k*');
                


















