clear 
format long



a = intval('1.4')
b = intval('0.5')


%Parameterization Controls:
M = 10
N = 50


%Parameter Domains:
tau = intval('0.1');
domRad0 = intval('0.5');


scale_eigs = intval(1)


%Fixed Point (floating point formula)

x_1 = ((b-1)/(2*a)) + (sqrt((1-b)^2 +4*a))/(2*a);
y_1 = b*x_1;

x_2 = ((b-1)/(2*a)) - (sqrt((1-b)^2 +4*a))/(2*a);
y_2 = b*x_2;



f1_error = henonMap(x_1, y_1, a, b) - [x_1;y_1];
f2_error = henonMap(x_2, y_2, a, b) - [x_2;y_2];


%Fixed Point Parameter Expansion:==========================================

[x_M1, y_M1] = henon_FP_expansion_intval(x_1, y_1, a, b, M);
[x_M2, y_M2] = henon_FP_expansion_intval(x_2, y_2, a, b, M);



%==========================================================================
%Compute Eigenvalue and Eigenvector Series:
%==========================================================================

[lambda1_1, lambda1_2, xi1_1, xi1_2, ...
 lambda_series1, lambda_series2, ...
 xi1_1_series1, xi1_2_series1, xi1_1_series2, xi1_2_series2, ...
 delta1_lambda1, delta1_lambda2, delta1_xi1, delta1_xi2] = ...
        henon_linearSeriesAtFP_intval(...
    M, tau, intval(1), x_M1, y_M1, a, b);



[lambda2_1, lambda2_2, xi2_1, xi2_2, ...
 lambda2_series1, lambda2_series2, ...
 xi2_1_series1, xi2_2_series1, xi2_1_series2, xi2_2_series2, ...
 delta2_lambda1, delta2_lambda2, delta2_xi1, delta2_xi2] = ...
        henon_linearSeriesAtFP_intval(...
    M, tau, intval(1), x_M2, y_M2, a, b);






%==========================================================================
%Compute 1D manifold coefficients at omega = 0:
%==========================================================================


[PN1_1, PN1_2] = henon_1D_manifold_intval([x_1, y_1], lambda1_2, xi1_2, ...
                                 a, b, N, intval(1));

                           
[QN1_1, QN1_2] = henon_1D_manifold_intval([x_1, y_1], lambda1_1, xi1_1, ...
                                 a, b, N, intval(1));
   

                             
                             

[PN2_1, PN2_2] = henon_1D_manifold_intval([x_2, y_2], lambda2_2, xi2_2, ...
                                 a, b, N, intval(1));
                             
                             
[QN2_1, QN2_2] = henon_1D_manifold_intval([x_2, y_2], lambda2_1, xi2_1, ...
                                 a, b, N, intval(1));
                             
                                                      
                             
theta = 10.0 

numPoints = 2000;
Ws1Loc = zeros(numPoints, 2);
Wu1Loc = zeros(numPoints, 2);
parmDom = linspace(-theta, theta, numPoints);
for n = 1:numPoints
    Ws1Loc(n, :) = [horner(parmDom(n), PN1_1.mid, N),
                   horner(parmDom(n), PN1_2.mid, N)];
    Wu1Loc(n, :) = [horner(parmDom(n), QN1_1.mid, N),
                   horner(parmDom(n), QN1_2.mid, N)];                    
end

                            
theta = 10.0 

numPoints = 2000;
Ws2Loc = zeros(numPoints, 2);
Wu2Loc = zeros(numPoints, 2);
parmDom = linspace(-theta, theta, numPoints);
for n = 1:numPoints
    Ws2Loc(n, :) = [horner(parmDom(n), PN2_1.mid, N),
                   horner(parmDom(n), PN2_2.mid, N)];
    Wu2Loc(n, :) = [horner(parmDom(n), QN2_1.mid, N),
                   horner(parmDom(n), QN2_2.mid, N)];                    
end


figure
hold on 
plot(Ws1Loc(:, 1), Ws1Loc(:,2), 'r.');
plot(Wu1Loc(:, 1), Wu1Loc(:,2), 'b.');
plot(Ws2Loc(:, 1), Ws2Loc(:,2), 'r.');
plot(Wu2Loc(:, 1), Wu2Loc(:,2), 'b.');
plot(x_1.mid, y_1.mid, 'ko')
plot(x_2.mid, y_2.mid, 'ko')
plot(x_1.mid, y_1.mid, 'k*')
plot(x_2.mid, y_2.mid, 'k*')

axis([-2 2 -1.0 1.0])



