clear 
format long



a = intval('1.4')
b = intval('0.3')


%Parameterization Controls:
M = 10
N = 50


%Parameter Domains:
tau = intval('0.1');
domRad0 = intval('0.5');


scale_eigs = intval(1)


%Fixed Point (floating point formula)

x_1 = ((b-1)/(2*a)) + (sqrt((1-b)^2 +4*a))/(2*a);
y_1 = b*x_1;

x_2 = ((b-1)/(2*a)) - (sqrt((1-b)^2 +4*a))/(2*a);
y_2 = b*x_2;



f1_error = henonMap(x_1, y_1, a, b) - [x_1;y_1];
f2_error = henonMap(x_2, y_2, a, b) - [x_2;y_2];


%Fixed Point Parameter Expansion:==========================================

[x_M1, y_M1] = henon_FP_expansion_intval(x_1, y_1, a, b, M);
[x_M2, y_M2] = henon_FP_expansion_intval(x_2, y_2, a, b, M);





%Test Code:
fpTest1 = [horner(tau, x_M1, M); horner(tau, y_M1, M)]
fp1_error = sup(norm(...
    henonMap_intval(fpTest1(1), fpTest1(2), a, b+tau) - fpTest1, inf)) 


fpTest2 = [horner(tau, x_M2, M); horner(tau, y_M2, M)]
fp2_error = sup(norm(...
    henonMap_intval(fpTest2(1), fpTest2(2), a, b+tau) - fpTest2, inf)) 

 


%==========================================================================
%Compute Eigenvalue and Eigenvector Series:
%==========================================================================

[lambda1, lambda2, xi1, xi2, ...
 lambda_series1, lambda_series2, ...
 xi_1_series1, xi_2_series1, xi_1_series2, xi_2_series2, ...
 delta_lambda1, delta_lambda2, delta_xi1, delta_xi2] = ...
        henon_linearSeriesAtFP_intval(...
    M, tau, intval(1), x_M1, y_M1, a, b);



%Test Code:
Dfp0_test = [-2*a*fpTest1(1), 1;
                b+tau, 0];   
[U_temp, Sigma_temp] = eigs(Dfp0_test.mid);
Sigma_temp
%Verify eigen data plus:
[lambda1_test, xi1_test] = ...
     verifyeig(Dfp0_test, Sigma_temp(1, 1), U_temp(:,1));
 xi1_test = 1*xi1_test
 [lambda2_test, xi2_test] = ...
     verifyeig(Dfp0_test, Sigma_temp(2, 2), U_temp(:,2));
 xi2_test = 1*xi2_test;
 
lambda1_error = sup(abs(horner(tau, lambda_series1, M) - lambda1_test)) 
lambda2_error = sup(abs(horner(tau, lambda_series2, M) - lambda2_test))
 
 
xi1_error = ...
    sup(norm(...
    [horner(tau, xi_1_series1, M); horner(tau, xi_2_series1, M)] - ...
     xi1_test, inf))

 
xi2_error = ...
    sup(norm(...
    [horner(tau, xi_1_series2, M); horner(tau, xi_2_series2, M)] - ...
     xi2_test, inf)) 
 
%==========================================================================
%Q and Q^-1 matrix series computations:
%==========================================================================


Q = [xi1, xi2];
Q_inv = inv(Q);

            
%==========================================================================
%Build the Q_omega matrix series:             
%==========================================================================
Q_0 = [xi_1_series1(1), xi_1_series2(1);
       xi_2_series1(1), xi_2_series2(1)]



%delta_Q = (delta_xi1 + delta_xi2)
delta_Q = 10^(-10)
Q_omega = intval(zeros(2, 2, M+1));
Q_omega(1:2, 1:2, 1) = Q_0;

for m = 1:M
    Q_m = [xi_1_series1(m+1), xi_1_series2(m+1);
           xi_2_series1(m+1), xi_2_series2(m+1)];
    Q_omega(1:2, 1:2, m+1) = Q_m;
end


%==========================================================================
%Invert and validate the Q series and compute matrix norms:
%==========================================================================

Q_omegaInv = matrixSeries_formalInverse_intval(Q_omega, M);
delta_Qinv = matrixSeries_validateInverse_intval(...
                   Q_omega, Q_omegaInv, delta_Q, tau, M)



Q_omega_sigmaNorm = 0;
Q_omegaInv_sigmaNorm = 0;

for m = 0:M
    Q_omega_sigmaNorm = Q_omega_sigmaNorm + ...
                        norm(Q_omega(1:2, 1:2, m+1), inf)*tau^m;
    Q_omegaInv_sigmaNorm = Q_omegaInv_sigmaNorm + ...
                        norm(Q_omegaInv(1:2, 1:2, m+1), inf)*tau^m;;
end


%==========================================================================
%Compute 1D manifold coefficients at omega = 0:
%==========================================================================


[PN1, PN2] = henon_1D_manifold_intval([x_1, y_1], lambda2, xi2, ...
                                 a, b, N, intval(1));



[QN1, QN2] = henon_1D_manifold_intval([x_1, y_1], lambda1, xi1, ...
                                 a, b, N, intval(1));
                             
                                                      
                             
                             
theta = 3.0;
K = 10;
p = [horner(theta, PN1.mid, N);
     horner(theta, PN2.mid, N)]                           
                           
for k = 1:K
    p = henonMap(p(1), p(2), a.mid, b.mid);
end

p
Ws_error = norm([x_1.mid; y_1.mid] - p, inf)                             
 
p = [horner(theta, QN1.mid, N);
     horner(theta, QN2.mid, N)]                           
                           
for k = 1:K
    p = henonMapInv(p(1), p(2), a.mid, b.mid);
end
p

Wu_error = norm([x_1.mid; y_1.mid] - p, inf)                             
            



numPoints = 200;
WsLoc = zeros(numPoints, 2);
WuLoc = zeros(numPoints, 2);
parmDom = linspace(-theta, theta, numPoints);
for n = 1:numPoints
    WsLoc(n, :) = [horner(parmDom(n), PN1.mid, N),
                   horner(parmDom(n), PN2.mid, N)];
    WuLoc(n, :) = [horner(parmDom(n), QN1.mid, N),
                   horner(parmDom(n), QN2.mid, N)];                    
end

figure
hold on 
plot(x_1.mid, y_1.mid, 'ko')
plot(WsLoc(:, 1), WsLoc(:,2), 'r.');
plot(WuLoc(:, 1), WuLoc(:,2), 'b.');

stopNow = stopHere
                               
%==========================================================================                                  
%1D Manifold Residual:     
%==========================================================================

                              
                                  
[goodComputation, delta0] = lorenzApostori_1D_intval(...
                              N, intval(1), sigma, rho, beta, ...
                              domRad0, PN1, PN2, PN3, lambda3, Q, Q_inv);                             


delta0  



                     
%==========================================================================
%Compute rho partial of 1D PN at omega = 0 (i.e. first order term in omega).
%==========================================================================


[PN1_rho, PN2_rho, PN3_rho] = lorenz_1D_rhoPartial_at0_intval(...
                                      p0, lambda3, xi3, ...
                                      PN1, PN2, PN3, ...
                                      sigma, rho, beta, ... 
                                      N, intval(1));


%==========================================================================
%Omega Partial at omega = 0; 1D Residuial:
%==========================================================================


E_N_residual_rho1D= residual_1D_rhoPartial_at0_intval(PN1, PN2, PN3, ...
    PN1_rho, PN2_rho, PN3_rho, ...
    lambda_series3, domRad0, sigma, beta, rho, N)



%==========================================================================
%Compute 1D manifold parameter expansion in omega:
%==========================================================================



[P_MN1, P_MN2, P_MN3] = lorenz_1D_unstableBranch_at0_intval(...
    PN1, PN2, PN3, PN1_rho, PN2_rho, PN3_rho, lambda_series3,...
    xi_1_series3, xi_2_series3, xi_3_series3, ...
    M, N, sigma, beta, rho);



%==========================================================================
%==========================================================================
%Validate individual tails: p_alpha^M(omega)
%==========================================================================
%==========================================================================

%Compute the residuals:
HM_alpha_bounds = compute_alphaM_bounds1D(P_MN1, P_MN2, P_MN3, ...
                              lambda_series3, delta_xi3, ...
                              p0, tau, intval(1), ...
                              sigma, rho, beta, M, N);

                                
                          
%Compute Bounds on Iverse Matrices:                          

resonanceBounds=compute_resonanceBounds1D(...
    lambda_series1, lambda_series2, lambda_series3, ...
    delta_lambda1, delta_lambda2, delta_lambda3, tau, N, M)
                         
    

%Compute Validated bounds on the coefficient tails:                          
H_alpha_validatedBounds= validated_coeff_bounds1D(P_MN1, P_MN2, P_MN3, ...
         Q_omega_sigmaNorm, delta_Q, Q_omegaInv_sigmaNorm, delta_Qinv,...
         resonanceBounds, HM_alpha_bounds, delta_lambda3, delta_xi3, ...
         p0, tau, intval(1), sigma, rho, beta, M, N);                       

 
     
 

%==========================================================================
%Omega Expansion 1D Residual Error:
%==========================================================================


E_MN_residual_1Dbranch = residual_unstableBranch_at0_intval(...
            P_MN1, P_MN2, P_MN3, lambda_series3, ...
            sigma, beta, rho, domRad0, tau, M, N)



 
%========================================================================== 
%Validated A-Posteriori Bounds:
%==========================================================================


s = intval('0.05')

[delta_HN, goodComputation] = validatedBranchError_unstableAtOrigin(...
    P_MN1, P_MN2, P_MN3, domRad0, tau, s, E_MN_residual_1Dbranch, ...
    Q_omega_sigmaNorm, delta_Q, Q_omegaInv_sigmaNorm, delta_Qinv,...
    lambda_series3, delta_lambda3,...
    H_alpha_validatedBounds, M, N, sigma, beta, rho);


delta_HN


