clear 
format long



a = intval('1.4')
b = intval('0.2')


%Parameterization Controls:
M = 30
N = 30

%Parameter Domains:
tau = intval('0.1');
domRad0 = intval('0.2');


scale_eigs = intval(1)


%Fixed Point (floating point formula)

x_1 = ((b-1)/(2*a)) + (sqrt((1-b)^2 +4*a))/(2*a);
y_1 = b*x_1;

x_2 = ((b-1)/(2*a)) - (sqrt((1-b)^2 +4*a))/(2*a);
y_2 = b*x_2;



f1_error = henonMap(x_1, y_1, a, b) - [x_1;y_1];
f2_error = henonMap(x_2, y_2, a, b) - [x_2;y_2];


%Fixed Point Parameter Expansion:==========================================

[x_M1, y_M1] = henon_FP_expansion_intval(x_1, y_1, a, b, M);
[x_M2, y_M2] = henon_FP_expansion_intval(x_2, y_2, a, b, M);




%==========================================================================
%Compute Eigenvalue and Eigenvector Series:
%==========================================================================

[lambda1, lambda2, xi1, xi2, ...
 lambda_series1, lambda_series2, ...
 xi_1_series1, xi_2_series1, xi_1_series2, xi_2_series2, ...
 delta_lambda1, delta_lambda2, delta_xi1, delta_xi2] = ...
        henon_linearSeriesAtFP_intval(...
    M, tau, scale_eigs, x_M1, y_M1, a, b);




[lambda3, lambda4, xi3, xi4, ...
 lambda_series3, lambda_series4, ...
 xi_1_series3, xi_2_series3, xi_1_series4, xi_2_series4, ...
 delta_lambda3, delta_lambda4, delta_xi3, delta_xi4] = ...
        henon_linearSeriesAtFP_intval(...
    M, tau, scale_eigs, x_M2, y_M2, a, b);



%==========================================================================
%Q and Q^-1 matrix series computations:
%==========================================================================


Q = [xi1, xi2];
Q_inv = inv(Q);

            
%==========================================================================
%Build the Q_omega matrix series:             
%==========================================================================
Q_0 = [xi_1_series1(1), xi_1_series2(1);
       xi_2_series1(1), xi_2_series2(1)]



%delta_Q = (delta_xi1 + delta_xi2)
delta_Q = 10^(-10)
Q_omega = intval(zeros(2, 2, M+1));
Q_omega(1:2, 1:2, 1) = Q_0;

for m = 1:M
    Q_m = [xi_1_series1(m+1), xi_1_series2(m+1);
           xi_2_series1(m+1), xi_2_series2(m+1)];
    Q_omega(1:2, 1:2, m+1) = Q_m;
end


%==========================================================================
%Invert and validate the Q series and compute matrix norms:
%==========================================================================

Q_omegaInv = matrixSeries_formalInverse_intval(Q_omega, M);
delta_Qinv = matrixSeries_validateInverse_intval(...
                   Q_omega, Q_omegaInv, delta_Q, tau, M)



Q_omega_sigmaNorm = 0;
Q_omegaInv_sigmaNorm = 0;

for m = 0:M
    Q_omega_sigmaNorm = Q_omega_sigmaNorm + ...
                        norm(Q_omega(1:2, 1:2, m+1), inf)*tau^m;
    Q_omegaInv_sigmaNorm = Q_omegaInv_sigmaNorm + ...
                        norm(Q_omegaInv(1:2, 1:2, m+1), inf)*tau^m;;
end


%==========================================================================
%Compute 1D manifold coefficients at omega = 0:
%==========================================================================


[PN1, PN2] = henon_1D_manifold_intval([x_1, y_1], lambda2, xi2, ...
                                 a, b, N, intval(1));



[QN1, QN2] = henon_1D_manifold_intval([x_1, y_1], lambda1, xi1, ...
                                 a, b, N, intval(1));
                             
                                                      

 
[RN1, RN2] = henon_1D_manifold_intval([x_2, y_2], lambda3, xi3, ...
                                 a, b, N, intval(1));



[SN1, SN2] = henon_1D_manifold_intval([x_2, y_2], lambda4, xi4, ...
                                 a, b, N, intval(1));
                             
                                  
                             
                             
                             
%==========================================================================
%Compute rho partial of 1D PN at omega = 0 (i.e. first order term in omega).
%==========================================================================



[PN1_rho, PN2_rho] = ...
    henon_rhoPartial_intval([x_1, y_1], lambda2, xi2, ...
                            PN1, PN2, a, b, N);



[QN1_rho, QN2_rho] = ...
    henon_rhoPartial_intval([x_1, y_1], lambda1, xi1, ...
                            QN1, QN2, a, b, N);


[RN1_rho, RN2_rho] = ...
    henon_rhoPartial_intval([x_2, y_2], lambda3, xi3, ...
                            RN1, RN2, a, b, N);

[SN1_rho, SN2_rho] = ...
    henon_rhoPartial_intval([x_2, y_2], lambda4, xi4, ...
                            SN1, SN2, a, b, N);
                     


%==========================================================================
%Compute 1D manifold parameter expansion in omega:
%==========================================================================


Lambda1_powers = compute_seriesPowers_intval(lambda_series1, N, M);
Lambda2_powers = compute_seriesPowers_intval(lambda_series2, N, M);

Lambda3_powers = compute_seriesPowers_intval(lambda_series3, N, M);
Lambda4_powers = compute_seriesPowers_intval(lambda_series4, N, M);

%omega = 0.1;
%n = 15;
%lambda_omega = horner(omega, lambda_series2, M)
%lambda_omega_npow = lambda_omega^n
%lambda_n = Lambda2_powers(:, n+1);
%testLambda_pow = horner(omega, lambda_n, M)
%error = abs(lambda_omega_npow - testLambda_pow)


%This is the hard one...
[P_MN1, P_MN2] = henon_invariantBranch_intval(...
                    x_M1, y_M1, ...
                    PN1, PN2, PN1_rho, PN2_rho, ...
                    lambda_series2, Lambda2_powers, ...
                    xi_1_series2, xi_2_series2, M, N, a, b); 


[Q_MN1, Q_MN2] = henon_invariantBranch_intval(...
                    x_M1, y_M1, ...
                    QN1, QN2, QN1_rho, QN2_rho, ...
                    lambda_series1, Lambda1_powers, ...
                    xi_1_series1, xi_2_series1, M, N, a, b); 


 
[R_MN1, R_MN2] = henon_invariantBranch_intval(...
                    x_M2, y_M2, ...
                    RN1, RN2, RN1_rho, RN2_rho, ...
                    lambda_series3, Lambda3_powers, ...
                    xi_1_series3, xi_2_series3, M, N, a, b); 



 
[S_MN1, S_MN2] = henon_invariantBranch_intval(...
                    x_M2, y_M2, ...
                    SN1, SN2, SN1_rho, SN2_rho, ...
                    lambda_series4, Lambda4_powers, ...
                    xi_1_series4, xi_2_series4, M, N, a, b); 


               
   
                
stopNow = stopHere                

omega = 0.1;
x_1_test = horner(omega, x_M1, M);
y_1_test = horner(omega, y_M1, M);
lambda2_test = horner(omega, lambda_series2, M);
xi2_test = [horner(omega, xi_1_series2, M);
            horner(omega, xi_2_series2, M)];

[PN1_test, PN2_test] = ...
    henon_1D_manifold_intval([x_1_test, y_1_test], ...
                             lambda2_test, xi2_test, ...
                             a, b+omega, N, intval(1));
                             
               
n = 20;
PMN1_at_omega = horner(omega, P_MN1(:, n+1), M);
 
error = ...
    abs(PMN1_at_omega - PN1_test(n+1))
relative_error = ...
    abs((PMN1_at_omega - PN1_test(n+1))/PN1_test(n+1))                
                      
                
 


omega = 0.1;
x_1_test = horner(omega, x_M1, M);
y_1_test = horner(omega, y_M1, M);
lambda1_test = horner(omega, lambda_series1, M);
xi1_test = [horner(omega, xi_1_series1, M);
            horner(omega, xi_2_series1, M)];

[QN1_test, QN2_test] = ...
    henon_1D_manifold_intval([x_1_test, y_1_test], ...
                             lambda1_test, xi1_test, ...
                             a, b+omega, N, intval(1));
                             
               
n = 4;
QMN1_at_omega = horner(omega, Q_MN1(:, n+1), M);
 
error = ...
    abs(QMN1_at_omega - QN1_test(n+1))
relative_error = ...
    abs((QMN1_at_omega - QN1_test(n+1))/QN1_test(n+1))                
                      
      


                
stopNow = stopHere                
                

omega = 0.1;
x_2_test = horner(omega, x_M2, M);
y_2_test = horner(omega, y_M2, M);
lambda3_test = horner(omega, lambda_series3, M);
xi3_test = [horner(omega, xi_1_series3, M);
            horner(omega, xi_2_series3, M)];

[RN1_test, RN2_test] = ...
    henon_1D_manifold_intval([x_2_test, y_2_test], ...
                             lambda3_test, xi3_test, ...
                             a, b+omega, N, intval(1));
                             
               
n = 4;
RMN1_at_omega = horner(omega, R_MN1(:, n+1), M);
 

error = ...
    abs(RMN1_at_omega - RN1_test(n+1))
relative_error = ...
    abs((RMN1_at_omega - RN1_test(n+1))/RN1_test(n+1))                
                
                
             

omega = 0.12;
x_2_test = horner(omega, x_M2, M);
y_2_test = horner(omega, y_M2, M);
lambda4_test = horner(omega, lambda_series4, M);
xi4_test = [horner(omega, xi_1_series4, M);
            horner(omega, xi_2_series4, M)];

[SN1_test, SN2_test] = ...
    henon_1D_manifold_intval([x_2_test, y_2_test], ...
                             lambda4_test, xi4_test, ...
                             a, b+omega, N, intval(1));
                             
               
n = 5;
SMN1_at_omega = horner(omega, S_MN1(:, n+1), M);
 

error = ...
    abs(SMN1_at_omega - SN1_test(n+1))
relative_error = ...
    abs((SMN1_at_omega - SN1_test(n+1))/SN1_test(n+1))                
          







