clear 
format long



a = intval('1.4')
b = intval('0.3')


%Parameterization Controls:
M = 2
N = 30

%Parameter Domains:
tau = intval('0.12');
domRad0 = intval('0.2');


scale_eigs = intval(1)


%Fixed Point (floating point formula)

x_1 = ((b-1)/(2*a)) + (sqrt((1-b)^2 +4*a))/(2*a);
y_1 = b*x_1;

x_2 = ((b-1)/(2*a)) - (sqrt((1-b)^2 +4*a))/(2*a);
y_2 = b*x_2;


%Fixed Point Parameter Expansion:==========================================

[x_M1, y_M1] = henon_FP_expansion_intval(x_1, y_1, a, b, M);
[x_M2, y_M2] = henon_FP_expansion_intval(x_2, y_2, a, b, M);




%==========================================================================
%Compute Eigenvalue and Eigenvector Series:
%==========================================================================

[lambda1, lambda2, xi1, xi2, ...
 lambda_series1, lambda_series2, ...
 xi_1_series1, xi_2_series1, xi_1_series2, xi_2_series2, ...
 delta_lambda1, delta_lambda2, delta_xi1, delta_xi2] = ...
        henon_linearSeriesAtFP_intval(...
    M, tau, scale_eigs, x_M1, y_M1, 10^(-14), a, b);




[lambda3, lambda4, xi3, xi4, ...
 lambda_series3, lambda_series4, ...
 xi_1_series3, xi_2_series3, xi_1_series4, xi_2_series4, ...
 delta_lambda3, delta_lambda4, delta_xi3, delta_xi4] = ...
        henon_linearSeriesAtFP_intval(...
    M, tau, scale_eigs, x_M2, y_M2, 10^(-14), a, b);



%==========================================================================
%Q and Q^-1 matrix series computations:
%==========================================================================


Q = [xi1, xi2];
Q_inv = inv(Q);

            
%==========================================================================
%Build the Q_omega matrix series:             
%==========================================================================
Q_0 = [xi_1_series1(1), xi_1_series2(1);
       xi_2_series1(1), xi_2_series2(1)]



%delta_Q = (delta_xi1 + delta_xi2)
delta_Q = 10^(-10)
Q_omega = intval(zeros(2, 2, M+1));
Q_omega(1:2, 1:2, 1) = Q_0;

for m = 1:M
    Q_m = [xi_1_series1(m+1), xi_1_series2(m+1);
           xi_2_series1(m+1), xi_2_series2(m+1)];
    Q_omega(1:2, 1:2, m+1) = Q_m;
end


%==========================================================================
%Invert and validate the Q series and compute matrix norms:
%==========================================================================

Q_omegaInv = matrixSeries_formalInverse_intval(Q_omega, M);
delta_Qinv = matrixSeries_validateInverse_intval(...
                   Q_omega, Q_omegaInv, delta_Q, tau, M)



Q_omega_sigmaNorm = 0;
Q_omegaInv_sigmaNorm = 0;

for m = 0:M
    Q_omega_sigmaNorm = Q_omega_sigmaNorm + ...
                        norm(Q_omega(1:2, 1:2, m+1), inf)*tau^m;
    Q_omegaInv_sigmaNorm = Q_omegaInv_sigmaNorm + ...
                        norm(Q_omegaInv(1:2, 1:2, m+1), inf)*tau^m;;
end


%==========================================================================
%Compute 1D manifold coefficients at omega = 0:
%==========================================================================
                                                 

 
                         
                                  

[PN1, PN2] = henon_1D_manifold_intval([x_1, y_1], lambda1, xi1, ...
                                 a, b, N, intval(1));

[QN1, QN2] = henon_1D_manifold_intval([x_1, y_1], lambda2, xi2, ...
                                 a, b, N, intval(1));
                              
[RN1, RN2] = henon_1D_manifold_intval([x_2, y_2], lambda3, xi3, ...
                                 a, b, N, intval(1));

[SN1, SN2] = henon_1D_manifold_intval([x_2, y_2], lambda4, xi4, ...
                                 a, b, N, intval(1));
    
                             
                             
                             

                             
                             
                           
                             
nuRad1 = 1.75;
numPoints1 = 300;
nuDom1 = linspace(-nuRad1, nuRad1, numPoints1);               
W_1 = zeros(numPoints1, 2);
for m = 1:numPoints1
    W_1(m, 1:2) = [horner(nuDom1(m), PN1.mid, N),
                   horner(nuDom1(m), PN2.mid, N)];      
end                             


nuRad2 = 2.75;
numPoints2 = 1000;
nuDom2 = linspace(-nuRad2, nuRad2, numPoints2);               
W_2 = zeros(numPoints2, 2);
for m = 1:numPoints2
    W_2(m, 1:2) = [horner(nuDom2(m), QN1.mid, N),
                   horner(nuDom2(m), QN2.mid, N)];      
end


%------------- Fundamental Domain 2 ---------------------------------------

numPoints2 = 200

I2_fd_left  = linspace(-0.9*nuRad2, -0.9*nuRad1*lambda2.mid, numPoints2);
I2_fd_right = linspace(0.9*nuRad2*lambda2.mid, 0.9*nuRad1,  numPoints2);

W2_fd_left  = zeros(numPoints2, 2);
W2_fd_right = zeros(numPoints2, 2);

for m = 1:numPoints2
    W2_fd_left(m, 1:2)  = [horner(I2_fd_left(m), QN1.mid, N),
                           horner(I2_fd_left(m), QN2.mid, N)];
    W2_fd_right(m, 1:2) = [horner(I2_fd_right(m), QN1.mid, N),
                           horner(I2_fd_right(m), QN2.mid, N)];                  
end


for m = 1:numPoints2
    finvW2_fd_right(m, :) = henonMapInv(...
                       W2_fd_right(m, 1), W2_fd_right(m,2), a.mid, b.mid)';
end



%------------- Fundamental Domain 1-----------------------------------------

numPoints1 = 2000

I1_fd_left  = linspace(-0.9*nuRad1, -0.9*nuRad1/abs(lambda1.mid), numPoints1);
I1_fd_right = linspace(0.9*nuRad1/abs(lambda1.mid), 0.9*nuRad1,  numPoints1);

W1_fd_left  = zeros(numPoints1, 2);
W1_fd_right = zeros(numPoints1, 2);

for m = 1:numPoints1
    W1_fd_left(m, 1:2)  = [horner(I1_fd_left(m), PN1.mid, N),
                           horner(I1_fd_left(m), PN2.mid, N)];
    W1_fd_right(m, 1:2) = [horner(I1_fd_right(m), PN1.mid, N),
                           horner(I1_fd_right(m), PN2.mid, N)];                  
end



fkW1_fd_right = W1_fd_right;
for k = 1:12
    for m = 1:numPoints1
    fkW1_fd_right(m, :) = henonMap(...
                       fkW1_fd_right(m, 1), fkW1_fd_right(m,2), a.mid, b.mid)';
    end

    
    dist = zeros(numPoints1, 1);
    minIndices = zeros(numPoints1, 1);
    for m = 1:numPoints1
       m
       numPoints1
       thisPoint = fkW1_fd_right(m, :);
       theMin = 1000;
       for n = 1:numPoints2
          thisMin = norm(thisPoint - finvW2_fd_right(n, :));
          if thisMin < theMin
             theMin = thisMin; 
             minIndices(m) = n;
          end
       end
       dist(m) = theMin;
    end
    
 figure
 hold on
 pointNum = linspace(1, numPoints1, numPoints1);
 plot(pointNum(:), log(dist(:))/log(10))   
 
 
figure
hold on
plot(finvW2_fd_right(:,1), finvW2_fd_right(:,2), 'r.')
plot(W1_fd_right(:, 1), W1_fd_right(:, 2), 'g*')
plot(fkW1_fd_right(:, 1), fkW1_fd_right(:, 2), 'm.')
plot(x_2.mid, y_2.mid, 'k*');
plot(x_1.mid, y_1.mid, 'ko');  
axis([-1.5 1.5 -0.8 0.8])
 
 
 %if k == 4
 %    stopNow = stopHere
 %end
 


end




n_u = k
n_s = 1
numIntersections = 2
indexNumbers = [665, 1064];

theseIntersectionParameters = zeros(numIntersections, 5);
theseIntersections = zeros(numIntersections, 2);

for m = 1:numIntersections

thisIndex = indexNumbers(m);
matchIndex = minIndices(thisIndex);
    
Pu_xn = [horner(I1_fd_right(thisIndex), PN1.mid, N);
         horner(I1_fd_right(thisIndex), PN2.mid, N)];
%Image of z_n = (s_n, t_n) Pu:
Ps_xn = [horner(I2_fd_right(matchIndex), QN1.mid, N);
         horner(I2_fd_right(matchIndex), QN2.mid, N)];
 
 %Evaluate Phi at x_n:
 Phi_xn = inverse_iterate_point_k(Ps_xn, n_s, a.mid, b.mid) ...
     - iterate_point_k(Pu_xn, n_u, a.mid, b.mid);
 thisError = norm(Phi_xn, inf)


[theta, tau, error] = henonHomoclinic(...
                      I1_fd_right(thisIndex), I2_fd_right(matchIndex), n_u, n_s, ...
    PN1.mid, PN2.mid, QN1.mid, QN2.mid, N, N, a.mid, b.mid)

error

Pu_xn = [horner(theta, PN1.mid, N);
         horner(theta, PN2.mid, N)];

theseIntersectionParameters(m, 1:5) = [theta, tau, n_u, n_s, error];
theseIntersections(m, 1:2) = Pu_xn';

end

save intersectionLevel_4.txt theseIntersectionParameters theseIntersections -ASCII

numIterates = 17
iterateNum = 1
     q = theseIntersections(iterateNum, :);
     p = theseIntersections(iterateNum, :);
 
     orbit = zeros(2*numIterates+1, 2);
     
     orbit(numIterates+1, 1:2) = q';
     
 for k = 1:numIterates
    q = henonMap(q(1), q(2), a.mid, b.mid);
    orbit(numIterates+1+k, 1:2) = q';
    p = henonMapInv(p(1), p(2), a.mid, b.mid);
    orbit(numIterates+1-k, 1:2) = p';
 end
     
 
 time = linspace(-numIterates, numIterates, 2*numIterates+1);
 
 figure
 hold on

 plot(time(:), orbit(:,1), 'k');
 
 
 
 
 
 
 
 stopNow = stopHere
 
figure
hold on
%plot(orbit0(:,1), orbit0(:,2), 'c.');
%plot(W_1(:, 1), W_1(:, 2), 'b.') 
%plot(W_2(:, 1), W_2(:, 2), 'r.') 
%plot(W1_fd_left(:, 1), W1_fd_left(:, 2), 'k*')
plot(W2_fd_right(:,1), W2_fd_right(:,2), 'r.')
plot(W1_fd_right(:, 1), W1_fd_right(:, 2), 'g*')
%plot(fW1_fd_right(:, 1), fW1_fd_right(:, 2), 'm.')
plot(f3W1_fd_right(:, 1), f3W1_fd_right(:, 2), 'm.')
%plot(W_3(:, 1), W_3(:, 2), 'g.') 
%plot(W_4(:, 1), W_4(:, 2), 'm.') 
plot(x_2.mid, y_2.mid, 'k*');
plot(x_1.mid, y_1.mid, 'ko');  
axis([-1.5 1.5 -0.8 0.8])


                          
                             
                             
                             
                             