clear 
format long



a = intval('1.4')
b = intval('0.3') %Tangency maybe around 0.7?


%Parameterization Controls:
M = 2
N = 30

%Parameter Domains:
tau = intval('0.12');
domRad0 = intval('0.2');


scale_eigs = intval(1)


%Fixed Point (floating point formula)

x_1 = ((b-1)/(2*a)) + (sqrt((1-b)^2 +4*a))/(2*a);
y_1 = b*x_1;

x_2 = ((b-1)/(2*a)) - (sqrt((1-b)^2 +4*a))/(2*a);
y_2 = b*x_2;



f1_error = henonMap(x_1, y_1, a, b) - [x_1;y_1];
f2_error = henonMap(x_2, y_2, a, b) - [x_2;y_2];


%Fixed Point Parameter Expansion:==========================================

[x_M1, y_M1] = henon_FP_expansion_intval(x_1, y_1, a, b, M);
[x_M2, y_M2] = henon_FP_expansion_intval(x_2, y_2, a, b, M);




%==========================================================================
%Compute Eigenvalue and Eigenvector Series:
%==========================================================================

[lambda1, lambda2, xi1, xi2, ...
 lambda_series1, lambda_series2, ...
 xi_1_series1, xi_2_series1, xi_1_series2, xi_2_series2, ...
 delta_lambda1, delta_lambda2, delta_xi1, delta_xi2] = ...
        henon_linearSeriesAtFP_intval(...
    M, tau, scale_eigs, x_M1, y_M1, a, b);




[lambda3, lambda4, xi3, xi4, ...
 lambda_series3, lambda_series4, ...
 xi_1_series3, xi_2_series3, xi_1_series4, xi_2_series4, ...
 delta_lambda3, delta_lambda4, delta_xi3, delta_xi4] = ...
        henon_linearSeriesAtFP_intval(...
    M, tau, scale_eigs, x_M2, y_M2, a, b);




%==========================================================================
%Compute 1D manifold coefficients at omega = 0:
%==========================================================================
                                                 


[PN1, PN2] = henon_1D_manifold_intval([x_1, y_1], lambda1, xi1, ...
                                 a, b, N, intval(1));



[QN1, QN2] = henon_1D_manifold_intval([x_1, y_1], lambda2, xi2, ...
                                 a, b, N, intval(1));
                             
                                                      

 
[RN1, RN2] = henon_1D_manifold_intval([x_2, y_2], lambda3, xi3, ...
                                 a, b, N, intval(1));



[SN1, SN2] = henon_1D_manifold_intval([x_2, y_2], lambda4, xi4, ...
                                 a, b, N, intval(1));
                
                                                        

                             
                             
nuRad1 = 1.75;
numPoints1 = 300;
nuDom1 = linspace(-nuRad1, nuRad1, numPoints1);               
W_1 = zeros(numPoints1, 2);
for m = 1:numPoints1
    W_1(m, 1:2) = [horner(nuDom1(m), PN1.mid, M),
                   horner(nuDom1(m), PN2.mid, M)];      
end                             
                             
                             
nuRad2 = 2.75;
numPoints2 = 1000;
nuDom2 = linspace(-nuRad2, nuRad2, numPoints2);               
W_2 = zeros(numPoints2, 2);
for m = 1:numPoints2
    W_2(m, 1:2) = [horner(nuDom2(m), QN1.mid, M),
                   horner(nuDom2(m), QN2.mid, M)];      
end





                           
nuRad3 = 2.5;
numPoints3 = 300;
nuDom3 = linspace(-nuRad3, nuRad3, numPoints3);               
W_3 = zeros(numPoints3, 2);
for m = 1:numPoints3
    W_3(m, 1:2) = [horner(nuDom3(m), RN1.mid, M),
                   horner(nuDom3(m), RN2.mid, M)];      
end

       

nuRad4 = 0.8;
numPoints4 = 300;
nuDom4 = linspace(-nuRad4, nuRad4, numPoints4);               
W_4 = zeros(numPoints4, 2);
for m = 1:numPoints4
    W_4(m, 1:2) = [horner(nuDom4(m), SN1.mid, M),
                   horner(nuDom4(m), SN2.mid, M)];      
end


orbitPoints = 50000
orbit0 = zeros(orbitPoints, 2);

parm = b.mid;


p = henonMap(1,1,a.mid, parm);


for m = 1:500
    p = henonMap(p(1), p(2), a.mid, parm);
    
end

for m = 1:orbitPoints
    orbit0(m, 1:2) = p';
    p = henonMap(p(1), p(2), a.mid, parm);
end




figure 
hold on
plot(orbit0(:,1), orbit0(:,2), 'c.');
plot(W_1(:, 1), W_1(:, 2), 'b.') 
plot(W_2(:, 1), W_2(:, 2), 'r.') 
plot(W_3(:, 1), W_3(:, 2), 'g.') 
plot(W_4(:, 1), W_4(:, 2), 'm.') 
plot(x_2.mid, y_2.mid, 'k*');
plot(x_1.mid, y_1.mid, 'ko');               









