clear 
format long

%Lorenz Parameters:
sigma = intval('10')                          
beta = intval('8')/intval('3')
rho = intval('13.9265')
%rho = intval('28')

%Parameter Domains:
%tau = intval(0.5)
tau = intval('4.0');


%domRad0 = intval(0.75)
domRad0 = intval('3.0'); 

%Parameterization Controls:
M = 10
N = 15

scale_eigs = intval(1)

p0 = [intval(0);
      intval(0);
      intval(0)];
 
 
  
  
 tic 
  
  
[lambda1, lambda3, lambda2, xi1, xi3, xi2, ...
 lambda_series1, lambda_series3, lambda_series2, ...
 xi_1_series1, xi_2_series1, xi_3_series1, ...
 xi_1_series3, xi_2_series3, xi_3_series3, ...
 xi_1_series2, xi_2_series2, xi_3_series2, ...
 delta_lambda1, delta_lambda3, delta_lambda2, ...
 delta_xi1, delta_xi3, delta_xi2] = ...
        lorenz_linearSeriesAtOrigin_intval(M, tau, scale_eigs, ...
                                           sigma, beta, rho);


                                       

%==========================================================================
%Compute 2D manifold coefficients at omega = 0:
%==========================================================================

[QN1, QN2, QN3] = ...
    lorenz_2D_manifold_intval_vectorized(...
                              p0, lambda1, lambda2, xi1, xi2, ...
                              sigma, rho, beta, N, intval(1));

  


%==========================================================================
%Compute rho partial of 2D PN at omega = 0 (i.e. first order term in omega).
%==========================================================================


[QN1_rho, QN2_rho, QN3_rho] = ...
    lorenz_2D_rhoPartial_at0_intval_vectorized(...
                                    p0, lambda1, lambda2, ...
                                    xi1, xi2, QN1, QN2, QN3, ...
                                    sigma, rho, beta, N, 1);

                                
                               
                                
    

%==========================================================================
%Compute 2D manifold parameter expansion in omega:
%==========================================================================

tic

[Q_MN1, Q_MN2, Q_MN3] = ...
lorenz_2D_stableBranch_at0_intval_vectorized(...
    QN1, QN2, QN3, QN1_rho, QN2_rho, QN3_rho, ...
    lambda_series1, lambda_series2, ...
    xi_1_series1, xi_2_series1, xi_3_series1, ...
    xi_1_series2, xi_2_series2, xi_3_series2, ...
    M, N, sigma, beta, rho);


time_2D_manifold_new = toc


    
%========================================================================
%Omega Expansion 2D Residual Error:
%========================================================================
 
E_MN_residual_2Dbranch = ...
     residual_stableBranch_at0_intval(Q_MN1, Q_MN2, Q_MN3, ...
     lambda_series1, lambda_series2, sigma, beta, rho, domRad0, tau, M, N);
 
 
E_MN_residual_2Dbranch




%==========================================================================
%Compute 1D manifold coefficients at omega = 0:
%==========================================================================



[PN1, PN2, PN3] = lorenz_1D_manifold_intval(p0, lambda3, xi3, ...
                                 sigma, rho, beta, N, intval(1));


%==========================================================================
%Compute 1D manifold parameter expansion in omega:
%==========================================================================



[PN1_rho, PN2_rho, PN3_rho] = lorenz_1D_rhoPartial_at0_intval(...
                                      p0, lambda3, xi3, ...
                                      PN1, PN2, PN3, ...
                                      sigma, rho, beta, ... 
                                      N, intval(1));




[P_MN1, P_MN2, P_MN3] = lorenz_1D_unstableBranch_at0_intval(...
    PN1, PN2, PN3, PN1_rho, PN2_rho, PN3_rho, lambda_series3,...
    xi_1_series3, xi_2_series3, xi_3_series3, ...
    M, N, sigma, beta, rho);





%==========================================================================
%==========================================================================
% Plotting
%==========================================================================
%==========================================================================

[Q1, Q2, Q3] = convertToPoly_3D(Q_MN1, Q_MN2, Q_MN3, M, N);
[P1, P2, P3] = convertToPoly_branch_2D(P_MN1, P_MN2, P_MN3, M, N);

anglePoints = linspace(0, 2*pi, 100);
parmPoints = linspace(-mid(tau), mid(tau), 25);

cylinderPoints = zeros(25, 100, 3);
linePoints = linspace(-mid(domRad0), mid(domRad0), 100);
manBd = zeros(25, 100, 3);
manifoldBranch = zeros(25, 100, 3);
r = mid(domRad0)
for n = 1:100
    for k = 1:25
        thisParm = parmPoints(k);
        this_x = r*cos(anglePoints(n));
        this_y = r*sin(anglePoints(n));
        thisTau = linePoints(n);
        
        cylinderPoints(k, n, :) = ...
            [parmPoints(k), r*cos(anglePoints(n)), r*sin(anglePoints(n))];
        x1 = polyval(mid(Q1), thisParm, this_x, this_y);
        y1 =  polyval(mid(Q2), thisParm, this_x, this_y);
        z1 =  polyval(mid(Q3), thisParm, this_x, this_y);
        
        x2 = polyval(mid(P1), thisParm, thisTau);
        y2 = polyval(mid(P2), thisParm, thisTau);
        z2 = polyval(mid(P3), thisParm, thisTau);
        
        manBd(k, n, :) = [x1, y1, z1];
        manifoldBranch(k, n, :) = [x2, y2, z2];
    end
end

figure 
hold on

for n = 1:100
plot3(manBd(:, n, 1), ...
      manBd(:, n, 2), ...
      manBd(:, n, 3), ...
      'r.')
 
plot3(manifoldBranch(:, n, 1), ...
      manifoldBranch(:, n, 2), ...
      manifoldBranch(:, n, 3), ...
      'b.')  
  
end
axis([-5 5 -5 5 -5 5])


         
%--------- orbit test----------------------------------------------

numSteps = 1000
tf =  2
tspan = linspace(0, tf, numSteps);
y0 = manBd(1, 100, :);           %initial conditions


options=odeset('RelTol',1e-12,'AbsTol',1e-8);
[t,y] = ode113('lorenzField', tspan, y0, ...
    options, flag, mid(sigma), mid(rho), mid(beta));


plot3(y(:, 1), y(:,2), y(:, 3), 'k')





