clear 
format long


%Lorenz Parameters:
sigma = intval('10')                          
beta = intval('8')/intval('3')
rho = intval('13.9265')
%rho = intval('28')




%Parameter Domains:
tau = intval('0.75');
domRad0 = intval('0.9'); 


%Parameterization Controls:
M = 2
N = 25


scale_eigs = intval(1)


p0 = [intval(0);
      intval(0);
      intval(0)];

tic
  
[lambda1, lambda3, lambda2, xi1, xi3, xi2, ...
 lambda_series1, lambda_series3, lambda_series2, ...
 xi_1_series1, xi_2_series1, xi_3_series1, ...
 xi_1_series3, xi_2_series3, xi_3_series3, ...
 xi_1_series2, xi_2_series2, xi_3_series2, ...
 delta_lambda1, delta_lambda3, delta_lambda2, ...
 delta_xi1, delta_xi3, delta_xi2] = ...
        lorenz_linearSeriesAtOrigin_intval(M, tau, scale_eigs, ...
                                           sigma, beta, rho);


Q = [xi1, xi3, xi2];
Q_inv = inv(Q);


%Near-resonances at omega = 0:
worstResIndices = [0, 0];
worstRes = 1000000000;

for n1 = 0:N
    for n2 = 0:N
        if ((n1 + n2 >= 2) & (n1 + n2 <= N))
            n1;
            n2;
          resTest1 = abs(lambda1 - (n1*lambda1 + n2*lambda2));
          resTest3 = abs(lambda2 - (n1*lambda1 + n2*lambda2));
          resMultiIndices = [n1, n2];
          resVect = [inf(resTest1), inf(resTest3)]; 
          resTest = min(resVect);
          if resTest < worstRes
              worstResIndices = resMultiIndices;
              worstRes = resTest;
          end
        end
    end
end

worstResIndices
worstRes
            
%==========================================================================
%Build the Q_omega matrix series:             
%==========================================================================
Q_0 = [xi_1_series1(1), xi_1_series2(1), xi_1_series3(1);
       xi_2_series1(1), xi_2_series2(1), xi_2_series3(1);
       xi_3_series1(1), xi_3_series2(1), xi_3_series3(1)]



delta_Q = (delta_xi1 + delta_xi2)
Q_omega = intval(zeros(3, 3,M+1));
Q_omega(1:3, 1:3, 1) = Q_0;

for m = 1:M
    Q_m = [xi_1_series1(m+1), xi_1_series2(m+1), xi_1_series3(m+1);
           xi_2_series1(m+1), xi_2_series2(m+1), xi_2_series3(m+1);
           xi_3_series1(m+1), xi_3_series2(m+1), xi_3_series3(m+1)];
    Q_omega(1:3, 1:3, m+1) = Q_m;
end


%==========================================================================
%Invert and validate the Q series and compute matrix norms:
%==========================================================================

Q_omegaInv = matrixSeries_formalInverse_intval(Q_omega, M);
delta_Qinv = matrixSeries_validateInverse_intval(...
                   Q_omega, Q_omegaInv, delta_Q, tau, M)



Q_omega_sigmaNorm = 0;
Q_omegaInv_sigmaNorm = 0;

for m = 0:M
    Q_omega_sigmaNorm = Q_omega_sigmaNorm + ...
                        norm(Q_omega(1:3, 1:3, m+1), inf)*tau^m;
    Q_omegaInv_sigmaNorm = Q_omegaInv_sigmaNorm + ...
                        norm(Q_omegaInv(1:3, 1:3, m+1), inf)*tau^m;;
end


%==========================================================================
%Compute 2D manifold coefficients at omega = 0:
%==========================================================================


tic
[QN1, QN2, QN3] = ...
    lorenz_2D_manifold_intval(p0, lambda1, lambda2, xi1, xi2, ...
                              sigma, rho, beta, N, intval(1));

time_old = toc 
                          
tic                          
[QN1_v, QN2_v, QN3_v] = ...                        
  lorenz_2D_manifold_intval_vectorized(...
                                    p0, lambda1, lambda2, xi1, xi2, ...
                              sigma, rho, beta, N, intval(1));
                          
                          
 time_new = toc                         
                          
 time_spedup = time_old/time_new                       
                          