clear 
format long


%Lorenz Parameters:
sigma = intval('10')                          
beta = intval('8')/intval('3')
rho = intval('13.9265')
%rho = intval('28')



%Parameter Domains:
tau = intval(0.2);
domRad0 = intval(1.0); 


%Parameterization Controls:
M = 15
N = 15

eig_num = 1
scale_eig = 1

%Fixed point at origin:
p0 = [intval(0);
      intval(0);
      intval(0)];
   
  
Dfp0 = [    -sigma,         sigma,     intval(0);
               rho,    intval(-1),     intval(0);
         intval(0),     intval(0),          -beta]

            
[U_temp, Sigma_temp] = eigs(Dfp0.mid);

Sigma_temp

%Verify eigen data plus:
[lambda1, xi1] = ...
     verifyeig(Dfp0, Sigma_temp(1, 1), U_temp(:,1));

 xi1 = scale_eig*xi1
 
 [lambda2, xi2] = ...
     verifyeig(Dfp0, Sigma_temp(2, 2), U_temp(:,2));

 xi2 = scale_eig*xi2
 
 [lambda3, xi3] = ...
     verifyeig(Dfp0, Sigma_temp(3, 3), U_temp(:,3));

 xi3 = scale_eig*xi3
 
 
[lambda_series1, xi_1_series1, xi_2_series1, ...
  delta_lambda1, delta_xi1] = ...
    lorenz_eigValVectValidate_intval(lambda1, xi1, M, tau, scale_eig, ...
                                       sigma, beta, rho);
 
                                   
 [lambda_series2, xi_1_series2, xi_2_series2, ...
  delta_lambda2, delta_xi2] = ...
    lorenz_eigValVectValidate_intval(lambda2, xi2, M, tau, scale_eig, ...
                                       sigma, beta, rho);
 
 
%Eigenvalue and eigenvection 3 have trivial series expansions:                                   

lambda_series3 = intval(zeros(1, M+1));
lambda_series3(1) = lambda3;                                  
  
xi_3_series1 = intval(zeros(1, M+1));
xi_3_series2 = intval(zeros(1, M+1));

xi_1_series3 = intval(zeros(1, M+1));
xi_2_series3 = intval(zeros(1, M+1));
xi_3_series3 = intval(zeros(1, M+1));
       
xi_3_series3(1) = xi3(3); 


Sigma_0 = [lambda1,       0,       0;
                 0, lambda2,       0;
                 0,       0, lambda3]


%There is no truncation error in lambda3 and xi_3.
delta_lambda3 = 0;             
delta_xi3 = 0;
             
%==========================================================================
%Build the Q_omega matrix series:             
%==========================================================================
Q_0 = [xi_1_series1(1), xi_1_series2(1), xi_1_series3(1);
       xi_2_series1(1), xi_2_series2(1), xi_2_series3(1);
       xi_3_series1(1), xi_3_series2(1), xi_3_series3(1)]



delta_Q = (delta_xi1 + delta_xi2)
Q_omega = intval(zeros(3, 3,M+1));
Q_omega(1:3, 1:3, 1) = Q_0;

for m = 1:M
    Q_m = [xi_1_series1(m+1), xi_1_series2(m+1), xi_1_series3(m+1);
           xi_2_series1(m+1), xi_2_series2(m+1), xi_2_series3(m+1);
           xi_3_series1(m+1), xi_3_series2(m+1), xi_3_series3(m+1)];
    Q_omega(1:3, 1:3, m+1) = Q_m;
end


%==========================================================================
%Invert and validate the Q series:
%==========================================================================

Q_omegaInv = matrixSeries_formalInverse_intval(Q_omega, M);
delta_Qinv = matrixSeries_validateInverse_intval(...
                   Q_omega, Q_omegaInv, delta_Q, tau, M)


               
               
%==========================================================================
%Resonance Computations: at omega = 0.
%==========================================================================



worstRes = 10000;
worstResIndices = [0, 0];


for n1 = 0:N
    for n2 = 0:N
        if ((n1 + n2 >= 2) & (n1 + n2 <= N))
            n1 ;
            n2;
          resTest1 = abs(lambda1 - (n1*lambda1 + n2*lambda3));
          resTest3 = abs(lambda3 - (n1*lambda1 + n2*lambda3));
          resMultiIndices = [n1, n2];
          resVect = [inf(resTest1), inf(resTest3)]; 
          resTest = min(resVect);
          if resTest < worstRes
              worstResIndices = resMultiIndices;
              worstRes = resTest;
          end
        end
    end
end

worstResIndices
worstRes

              
%==========================================================================
%Resonance Computations: for |omega| <= tau. 
%==========================================================================


resonanceBounds=compute_resonanceBounds(...
    lambda_series1, lambda_series2, lambda_series3, ...
    delta_lambda1, delta_lambda2, delta_lambda3, tau, N, M)


stopNow = stopHere


theResBound = 0;
for m = 1:M
  theResBound = ...
     theResBound + abs(lambda_series1(m+1) - 7*lambda_series3(m+1))*tau^m;
end


theResBound

boundFromZero = worstRes - theResBound - 7*delta_lambda1



resonanceBounds = intval(ones(N+1, N+1));

for n1 = 0:N
    n1
    N
    for n2 = 0:N
        if ((n1 + n2 >= 2) & (n1 + n2 <= N))
          
            theResBound1 = 0;
            for m = 1:M
                theResBound1 = theResBound1 ...
                  + abs(lambda_series1(m+1) - ...
                  (n1*lambda_series1(m+1) + n2*lambda_series3(m+1)))*tau^m;
            end
            theResBound2 = 0;
            for m = 1:M
                theResBound2 = theResBound2 ...
                  + abs(lambda_series3(m+1) - ...
                  (n1*lambda_series1(m+1) + n2*lambda_series3(m+1)))*tau^m;
            end
            theResBound3 = 0;
            for m = 1:M
                theResBound3 = theResBound3 ...
                  + abs(lambda_series2(m+1) - ...
                  (n1*lambda_series1(m+1) + n2*lambda_series3(m+1)))*tau^m;
            end
            res1 = abs(lambda1 - (n1*lambda1 + n2*lambda3));
            res2 = abs(lambda3 - (n1*lambda1 + n2*lambda3));
            res3 = abs(lambda2 - (n1*lambda1 + n2*lambda3));
            boundFromZero1 = res1 - theResBound1 - (n1+n2)*delta_lambda1;
            boundFromZero2 = res2 - ...
                      theResBound2 - (n1+n2)*(delta_lambda1+delta_lambda2);
            boundFromZero3 = res3 - theResBound3 - (n1+n2)*delta_lambda1;
            
            if ((inf(boundFromZero1) < inf(boundFromZero2)) & ...
                    (inf(boundFromZero1) < inf(boundFromZero3)))
                theBound = 1/boundFromZero1;
            elseif ((inf(boundFromZero2) < inf(boundFromZero1)) & ...
                    (inf(boundFromZero2) < inf(boundFromZero3)))
                theBound = 1/boundFromZero2;
            elseif ((inf(boundFromZero3) < inf(boundFromZero1)) & ...
                    (inf(boundFromZero3) < inf(boundFromZero2)))
                theBound = 1/boundFromZero3;
            end        
            
            
            resonanceBounds(n1+1, n2+1) = theBound;
        end
    end
end







