%% a_paperCodeEx3 demo henon eigenvalue powers (stable) Taylor models
%
%%

clear 
format long

%================ User Parameters/Controls ================================


a = intval('1.4')
b = intval('0.3')

%Parameterization Controls:
M = 30


%Parameter Domains:
tau = intval('0.23');   %0.23 works

%=======Fixed Point (floating point formula)===============================

%First Fixed Point:
x_1 = ((b-1)/(2*a)) + (sqrt((1-b)^2 +4*a))/(2*a);
y_1 = b*x_1;

%Second Fixed Point:
x_2 = ((b-1)/(2*a)) - (sqrt((1-b)^2 +4*a))/(2*a);
y_2 = b*x_2;

f1_error = henonMap(x_1, y_1, a, b) - [x_1;y_1]
f2_error = henonMap(x_2, y_2, a, b) - [x_2;y_2]


%Fixed Point Parameter Expansions:==========================================

[x_M1, y_M1] = henon_FP_expansion_intval(x_1, y_1, a, b, M);
delta_pM1 = henon_FP_validation(x_M1, y_M1, tau, a, b, M);

[x_M2, y_M2] = henon_FP_expansion_intval(x_2, y_2, a, b, M);
delta_pM2 = henon_FP_validation(x_M2, y_M2, tau, a, b, M);

sup(delta_pM1)
sup(delta_pM2)


%==========================================================================
%Compute Eigenvalue and Eigenvector Series:
%==========================================================================

scale_eigs = 1

%Domain on which the differntial of the eigenvector and eigenvalue
%       operators can be inverted for fixed point branch one.
tauBar1 = 0.13

tic

[lambda1, lambda2, xi1, xi2, ...
 lambda_series1, lambda_series2, ...
 xi_1_series1, xi_2_series1, xi_1_series2, xi_2_series2, ...
 delta_lambda1, delta_lambda2, delta_xi1, delta_xi2] = ...
        henon_linearSeriesAtFP_intval(...
    M, tauBar1, scale_eigs, x_M1, y_M1, delta_pM1, a, b);


%Domain on which the differntial of the eigenvector and eigenvalue
%       operators can be inverted for fixed point branch two.
tauBar2 = 0.14


[lambda3, lambda4, xi3, xi4, ...
 lambda_series3, lambda_series4, ...
 xi_1_series3, xi_2_series3, xi_1_series4, xi_2_series4, ...
 delta_lambda3, delta_lambda4, delta_xi3, delta_xi4] = ...
        henon_linearSeriesAtFP_intval(...
    M, tauBar2, scale_eigs, x_M2, y_M2, delta_pM2, a, b);


time = toc


sup(delta_lambda1)
sup(delta_lambda2)
sup(delta_xi1)
sup(delta_xi2)


sup(delta_lambda3)
sup(delta_lambda4)
sup(delta_xi3)
sup(delta_xi4)


%==========================================================================
% Powers of the eigenvalues.
%==========================================================================


N = 15

tic
Lambda1_powers = compute_seriesPowers_intval(lambda_series1, N, M);
powers_formalTime = toc

sigma = 0.08;
delta_powers = validate_LambdaPowers_intval(...
                       lambda_series1, tauBar1, delta_lambda1, ...
                       Lambda1_powers, sigma, N, M)



tauBar1_prime = tauBar1*exp(-sigma)












