%% a_paperCodeEx4 demo of powers of analytic taylor models.
%
%%

clear 
format long

%================ User Parameters/Controls ================================

%Henon Parameters:
a = intval('1.4')
b = intval('0.3')



%Parameterization Controls:
M = 10   %Order of Taylor Models
N = 20   %Number of Powers to compute.


%Taylor Model Domains:
tau = intval('0.1');   %0.23 work
tauBar1 = tau;
%Codes will crash if any part of the computation
%fails on the tau domain.  If codes succede then tau
%is the domain for all Taylor models in the program.
%
%tauBar1 is the domain for the eigenvalues and eigenvectors
%tau is the domain for the fixed points
%these can be adjusted independantly if desired.

%Normalization of the eigenvectors.
scale_eigs = intval(1)

%Loss of Domain Parameter. Must 0 < sigma <= 1.  
sigma = intval(0.005);

%=======Fixed Points of Henon)===============================


%First Fixed Point:
x_1 = ((b-1)/(2*a)) + (sqrt((1-b)^2 +4*a))/(2*a);
y_1 = b*x_1;

%Second Fixed Point:
x_2 = ((b-1)/(2*a)) - (sqrt((1-b)^2 +4*a))/(2*a);
y_2 = b*x_2;


%Fixed Point Taylor Models:==========================================

%Fixed Point One.
[x_M1, y_M1] = henon_FP_expansion_intval(x_1, y_1, a, b, M);
delta_pM1 = henon_FP_validation(x_M1, y_M1, tau, a, b, M);

sup(delta_pM1)



%==========================================================================
%Compute Eigenvalue and Eigenvector Taylor Models:
%==========================================================================

      
tic

[lambda1, lambda2, xi1, xi2, ...
 lambda_series1, lambda_series2, ...
 xi_1_series1, xi_2_series1, ...
 xi_1_series2, xi_2_series2, ...
 delta_lambda1, delta_lambda2, ...
 delta_xi1, delta_xi2] = ...
        henon_linearSeriesAtFP_intval(...
                M, tauBar1, scale_eigs, ...
                x_M1, y_M1, delta_pM1, ...
                a, b);




sup(delta_lambda1)
sup(delta_lambda2)
sup(delta_xi1)
sup(delta_xi2)



%==========================================================================
%Compute Powers of an eigenvalue:
%==========================================================================

%Stable Eigenvalue Powers:
tic

Lambda2_powers = compute_seriesPowers_intval(lambda_series2, N, M);


delta_powers = validate_LambdaPowers_intval(...
                       lambda_series2, tauBar1, delta_lambda2, ...
                       Lambda2_powers, sigma, N, M);
             
                   
  
%The following lines of code compute the powers of the unstable
%eigenvalues.  However (as discussed in the paper) this is a 
%bad idea as the leading term is greater than zero.  However
%we have included the code as the results are included in the 
%paper.
                   
%Lambda1_powers = compute_seriesPowers_intval(lambda_series1, N, M);
%
%Loss of Domain Parameter. Must 0 < sigma <= 1.  
%sigma = intval(0.005);
%delta1_powers = validate_LambdaPowers_intval(...
%                       lambda_series1, tauBar1, delta_lambda1, ...
%                       Lambda1_powers, sigma, N, M);
             
                   
                         
                   
                   
                   
%========================= Results ========================================                   
                  

%The error in the original eigenvalue series:
delta_lambda2

%The power errors:
delta_powers'


%Reduced Domain on which the powers of the 
%Taylor models are valid.
tauBar1_prime = tauBar1*exp(-sigma)

%Computation time
powerTime = toc






