clear 
format long



a = intval('1.4')
b = intval('3.0')


%Parameterization Controls:
M = 15
N = 35

%Parameter Domains:
tau = intval('0.25');
tauBar1 = intval('0.25')
domRad0 = intval('3.25');

nuRad1 = 3.25;
nuRad2 = 3.25;
nuRad3 = 3.25;
nuRad4 = 3.25;
tau = 0.25


scale_eigs = intval(1)


%Fixed Point (floating point formula)

x_1 = ((b-1)/(2*a)) + (sqrt((1-b)^2 +4*a))/(2*a);
y_1 = b*x_1;

x_2 = ((b-1)/(2*a)) - (sqrt((1-b)^2 +4*a))/(2*a);
y_2 = b*x_2;



f1_error = henonMap(x_1, y_1, a, b) - [x_1;y_1];
f2_error = henonMap(x_2, y_2, a, b) - [x_2;y_2];


%Fixed Point Parameter Expansion:==========================================



[x_M1, y_M1] = henon_FP_expansion_intval(x_1, y_1, a, b, M);
delta_pM1 = henon_FP_validation_hack(x_M1, y_M1, tau, a, b, M);

[x_M2, y_M2] = henon_FP_expansion_intval(x_2, y_2, a, b, M);
delta_pM2 = henon_FP_validation_hack(x_M2, y_M2, tau, a, b, M);

sup(delta_pM1)
sup(delta_pM2)





%==========================================================================
%Compute Eigenvalue and Eigenvector Series:
%==========================================================================




scale_eigs = 1

%Domain on which the differntial of the eigenvector and eigenvalue
%       operators can be inverted for fixed point branch one.


tic

[lambda1, lambda2, xi1, xi2, ...
 lambda_series1, lambda_series2, ...
 xi_1_series1, xi_2_series1, xi_1_series2, xi_2_series2, ...
 delta_lambda1, delta_lambda2, delta_xi1, delta_xi2] = ...
        henon_linearSeriesAtFP_intval_hack(...
    M, tauBar1, scale_eigs, x_M1, y_M1, delta_pM1, a, b);


%Domain on which the differntial of the eigenvector and eigenvalue
%       operators can be inverted for fixed point branch two.
tauBar2 = 0.14
%tauBar2 = 0.001

[lambda3, lambda4, xi3, xi4, ...
 lambda_series3, lambda_series4, ...
 xi_1_series3, xi_2_series3, xi_1_series4, xi_2_series4, ...
 delta_lambda3, delta_lambda4, delta_xi3, delta_xi4] = ...
        henon_linearSeriesAtFP_intval_hack(...
    M, tauBar2, scale_eigs, x_M2, y_M2, delta_pM2, a, b);


time = toc


sup(delta_lambda1)
sup(delta_lambda2)
sup(delta_xi1)
sup(delta_xi2)


sup(delta_lambda3)
sup(delta_lambda4)
sup(delta_xi3)
sup(delta_xi4)






%==========================================================================
%Q and Q^-1 matrix series computations:
%==========================================================================


Q = [xi1, xi2];
Q_inv = inv(Q);

            
%==========================================================================
%Build the Q_omega matrix series:             
%==========================================================================
Q_0 = [xi_1_series1(1), xi_1_series2(1);
       xi_2_series1(1), xi_2_series2(1)]



%delta_Q = (delta_xi1 + delta_xi2)
delta_Q = 10^(-10)
Q_omega = intval(zeros(2, 2, M+1));
Q_omega(1:2, 1:2, 1) = Q_0;

for m = 1:M
    Q_m = [xi_1_series1(m+1), xi_1_series2(m+1);
           xi_2_series1(m+1), xi_2_series2(m+1)];
    Q_omega(1:2, 1:2, m+1) = Q_m;
end


%==========================================================================
%Invert and validate the Q series and compute matrix norms:
%==========================================================================

Q_omegaInv = matrixSeries_formalInverse_intval(Q_omega, M);
delta_Qinv = matrixSeries_validateInverse_intval(...
                   Q_omega, Q_omegaInv, delta_Q, tau, M)



Q_omega_sigmaNorm = 0;
Q_omegaInv_sigmaNorm = 0;

for m = 0:M
    Q_omega_sigmaNorm = Q_omega_sigmaNorm + ...
                        norm(Q_omega(1:2, 1:2, m+1), inf)*tau^m;
    Q_omegaInv_sigmaNorm = Q_omegaInv_sigmaNorm + ...
                        norm(Q_omegaInv(1:2, 1:2, m+1), inf)*tau^m;;
end


%==========================================================================
%Compute 1D manifold coefficients at omega = 0:
%==========================================================================
                                                 

 
                         
                                 
[PN1, PN2] = henon_1D_manifold_intval(...
    [x_1, y_1], lambda1, xi1, a, b, N, intval(1));

[QN1, QN2] = henon_1D_manifold_intval(...
    [x_1, y_1], lambda2, xi2, a, b, N, intval(1));
                              
[RN1, RN2] = henon_1D_manifold_intval(...
    [x_2, y_2], lambda3, xi3, a, b, N, intval(1));

[SN1, SN2] = henon_1D_manifold_intval(...
    [x_2, y_2], lambda4, xi4, a, b, N, intval(1));
                                                                                            
%==========================================================================
%Compute rho partial of 1D PN at omega = 0 (i.e. first order term in omega).
%==========================================================================

[PN1_rho, PN2_rho] = ...
    henon_rhoPartial_intval([x_1, y_1], lambda1, xi1, ...
                            PN1, PN2, a, b, N);     
[QN1_rho, QN2_rho] = ...
    henon_rhoPartial_intval([x_1, y_1], lambda2, xi2, ...
                            QN1, QN2, a, b, N);
[RN1_rho, RN2_rho] = ...
    henon_rhoPartial_intval([x_2, y_2], lambda3, xi3, ...
                            RN1, RN2, a, b, N);
     
[SN1_rho, SN2_rho] = ...
    henon_rhoPartial_intval([x_2, y_2], lambda4, xi4, ...
                            SN1, SN2, a, b, N);
                     


%==========================================================================
%Compute 1D manifold parameter expansion in omega:
%==========================================================================

Lambda1_powers = compute_seriesPowers_intval(lambda_series1, N, M);
Lambda2_powers = compute_seriesPowers_intval(lambda_series2, N, M);
Lambda3_powers = compute_seriesPowers_intval(lambda_series3, N, M);
Lambda4_powers = compute_seriesPowers_intval(lambda_series4, N, M);



[P_MN1, P_MN2] = henon_invariantBranch_intval_vectorized(...
                    x_M1, y_M1, ...
                    PN1, PN2, PN1_rho, PN2_rho, ...
                    lambda_series1, Lambda1_powers, ...
                    xi_1_series1, xi_2_series1, M, N, a, b); 

[Q_MN1, Q_MN2] = henon_invariantBranch_intval_vectorized(...
                    x_M1, y_M1, ...
                    QN1, QN2, QN1_rho, QN2_rho, ...
                    lambda_series2, Lambda2_powers, ...
                    xi_1_series2, xi_2_series2, M, N, a, b); 

[R_MN1, R_MN2] = henon_invariantBranch_intval_vectorized(...
                    x_M2, y_M2, ...
                    RN1, RN2, RN1_rho, RN2_rho, ...
                    lambda_series3, Lambda3_powers, ...
                    xi_1_series3, xi_2_series3, M, N, a, b); 


[S_MN1, S_MN2] = henon_invariantBranch_intval_vectorized(...
                    x_M2, y_M2, ...
                    SN1, SN2, SN1_rho, SN2_rho, ...
                    lambda_series4, Lambda4_powers, ...
                    xi_1_series4, xi_2_series4, M, N, a, b); 


 [Pmn1, Pmn2] = convertToPoly_2Dbranch_2Dspace(P_MN1, P_MN2, M, N);
 [Qmn1, Qmn2] = convertToPoly_2Dbranch_2Dspace(Q_MN1, Q_MN2, M, N);
 [Rmn1, Rmn2] = convertToPoly_2Dbranch_2Dspace(R_MN1, R_MN2, M, N);
 [Smn1, Smn2] = convertToPoly_2Dbranch_2Dspace(S_MN1, S_MN2, M, N);               
                
               
%==========================================================================
%Plotting:
%==========================================================================
     

scaleParm = 8;

 num_x = 60
 num_y = 35
 numParm = num_x*num_y
 parmDomain1 = zeros(numParm, 2);
 parmDomain2 = zeros(numParm, 2);
 parmDomain3 = zeros(numParm, 2);
 parmDomain4 = zeros(numParm, 2);
 


 
 
 
 %theXs1 = linspace(0.95, 3.25, num_x);
 %theXs2 = linspace(0.95, 3.25, num_x);
 %theXs3 = linspace(0.95, 3.25, num_x);
 %theXs4 = linspace(0.95, 3.25, num_x);
 
 
 %theXs1 = linspace(-nuRad1, nuRad1, num_x);
 %theXs2 = linspace(-nuRad2/8, nuRad2, num_x);
 %theXs3 = linspace(-nuRad3/4, nuRad3, num_x);
 %theXs4 = linspace(-nuRad4/2, nuRad4/2, num_x);
 
 theXs1 = linspace(-nuRad1, nuRad1, num_x);
 theXs2 = linspace(-nuRad2, nuRad2, num_x);
 theXs3 = linspace(-nuRad3, nuRad3, num_x);
 theXs4 = linspace(-nuRad4, nuRad4, num_x);
 
 theYs = linspace(-tau, tau, num_y);
 
 
 pointNum = 0;
 for n = 1:num_y
     for m = 1:num_x
        pointNum = pointNum + 1;
        parmDomain1(pointNum, 1:2) = [theXs1(m), theYs(n)];
        parmDomain2(pointNum, 1:2) = [theXs2(m), theYs(n)];
        parmDomain3(pointNum, 1:2) = [theXs3(m), theYs(n)];
        parmDomain4(pointNum, 1:2) = [theXs4(m), theYs(n)];
     end
 end

branchManifold1 = zeros(numParm, 3);
branchManifold2 = zeros(numParm, 3);
branchManifold3 = zeros(numParm, 3);
branchManifold4 = zeros(numParm, 3);


pointNum = 0;
for n = 1:num_y
    for m = 1:num_x
        pointNum = pointNum + 1;
        thisParameter = parmDomain1(pointNum, 1:2);
        thisPoint1 = polyval(Pmn1, thisParameter(2), thisParameter(1));
        thisPoint2 = polyval(Pmn2, thisParameter(2), thisParameter(1));
        branchManifold1(pointNum, 1:3) = ...
            [thisPoint1.mid, thisPoint2.mid, scaleParm*theYs(n)];
        
        thisParameter = parmDomain2(pointNum, 1:2);
        thisPoint1 = polyval(Qmn1, thisParameter(2), thisParameter(1));
        thisPoint2 = polyval(Qmn2, thisParameter(2), thisParameter(1));
        branchManifold2(pointNum, 1:3) = ...
            [thisPoint1.mid, thisPoint2.mid, scaleParm*theYs(n)];
        
        thisParameter = parmDomain3(pointNum, 1:2);
        thisPoint1 = polyval(Rmn1, thisParameter(2), thisParameter(1));
        thisPoint2 = polyval(Rmn2, thisParameter(2), thisParameter(1));
        branchManifold3(pointNum, 1:3) = ...
            [thisPoint1.mid, thisPoint2.mid, scaleParm*theYs(n)];
        
        thisParameter = parmDomain4(pointNum, 1:2);
        thisPoint1 = polyval(Smn1, thisParameter(2), thisParameter(1));
        thisPoint2 = polyval(Smn2, thisParameter(2), thisParameter(1));
        branchManifold4(pointNum, 1:3) = ...
            [thisPoint1.mid, thisPoint2.mid, scaleParm*theYs(n)];
    end
end
 
%figure
%hold on
%plot3(branchManifold(:, 1), branchManifold(:, 2), branchManifold(:, 3), 'b.')

%stopNow = stopHere

tauDom = 0.15;
numPoints = 100;

tauDom = linspace(-tauDom, tauDom, numPoints)               
fp_branch1 = zeros(numPoints, 3);
fp_branch2 = zeros(numPoints, 3);
manifold0_1 = zeros(numPoints, 3);
manifold0_2 = zeros(numPoints, 3);
manifold0_3 = zeros(numPoints, 3);
manifold0_4 = zeros(numPoints, 3);

for m = 1:numPoints
    fp_branch1(m, 1:2) = [horner(tauDom(m), x_M1.mid, M),
                         horner(tauDom(m), y_M1.mid, M)];
    fp_branch1(m, 3) = scaleParm*tauDom(m);                 
    fp_branch2(m, 1:2) = [horner(tauDom(m), x_M2.mid, M),
                         horner(tauDom(m), y_M2.mid, M)];
    fp_branch2(m,3) = scaleParm*tauDom(m);                 
end



 num_x = 600
 
 theXs1 = linspace(-nuRad1, nuRad1, num_x);
 theXs2 = linspace(-nuRad2, nuRad2, num_x);
 theXs3 = linspace(-nuRad3, nuRad3, num_x);
 theXs4 = linspace(-nuRad4, nuRad4, num_x);

singManParm = 0.12;
 
 
for m = 1:num_x
    the_x = polyval(Pmn1, singManParm, theXs1(m));
    the_y = polyval(Pmn2, singManParm, theXs1(m));
    manifold0_1(m, 1) = the_x.mid;
    manifold0_1(m, 2) = the_y.mid;
    manifold0_1(m, 3) = scaleParm*singManParm;                 
                   
end


for m = 1:num_x
    the_x = polyval(Qmn1, singManParm, theXs2(m));
    the_y = polyval(Qmn2, singManParm, theXs2(m));
    manifold0_2(m, 1) = the_x.mid;
    manifold0_2(m, 2) = the_y.mid;
    manifold0_2(m, 3) = scaleParm*singManParm;                 
                   
end


for m = 1:num_x
    the_x = polyval(Rmn1, singManParm, theXs3(m));
    the_y = polyval(Rmn2, singManParm, theXs3(m));
    manifold0_3(m, 1) = the_x.mid;
    manifold0_3(m, 2) = the_y.mid;
    manifold0_3(m, 3) = scaleParm*singManParm;                 
                   
end


for m = 1:num_x
    the_x = polyval(Smn1, singManParm, theXs4(m));
    the_y = polyval(Smn2, singManParm, theXs4(m));
    manifold0_4(m, 1) = the_x.mid;
    manifold0_4(m, 2) = the_y.mid;
    manifold0_4(m, 3) = scaleParm*singManParm;                 
                   
end



orbitPoints = 2
orbit0 = zeros(orbitPoints, 2);
orbit1 = zeros(orbitPoints, 2);
orbit2 = zeros(orbitPoints, 2);
parm = singManParm;
parm1 = 0.4;
parm2 = 0.2;

p = henonMap(1,1,a.mid, parm);
p1 = henonMap(1,1,a.mid, parm1);
p2 = henonMap(1,1,a.mid, parm2);

for m = 1:5
    p = henonMap(p(1), p(2), a.mid, b.mid+parm);
    p1 = henonMap(p1(1), p1(2), a.mid, parm1);
    p2 = henonMap(p2(1), p2(2), a.mid, parm2);
end

for m = 1:orbitPoints
    orbit0(m, 1:2) = p';
    p = henonMap(p(1), p(2), a.mid, b.mid+parm);
    orbit1(m, 1:2) = p1';
    p1 = henonMap(p1(1), p1(2), a.mid, parm1);
    orbit2(m, 1:2) = p2';
    p2 = henonMap(p2(1), p2(2), a.mid, parm2);
end

Orbit = [orbit0(:, 1), orbit0(:,2), ...
         scaleParm*singManParm*ones(orbitPoints, 1)];


     
figure 
hold on
plot3(branchManifold1(:, 1), branchManifold1(:, 2), branchManifold1(:, 3), 'b.')
plot3(branchManifold2(:, 1), branchManifold2(:, 2), branchManifold2(:, 3), 'r.')
plot3(branchManifold3(:, 1), branchManifold3(:, 2), branchManifold3(:, 3), 'g.')
plot3(branchManifold4(:, 1), branchManifold4(:, 2), branchManifold4(:, 3), 'm.')

%plot3(orbit0(:,1), orbit0(:,2), parm*ones(orbitPoints, 1), 'y.');
%plot3(orbit1(:,1), orbit1(:,2), parm1*ones(orbitPoints, 1), 'y.');
%plot3(orbit2(:,1), orbit2(:,2), parm2*ones(orbitPoints, 1), 'y.');
plot3(fp_branch2(:, 1), fp_branch2(:, 2), fp_branch2(:, 3), 'k.') 
plot3(fp_branch1(:, 1), fp_branch1(:, 2), fp_branch1(:, 3), 'k.') 
plot3(x_2.mid, y_2.mid, parm, 'k*');
axis([-2, 2, -2, 2, -2, 2]);                




%Unstable-stable-unstable-stable

figure
hold on
plot3(Orbit(:,1), Orbit(:,2), Orbit(:,3), 'k.');
plot3(manifold0_1(:,1), manifold0_1(:,2), manifold0_1(:,3), 'b.');
plot3(manifold0_2(:,1), manifold0_2(:,2), manifold0_2(:,3), 'r.');
plot3(manifold0_3(:,1), manifold0_3(:,2), manifold0_3(:,3), 'g.');
plot3(manifold0_4(:,1), manifold0_4(:,2), manifold0_4(:,3), 'm.');
axis([-2, 2, -2, 2, -2, 2])


omega = 0.12
fp_1 = [horner(omega, x_M1.mid, M); ...
        horner(omega, y_M1.mid, M)]

the_x = polyval(Pmn1, omega, 3.2);
the_y = polyval(Pmn2, omega, 3.2);    
        
p_1 = [the_x.mid;...
       the_y.mid]

%p_1 = [horner(1, PN1.mid, N); ...
%       horner(1, PN2.mid, N)]

p_2 = henonMap(p_1(1), p_1(2), a.mid, b.mid+omega)
      
k = 14
testOrbit = zeros(k, 1);

for j = 1:k
    p_2 = henonMapInv(p_2(1), p_2(2), a.mid, b.mid+omega)
    error = norm(p_2 - fp_1)
    testOrbit(j, 1) = error;
end

figure 
hold on
plot(linspace(1,k, k), testOrbit(:), 'b.')




