%% a_paperCode_henonBranchProofII fixed point 2. stable manifold.
%
%%

clear 
format long

%================ User Parameters/Controls ================================

a = intval('1.4')
b = intval('0.3')

%Parameterization Controls:
M = 15
N = 35

%Parameter Domains:
tauBar1 = intval('0.14');
tau = intval('0.15');  
nu = intval('0.95');

scale_eigs = intval(1.0)

%=======Fixed Point (floating point formula)===============================

%First Fixed Point:
x_1 = ((b-1)/(2*a)) + (sqrt((1-b)^2 +4*a))/(2*a);
y_1 = b*x_1;

%Second Fixed Point:
x_2 = ((b-1)/(2*a)) - (sqrt((1-b)^2 +4*a))/(2*a);
y_2 = b*x_2;


%Fixed Point Parameter Expansions:==========================================

[x_M2, y_M2] = henon_FP_expansion_intval(x_2, y_2, a, b, M);
delta_pM2 = henon_FP_validation(x_M2, y_M2, tau, a, b, M);


sup(delta_pM2)


%==========================================================================
%Compute Eigenvalue and Eigenvector Series:
%==========================================================================


%Domain on which the differntial of the eigenvector and eigenvalue
%       operators can be inverted for fixed point branch two.
tauBar2 = tauBar1
%tauBar2 = 0.001

[lambda3, lambda4, xi3, xi4, ...
 lambda_series3, lambda_series4, ...
 xi_1_series3, xi_2_series3, xi_1_series4, xi_2_series4, ...
 delta_lambda3, delta_lambda4, delta_xi3, delta_xi4] = ...
        henon_linearSeriesAtFP_intval(...
    M, tauBar2, scale_eigs, x_M2, y_M2, delta_pM2, a, b);



sup(delta_lambda3)
sup(delta_lambda4)
sup(delta_xi3)
sup(delta_xi4)


%==========================================================================
%Q and Q^-1 matrix series computations:
%==========================================================================


Q = [xi3, xi4];
Q_inv = inv(Q);

            
%==========================================================================
%Build the Q_omega matrix series:             
%==========================================================================
Q_0 = [xi_1_series3(1), xi_1_series4(1);
       xi_2_series3(1), xi_2_series4(1)]



delta_Q = (delta_xi3 + delta_xi4)
Q_omega = intval(zeros(2, 2, M+1));
Q_omega(1:2, 1:2, 1) = Q_0;

for m = 1:M
    Q_m = [xi_1_series3(m+1), xi_1_series4(m+1);
           xi_2_series3(m+1), xi_2_series4(m+1)];
    Q_omega(1:2, 1:2, m+1) = Q_m;
end


%==========================================================================
%Invert and validate the Q series and compute matrix norms:
%==========================================================================

Q_omegaInv = matrixSeries_formalInverse_intval(Q_omega, M);
delta_Qinv = matrixSeries_validateInverse_intval(...
                   Q_omega, Q_omegaInv, delta_Q, tau, M)



Q_omega_sigmaNorm = 0;
Q_omegaInv_sigmaNorm = 0;

for m = 0:M
    Q_omega_sigmaNorm = Q_omega_sigmaNorm + ...
                        norm(Q_omega(1:2, 1:2, m+1), inf)*tau^m;
    Q_omegaInv_sigmaNorm = Q_omegaInv_sigmaNorm + ...
                        norm(Q_omegaInv(1:2, 1:2, m+1), inf)*tau^m;;
end




tic

%==========================================================================
%Compute 1D manifold coefficients at omega = 0:
%==========================================================================


[PN1, PN2] = henon_1D_manifold_intval([x_2, y_2], lambda4, xi4, ...
                                 a, b, N, scale_eigs);


                           
                             
%==========================================================================
%Compute rho partial of 1D PN at omega = 0 (i.e. first order term in omega).
%==========================================================================



[PN1_rho, PN2_rho] = ...
    henon_rhoPartial_intval([x_2, y_2], lambda4, xi4, ...
                            PN1, PN2, a, b, N);




%==========================================================================
%Compute 1D manifold parameter expansion in omega:
%==========================================================================

Lambda4_powers = compute_seriesPowers_intval(lambda_series4, N, M);


sigma = intval(0.005);
delta_powers = validate_LambdaPowers_intval(...
                       lambda_series4, tauBar2, delta_lambda4, ...
                       Lambda4_powers, sigma, N, M)



tauBar2_prime = tauBar2*exp(-sigma)



%==========================================================================
% Compute branch manifold coefficients:
%==========================================================================

                
[P_MN1, P_MN2] = henon_invariantBranch_intval_vectorized(...
                    x_M2, y_M2, ...
                    PN1, PN2, PN1_rho, PN2_rho, ...
                    lambda_series4, Lambda4_powers, ...
                    xi_1_series4, xi_2_series4, M, N, a, b); 

                
   
                
%==========================================================================
% Validate the Branch Manifold Series
%==========================================================================

s = intval(0.001)

[delta, delta_h] = validateStableBranch_Henon(P_MN1, P_MN2, ...
    x_M2, y_M2, delta_pM2, lambda_series4, delta_lambda4, ...
    Lambda4_powers, delta_powers, ...
    Q_omega_sigmaNorm, Q_omegaInv_sigmaNorm, delta_xi4, ...
    tauBar2_prime, nu, s, M, N, a, b);
                
  
delta_bound = sup(delta)

tauBar2_prime
proofTime = toc/60





