%% a_paperCode_henonBranchProofII fixed point 1. unstable manifold.
%
%%

clear 
format long

%================ User Parameters/Controls ================================


a = intval('1.4')
b = intval('0.3')

%Parameterization Controls:
M = 15
N = 35

%Parameter Domains:
tauBar1 = intval('0.1325')
tau = intval('0.15');   %0.23 works
nu = intval('0.95');

scale_eigs = 1.0

%=======Fixed Point (floating point formula)===============================

%First Fixed Point:
x_1 = ((b-1)/(2*a)) + (sqrt((1-b)^2 +4*a))/(2*a);
y_1 = b*x_1;



%Fixed Point Parameter Expansions:==========================================

[x_M1, y_M1] = henon_FP_expansion_intval(x_1, y_1, a, b, M);
delta_pM1 = henon_FP_validation(x_M1, y_M1, tau, a, b, M);

sup(delta_pM1)


%==========================================================================
%Compute Eigenvalue and Eigenvector Series:
%==========================================================================

%Domain on which the differntial of the eigenvector and eigenvalue
%       operators can be inverted for fixed point branch one.



[lambda1, lambda2, xi1, xi2, ...
 lambda_series1, lambda_series2, ...
 xi_1_series1, xi_2_series1, xi_1_series2, xi_2_series2, ...
 delta_lambda1, delta_lambda2, delta_xi1, delta_xi2] = ...
        henon_linearSeriesAtFP_intval(...
    M, tauBar1, scale_eigs, x_M1, y_M1, delta_pM1, a, b);



sup(delta_lambda1)
sup(delta_lambda2)
sup(delta_xi1)
sup(delta_xi2)








%==========================================================================
%Q and Q^-1 matrix series computations:
%==========================================================================


Q = [xi1, xi2];
Q_inv = inv(Q);

            
%==========================================================================
%Build the Q_omega matrix series:             
%==========================================================================
Q_0 = [xi_1_series1(1), xi_1_series2(1);
       xi_2_series1(1), xi_2_series2(1)]



delta_Q = (delta_xi1 + delta_xi2)
Q_omega = intval(zeros(2, 2, M+1));
Q_omega(1:2, 1:2, 1) = Q_0;

for m = 1:M
    Q_m = [xi_1_series1(m+1), xi_1_series2(m+1);
           xi_2_series1(m+1), xi_2_series2(m+1)];
    Q_omega(1:2, 1:2, m+1) = Q_m;
end


%==========================================================================
%Invert and validate the Q series and compute matrix norms:
%==========================================================================

Q_omegaInv = matrixSeries_formalInverse_intval(Q_omega, M);
delta_Qinv = matrixSeries_validateInverse_intval(...
                   Q_omega, Q_omegaInv, delta_Q, tau, M)



Q_omega_sigmaNorm = 0;
Q_omegaInv_sigmaNorm = 0;

for m = 0:M
    Q_omega_sigmaNorm = Q_omega_sigmaNorm + ...
                        norm(Q_omega(1:2, 1:2, m+1), inf)*tau^m;
    Q_omegaInv_sigmaNorm = Q_omegaInv_sigmaNorm + ...
                        norm(Q_omegaInv(1:2, 1:2, m+1), inf)*tau^m;;
end



%==========================================================================
%Compute 1D manifold coefficients at omega = 0:
%==========================================================================

tic

[PN1, PN2] = henon_1D_manifold_intval([x_1, y_1], lambda1, xi1, ...
                                 a, b, N, scale_eigs);


                           
                             
%==========================================================================
%Compute rho partial of 1D PN at omega = 0 (i.e. first order term in omega).
%==========================================================================



[PN1_rho, PN2_rho] = ...
    henon_rhoPartial_intval([x_1, y_1], lambda1, xi1, ...
                            PN1, PN2, a, b, N);




%==========================================================================
%Compute 1D manifold parameter expansion in omega:
%==========================================================================

Lambda1_powers = compute_seriesPowers_intval(lambda_series1, N, M);


%Invert the series and validate:
lambda_series1_inv = matrixSeries_formalInverse_intval(lambda_series1, M);
Lambda1_inv_powers = compute_seriesPowers_intval(lambda_series1_inv, N, M);
delta_lambda1_inv = matrixSeries_validateInverse_intval(...
                   lambda_series1, lambda_series1_inv, ...
                   delta_lambda1, tauBar1, M)


%Powers if the inverse series:               
sigma = intval(0.005);
delta_powers = validate_LambdaPowers_intval(...
                       lambda_series1_inv, tauBar1, delta_lambda1_inv, ...
                       Lambda1_inv_powers, sigma, N, M)



tauBar1_prime = tauBar1*exp(-sigma)


%==========================================================================
% Compute branch manifold coefficients:
%==========================================================================

                
[P_MN1, P_MN2] = henon_invariantBranch_intval_vectorized(...
                    x_M1, y_M1, ...
                    PN1, PN2, PN1_rho, PN2_rho, ...
                    lambda_series1, Lambda1_powers, ...
                    xi_1_series1, xi_2_series1, M, N, a, b); 

               
                
%==========================================================================
% Validate the Branch Manifold Series
%==========================================================================


s = intval(0.001)

[delta, delta_h] = validateUnstableBranch_Henon(P_MN1, P_MN2, ...
    x_M1, y_M1, delta_pM1, lambda_series1_inv, delta_lambda1_inv, ...
    Lambda1_inv_powers, delta_powers, ...
    Q_omega_sigmaNorm, Q_omegaInv_sigmaNorm, delta_xi1, ...
    tauBar1_prime, nu, s, M, N, a, b);
                
  
delta_bound = sup(delta)
tauBar1_prime
proofTime = toc/60






