clear 
format long

a = intval('1.4')
b = intval('0.3')

%Parameterization Controls:
M = 10
N = 10

%Parameter Domains:
tauBar1 = intval('0.1')
tau = tauBar1
nu = intval('0.1');

%=======Fixed Point (floating point formula)===============================

%First Fixed Point:
x_1 = ((b-1)/(2*a)) + (sqrt((1-b)^2 +4*a))/(2*a);
y_1 = b*x_1;

%Second Fixed Point:
x_2 = ((b-1)/(2*a)) - (sqrt((1-b)^2 +4*a))/(2*a);
y_2 = b*x_2;


%Fixed Point Parameter Expansions:==========================================

[x_M1, y_M1] = henon_FP_expansion_intval(x_1, y_1, a, b, M);
delta_pM1 = henon_FP_validation(x_M1, y_M1, tau, a, b, M);

[x_M2, y_M2] = henon_FP_expansion_intval(x_2, y_2, a, b, M);
delta_pM2 = henon_FP_validation(x_M2, y_M2, tau, a, b, M);

sup(delta_pM1)
sup(delta_pM2)


%==========================================================================
%Compute Eigenvalue and Eigenvector Series:
%==========================================================================

scale_eigs = 1

%Domain on which the differntial of the eigenvector and eigenvalue
%       operators can be inverted for fixed point branch one.


tic

[lambda1, lambda2, xi1, xi2, ...
 lambda_series1, lambda_series2, ...
 xi_1_series1, xi_2_series1, xi_1_series2, xi_2_series2, ...
 delta_lambda1, delta_lambda2, delta_xi1, delta_xi2] = ...
        henon_linearSeriesAtFP_intval(...
    M, tauBar1, scale_eigs, x_M1, y_M1, delta_pM1, a, b);


%Domain on which the differntial of the eigenvector and eigenvalue
%       operators can be inverted for fixed point branch two.
tauBar2 = 0.14
%tauBar2 = 0.001

[lambda3, lambda4, xi3, xi4, ...
 lambda_series3, lambda_series4, ...
 xi_1_series3, xi_2_series3, xi_1_series4, xi_2_series4, ...
 delta_lambda3, delta_lambda4, delta_xi3, delta_xi4] = ...
        henon_linearSeriesAtFP_intval(...
    M, tauBar2, scale_eigs, x_M2, y_M2, delta_pM2, a, b);


time = toc


sup(delta_lambda1)
sup(delta_lambda2)
sup(delta_xi1)
sup(delta_xi2)


sup(delta_lambda3)
sup(delta_lambda4)
sup(delta_xi3)
sup(delta_xi4)






%==========================================================================
%Q and Q^-1 matrix series computations:
%==========================================================================


Q = [xi1, xi2];
Q_inv = inv(Q);

            
%==========================================================================
%Build the Q_omega matrix series:             
%==========================================================================
Q_0 = [xi_1_series1(1), xi_1_series2(1);
       xi_2_series1(1), xi_2_series2(1)]



%delta_Q = (delta_xi1 + delta_xi2)
delta_Q = 10^(-10)
Q_omega = intval(zeros(2, 2, M+1));
Q_omega(1:2, 1:2, 1) = Q_0;

for m = 1:M
    Q_m = [xi_1_series1(m+1), xi_1_series2(m+1);
           xi_2_series1(m+1), xi_2_series2(m+1)];
    Q_omega(1:2, 1:2, m+1) = Q_m;
end


%==========================================================================
%Invert and validate the Q series and compute matrix norms:
%==========================================================================

Q_omegaInv = matrixSeries_formalInverse_intval(Q_omega, M);
delta_Qinv = matrixSeries_validateInverse_intval(...
                   Q_omega, Q_omegaInv, delta_Q, tau, M)



Q_omega_sigmaNorm = 0;
Q_omegaInv_sigmaNorm = 0;

for m = 0:M
    Q_omega_sigmaNorm = Q_omega_sigmaNorm + ...
                        norm(Q_omega(1:2, 1:2, m+1), inf)*tau^m;
    Q_omegaInv_sigmaNorm = Q_omegaInv_sigmaNorm + ...
                        norm(Q_omegaInv(1:2, 1:2, m+1), inf)*tau^m;;
end


%==========================================================================
%Compute 1D manifold coefficients at omega = 0:
%==========================================================================



%==========================================================================
%Q and Q^-1 matrix series computations:
%==========================================================================


Q = [xi1, xi2];
Q_inv = inv(Q);

            
%==========================================================================
%Build the Q_omega matrix series:             
%==========================================================================
Q_0 = [xi_1_series1(1), xi_1_series2(1);
       xi_2_series1(1), xi_2_series2(1)]



%delta_Q = (delta_xi1 + delta_xi2)
delta_Q = 10^(-10)
Q_omega = intval(zeros(2, 2, M+1));
Q_omega(1:2, 1:2, 1) = Q_0;

for m = 1:M
    Q_m = [xi_1_series1(m+1), xi_1_series2(m+1);
           xi_2_series1(m+1), xi_2_series2(m+1)];
    Q_omega(1:2, 1:2, m+1) = Q_m;
end


%==========================================================================
%Invert and validate the Q series and compute matrix norms:
%==========================================================================

Q_omegaInv = matrixSeries_formalInverse_intval(Q_omega, M);
delta_Qinv = matrixSeries_validateInverse_intval(...
                   Q_omega, Q_omegaInv, delta_Q, tau, M)



Q_omega_sigmaNorm = 0;
Q_omegaInv_sigmaNorm = 0;

for m = 0:M
    Q_omega_sigmaNorm = Q_omega_sigmaNorm + ...
                        norm(Q_omega(1:2, 1:2, m+1), inf)*tau^m;
    Q_omegaInv_sigmaNorm = Q_omegaInv_sigmaNorm + ...
                        norm(Q_omegaInv(1:2, 1:2, m+1), inf)*tau^m;;
end


%==========================================================================
%Compute 1D manifold coefficients at omega = 0:
%==========================================================================


[PN1, PN2] = henon_1D_manifold_intval([x_1, y_1], lambda2, xi2, ...
                                 a, b, N, intval(1));


                           
                             
%==========================================================================
%Compute rho partial of 1D PN at omega = 0 (i.e. first order term in omega).
%==========================================================================



[PN1_rho, PN2_rho] = ...
    henon_rhoPartial_intval([x_1, y_1], lambda2, xi2, ...
                            PN1, PN2, a, b, N);




%==========================================================================
%Compute 1D manifold parameter expansion in omega:
%==========================================================================

Lambda2_powers = compute_seriesPowers_intval(lambda_series2, N, M);


sigma = intval(0.005);
delta_powers = validate_LambdaPowers_intval(...
                       lambda_series2, tauBar1, delta_lambda2, ...
                       Lambda2_powers, sigma, N, M)



tauBar1_prime = tauBar1*exp(-sigma)



%==========================================================================
% Compute branch manifold coefficients:
%==========================================================================

                
[P_MN1, P_MN2] = henon_invariantBranch_intval_vectorized(...
                    x_M1, y_M1, ...
                    PN1, PN2, PN1_rho, PN2_rho, ...
                    lambda_series2, Lambda2_powers, ...
                    xi_1_series2, xi_2_series2, M, N, a, b); 

                
 
                
                
                
                
                
                

%==========================================================================
% Validate the coefficient tails:
%==========================================================================
                
%Compute the M_alpha terms:

M_alphas(1) = intval(1);
M_alphas(2) = intval(1);

for n = 2:N
    termZero = abs(lambda_series2(1) - Lambda2_powers(1, n+1));
    thisSum = intval(0);
    for m = 1:M
        thisSum = thisSum + ...
            abs(lambda_series2(m+1) - ...
                Lambda2_powers(m+1, n+1))*tauBar1_prime^m;
    end
    boundFromZero = termZero - thisSum - ...
                   (delta_lambda2 + delta_powers(n-1));
    if (inf(boundFromZero) <= 0)
        n
       crashM_alpha = tauTooBig   
    end
    M_alphas(n+1) = 1/boundFromZero;
end
                
                
%Compute the alpha coeffieicnt polynomial a-posteriori error:

delta_A = 2*abs(a)*delta_pM1

error_aPosPoly(1) = intval(0);
error_aPosPoly(2) = intval(0);

delta_sn(1) = delta_pM1;
delta_sn(2) = delta_xi2;
epsilon_n(1) = intval(0);
epsilon_n(2) = intval(0);
delta_hn(1) = delta_pM1;
delta_hn(2) = delta_xi2;

for n = 2:N
   lambda_n_poly = convertToPoly_1D1D(Lambda2_powers(:, n+1)', M);
   p1_poly = convertToPoly_1D1D(x_M1, M);
   a_n1_poly = convertToPoly_1D1D(P_MN1(:, n+1)', M);
   a_n2_poly = convertToPoly_1D1D(P_MN2(:, n+1)', M);
   omega_poly = convertToPoly_1D1D([0 1], 1);
   
   s_n = convertToPoly_1D1D(intval(zeros(1, M+1)), M);
   deltaTerm = intval(0);
   for k = 1:(n-1)
       a_nk = convertToPoly_1D1D(P_MN1(:, n-k+1)', M);
       a_k = convertToPoly_1D1D(P_MN1(:, k+1)', M);
       s_n = s_n + a_nk*a_k;
       deltaTerm = deltaTerm + ...
                  polyval(abs(a_nk), tauBar1_prime)*delta_sn(k+1) + ...
                  polyval(abs(a_k), tauBar1_prime)*delta_sn(n-k+1) + ...
                  delta_sn(n-k+1)*delta_sn(k+1);
   end
   delta_sn(n+1) = abs(a)*deltaTerm;
   
   thisError1 = a*s_n + 2*a*p1_poly*a_n1_poly - ...
                        a_n2_poly + lambda_n_poly*a_n1_poly;
   thisError2 = -b*a_n1_poly - omega_poly*a_n1_poly + ...
                           lambda_n_poly*a_n2_poly;
   
   error_aPosPoly(n+1) = polyval(abs(thisError1), tauBar1_prime) + ... 
                         polyval(abs(thisError2), tauBar1_prime);                     

    epsilon_n(n+1) = error_aPosPoly(n+1) + delta_sn(n+1) + ...
        (delta_A + delta_powers(n-1))*...
        (polyval(abs(a_n1_poly), tauBar1_prime)+ ...
         polyval(abs(a_n2_poly), tauBar1_prime)); 


    delta_hn(n+1) = ...
       Q_omega_sigmaNorm*Q_omegaInv_sigmaNorm*M_alphas(n+1)*epsilon_n(n+1);
end

delta_hn'


%==========================================================================
% Total polynomial a-posteriori error:
%==========================================================================

    
[P1_poly, P2_poly] = convertToPoly_2Dbranch_2Dspace(P_MN1, P_MN2, M, N);

bigZeros = intval(zeros(M+1, N+1));
[P1_lambda_poly, P2_lambda_poly] = ...
    convertToPoly_2Dbranch_2Dspace(bigZeros, bigZeros, M, N);
for n = 0:N
    lambda_n_poly = convertToPoly_1D1D(Lambda2_powers(:, n+1)', M);
    a1_mn_poly = convertToPoly_1D1D(P_MN1(:, n+1)', M);
    a2_mn_poly = convertToPoly_1D1D(P_MN2(:, n+1)', M);
    
    thetaTerm = polynom(1, n, {'x'});
    thisPoly_1 = a1_mn_poly*lambda_n_poly*thetaTerm;
    thisPoly_2 = a2_mn_poly*lambda_n_poly*thetaTerm;
    P1_lambda_poly = P1_lambda_poly + thisPoly_1;
    P2_lambda_poly = P2_lambda_poly + thisPoly_2;
end


omega_poly = polynom([0 1], [0 1], {'t'});
one_poly = polynom([1], [0], {'t'});

fP1_poly = one_poly + P2_poly - a*P1_poly*P1_poly;
fP2_poly = b*P1_poly + omega_poly*P1_poly;

E_MN1 = fP1_poly - P1_lambda_poly;
E_MN2 = fP2_poly - P2_lambda_poly;

epsilon_MN1 = polyval(abs(E_MN1), tauBar1_prime, nu)
epsilon_MN2 = polyval(abs(E_MN2), tauBar1_prime, nu)
epsilon_MN = epsilon_MN1 + epsilon_MN2


%==========================================================================
% Total a-posteriori error
%==========================================================================


lambda2_poly = convertToPoly_1D1D(lambda_series2, M);
mu_supStar = polyval(lambda2_poly, tauBar1_prime) + delta_lambda2

delta_H = intval(0);
delta_N = intval(0);
for n = 0:N
   delta_H = delta_H + ...
       delta_hn(n+1)*(mu_supStar^n)*(nu^n);
   delta_N = delta_N + delta_hn(n+1)*(nu^n);
end

thisTerm = intval(0);
for m = 0:M
   thisTerm = thisTerm +  ...
        max(sup(abs(P_MN1(m+1, 2))), sup(abs(P_MN2(m+1, 2))))*...
        delta_lambda2*(tauBar1_prime^m)*nu;
end

for n = 2:N
    for m = 0:M
        thisTerm = thisTerm + ...
            max(sup(abs(P_MN1(m+1, n+1))), sup(abs(P_MN2(m+1, n+1))))*...
            delta_powers(n-1)*(tauBar1_prime^m)*(nu^n);
    end
end


norm_PMN1 = polyval(abs(P1_poly), tauBar1_prime, nu);
norm_PMN2 = polyval(abs(P2_poly), tauBar1_prime, nu);
norm_PMN = intval(max(sup(norm_PMN1), sup(norm_PMN1)));

epsilon_N = epsilon_MN + thisTerm + delta_H + ...
    intval(max(sup(delta_N + 2*abs(a)*norm_PMN*delta_N + ...
    abs(a)*delta_N^2), sup(abs(b)*delta_N)))



%==========================================================================
%Complete Validation:
%==========================================================================


mu_supStar

s = intval(0.001)
rho_prime = norm_PMN + delta_H
rho = rho_prime + s

%Bound on the differential inverse:
M = intval(1) + (2*abs(a)/(abs(b) - tauBar1_prime))*rho_prime

%Bound on second derivatives:
M1 = 2*abs(a)

%Number of non-zero second derivatives:
M2 = 1

%dimension:
dim = 2

%Check order condition:
orderCheck = M*(mu_supStar)^(N+1)

if sup(orderCheck) >= 1
    crashNow = orderCheckFailed
end

%Bound on Truncation Error:
delta_P = 2*M/(1 - M*(mu_supStar)^(N+1)) * epsilon_N

%Check that bound is good enough:
cond1_num = (1 - M*(mu_supStar)^(N+1))/(2*dim*pi*M*M1*M2)

boundCondition = exp(intval(-1)) * min(inf(cond1_num), inf(s))

if inf(boundCondition) < sup(delta_P)
    crashNow = boundFailed
end


sup(delta_P)








