clear 
format long


%Lorenz Parameters:
sigma = intval('10')                          
beta = intval('8')/intval('3')
rho = intval('13.9265')
%rho = intval('28')



%Parameter Domains:
tau = intval('5.0');
domRad0 = intval('6.0'); 



%Parameterization Controls:
M = 10
N = 15


scale_eigs = intval(1)

p0 = [intval(0);
      intval(0);
      intval(0)];


[lambda1, lambda3, lambda2, xi1, xi3, xi2, ...
 lambda_series1, lambda_series3, lambda_series2, ...
 xi_1_series1, xi_2_series1, xi_3_series1, ...
 xi_1_series3, xi_2_series3, xi_3_series3, ...
 xi_1_series2, xi_2_series2, xi_3_series2, ...
 delta_lambda1, delta_lambda3, delta_lambda2, ...
 delta_xi1, delta_xi3, delta_xi2] = ...
        lorenz_linearSeriesAtOrigin_intval(M, tau, scale_eigs, ...
                                           sigma, beta, rho);


Q = [xi1, xi3, xi2];
Q_inv = inv(Q);


%Near-resonances at omega = 0:
worstResIndices = [0, 0];
worstRes = 1000000000;

for n1 = 0:N
    for n2 = 0:N
        if ((n1 + n2 >= 2) & (n1 + n2 <= N))
            n1;
            n2;
          resTest1 = abs(lambda1 - (n1*lambda1 + n2*lambda2));
          resTest3 = abs(lambda2 - (n1*lambda1 + n2*lambda2));
          resMultiIndices = [n1, n2];
          resVect = [inf(resTest1), inf(resTest3)]; 
          resTest = min(resVect);
          if resTest < worstRes
              worstResIndices = resMultiIndices;
              worstRes = resTest;
          end
        end
    end
end

worstResIndices
worstRes
            
%==========================================================================
%Build the Q_omega matrix series:             
%==========================================================================
Q_0 = [xi_1_series1(1), xi_1_series2(1), xi_1_series3(1);
       xi_2_series1(1), xi_2_series2(1), xi_2_series3(1);
       xi_3_series1(1), xi_3_series2(1), xi_3_series3(1)]



delta_Q = (delta_xi1 + delta_xi2)
Q_omega = intval(zeros(3, 3,M+1));
Q_omega(1:3, 1:3, 1) = Q_0;

for m = 1:M
    Q_m = [xi_1_series1(m+1), xi_1_series2(m+1), xi_1_series3(m+1);
           xi_2_series1(m+1), xi_2_series2(m+1), xi_2_series3(m+1);
           xi_3_series1(m+1), xi_3_series2(m+1), xi_3_series3(m+1)];
    Q_omega(1:3, 1:3, m+1) = Q_m;
end


%==========================================================================
%Invert and validate the Q series and compute matrix norms:
%==========================================================================

Q_omegaInv = matrixSeries_formalInverse_intval(Q_omega, M);
delta_Qinv = matrixSeries_validateInverse_intval(...
                   Q_omega, Q_omegaInv, delta_Q, tau, M)



Q_omega_sigmaNorm = 0;
Q_omegaInv_sigmaNorm = 0;

for m = 0:M
    Q_omega_sigmaNorm = Q_omega_sigmaNorm + ...
                        norm(Q_omega(1:3, 1:3, m+1), inf)*tau^m;
    Q_omegaInv_sigmaNorm = Q_omegaInv_sigmaNorm + ...
                        norm(Q_omegaInv(1:3, 1:3, m+1), inf)*tau^m;;
end




%==========================================================================
%Compute 1D manifold coefficients at omega = 0:
%==========================================================================



[PN1, PN2, PN3] = lorenz_1D_manifold_intval(p0, lambda3, xi3, ...
                                 sigma, rho, beta, N, intval(1));

 

                               
%==========================================================================                                  
%1D Manifold Residual:     
%==========================================================================

                              
                                  
[goodComputation, delta0] = lorenzApostori_1D_intval(...
                              N, intval(1), sigma, rho, beta, ...
                              domRad0, PN1, PN2, PN3, lambda3, Q, Q_inv);                             


delta0  



                     
%==========================================================================
%Compute rho partial of 1D PN at omega = 0 (i.e. first order term in omega).
%==========================================================================


[PN1_rho, PN2_rho, PN3_rho] = lorenz_1D_rhoPartial_at0_intval(...
                                      p0, lambda3, xi3, ...
                                      PN1, PN2, PN3, ...
                                      sigma, rho, beta, ... 
                                      N, intval(1));


%==========================================================================
%Omega Partial at omega = 0; 1D Residuial:
%==========================================================================


E_N_residual_rho1D= residual_1D_rhoPartial_at0_intval(PN1, PN2, PN3, ...
    PN1_rho, PN2_rho, PN3_rho, ...
    lambda_series3, domRad0, sigma, beta, rho, N)



%==========================================================================
%Compute 1D manifold parameter expansion in omega:
%==========================================================================



[P_MN1, P_MN2, P_MN3] = lorenz_1D_unstableBranch_at0_intval(...
    PN1, PN2, PN3, PN1_rho, PN2_rho, PN3_rho, lambda_series3,...
    xi_1_series3, xi_2_series3, xi_3_series3, ...
    M, N, sigma, beta, rho);



%==========================================================================
%==========================================================================
%Validate individual tails: p_alpha^M(omega)
%==========================================================================
%==========================================================================

%Compute the residuals:
HM_alpha_bounds = compute_alphaM_bounds1D(P_MN1, P_MN2, P_MN3, ...
                              lambda_series3, delta_xi3, ...
                              p0, tau, intval(1), ...
                              sigma, rho, beta, M, N);

                                
                          
%Compute Bounds on Iverse Matrices:                          

resonanceBounds=compute_resonanceBounds1D(...
    lambda_series1, lambda_series2, lambda_series3, ...
    delta_lambda1, delta_lambda2, delta_lambda3, tau, N, M)
                         
    

%Compute Validated bounds on the coefficient tails:                          
H_alpha_validatedBounds= validated_coeff_bounds1D(P_MN1, P_MN2, P_MN3, ...
         Q_omega_sigmaNorm, delta_Q, Q_omegaInv_sigmaNorm, delta_Qinv,...
         resonanceBounds, HM_alpha_bounds, delta_lambda3, delta_xi3, ...
         p0, tau, intval(1), sigma, rho, beta, M, N);                       

 
     
 

%==========================================================================
%Omega Expansion 1D Residual Error:
%==========================================================================


E_MN_residual_1Dbranch = residual_unstableBranch_at0_intval(...
            P_MN1, P_MN2, P_MN3, lambda_series3, ...
            sigma, beta, rho, domRad0, tau, M, N)



 
%========================================================================== 
%Validated A-Posteriori Bounds:
%==========================================================================


s = intval('0.05')

[delta_HN, goodComputation] = validatedBranchError_unstableAtOrigin(...
    P_MN1, P_MN2, P_MN3, domRad0, tau, s, E_MN_residual_1Dbranch, ...
    Q_omega_sigmaNorm, delta_Q, Q_omegaInv_sigmaNorm, delta_Qinv,...
    lambda_series3, delta_lambda3,...
    H_alpha_validatedBounds, M, N, sigma, beta, rho);


delta_HN
















