%% a_validated2DLorenzBranch computation of family of 2D manifolds.
%
%%

clear 
format long

%================ User Parameters/Controls ================================


%Lorenz Parameters:
sigma = intval('10')                          
beta = intval('8')/intval('3')
rho = intval('13.9265')
%rho = intval('28')

%Parameter Interval:
tau = intval(10^(-2))
%All Taylor Models must validate on this 
%interval.  If any one faile program will 
%crash.

%Parameterization Disk (poly-disk radius):
domRad0 = intval(1.2)

%Parameterization Controls:
M = 5     %Order in parameter variable
N = 10    %Order in dyanmical variables

%Eigenvector Normalization:
scale_eigs = intval(1)

%Fixed Point at origin:
p0 = [intval(0);
      intval(0);
      intval(0)];
  
  
  
 %============= Start Computation =========================================
  
  
tic 
   
%Compute Analytic Taylor Models of all linear data:
[lambda1, lambda3, lambda2, xi1, xi3, xi2, ...
 lambda_series1, lambda_series3, lambda_series2, ...
 xi_1_series1, xi_2_series1, xi_3_series1, ...
 xi_1_series3, xi_2_series3, xi_3_series3, ...
 xi_1_series2, xi_2_series2, xi_3_series2, ...
 delta_lambda1, delta_lambda3, delta_lambda2, ...
 delta_xi1, delta_xi3, delta_xi2] = ...
        lorenz_linearSeriesAtOrigin_intval(M, tau, scale_eigs, ...
                                           sigma, beta, rho);

       
%Diagnalizing Transformation:                                       
Q = [xi1, xi3, xi2];
%Inverse:
Q_inv = inv(Q);



%-------This computation tests the eigenvalues and------------------------- 
%-------finds the closest resonance, or `most resonant eig-----------------

%Near-resonances at omega = 0:
worstResIndices = [0, 0];
worstRes = 1000000000;

for n1 = 0:N
    for n2 = 0:N
        if ((n1 + n2 >= 2) & (n1 + n2 <= N))
            n1;
            n2;
          resTest1 = abs(lambda1 - (n1*lambda1 + n2*lambda2));
          resTest3 = abs(lambda2 - (n1*lambda1 + n2*lambda2));
          resMultiIndices = [n1, n2];
          resVect = [inf(resTest1), inf(resTest3)]; 
          resTest = min(resVect);
          if resTest < worstRes
              worstResIndices = resMultiIndices;
              worstRes = resTest;
          end
        end
    end
end

worstResIndices
worstRes
            
%==========================================================================
%Build the Q_omega matrix series:             
%==========================================================================
Q_0 = [xi_1_series1(1), xi_1_series2(1), xi_1_series3(1);
       xi_2_series1(1), xi_2_series2(1), xi_2_series3(1);
       xi_3_series1(1), xi_3_series2(1), xi_3_series3(1)]



delta_Q = intval(max(sup(delta_xi1),sup(delta_xi2)))
Q_omega = intval(zeros(3, 3,M+1));
Q_omega(1:3, 1:3, 1) = Q_0;

for m = 1:M
    Q_m = [xi_1_series1(m+1), xi_1_series2(m+1), xi_1_series3(m+1);
           xi_2_series1(m+1), xi_2_series2(m+1), xi_2_series3(m+1);
           xi_3_series1(m+1), xi_3_series2(m+1), xi_3_series3(m+1)];
    Q_omega(1:3, 1:3, m+1) = Q_m;
end


%==========================================================================
%Invert and validate the Q series and compute matrix norms:
%==========================================================================

Q_omegaInv = matrixSeries_formalInverse_intval(Q_omega, M);
delta_Qinv = matrixSeries_validateInverse_intval(...
                   Q_omega, Q_omegaInv, delta_Q, tau, M)



Q_omega_sigmaNorm = 0;
Q_omegaInv_sigmaNorm = 0;

for m = 0:M
    Q_omega_sigmaNorm = Q_omega_sigmaNorm + ...
                        norm(Q_omega(1:3, 1:3, m+1), inf)*tau^m;
    Q_omegaInv_sigmaNorm = Q_omegaInv_sigmaNorm + ...
                        norm(Q_omegaInv(1:3, 1:3, m+1), inf)*tau^m;;
end



%==========================================================================
%Compute 2D manifold coefficients at omega = 0:
%==========================================================================

[QN1, QN2, QN3] = ...
    lorenz_2D_manifold_intval_vectorized(...
                              p0, lambda1, lambda2, xi1, xi2, ...
                              sigma, rho, beta, N, intval(1));

                                
%==========================================================================                                  
%2D Manifold Residual:     
%==========================================================================
                      
[goodComputation, delta0] = lorenzApostori_intval(N, scale_eigs, ... 
                                         sigma, rho, beta, domRad0, ...
                                         QN1, QN2, QN3,lambda1,lambda2,...
                                         Q, Q_inv);                             



delta0  


%==========================================================================
%Compute rho partial of 2D PN at omega = 0 (i.e. first order term in omega).
%==========================================================================


[QN1_rho, QN2_rho, QN3_rho] = ...
    lorenz_2D_rhoPartial_at0_intval_vectorized(...
                                    p0, lambda1, lambda2, ...
                                    xi1, xi2, QN1, QN2, QN3, ...
                                    sigma, rho, beta, N, 1);

                                
                               
                                
                                
%==========================================================================
%Omega Partial at omega = 0; 2D Residuial:
%==========================================================================



E_N_residual_rho2D = residual_2D_rhoPartial_at0_intval_poly(...
    QN1, QN2, QN3, QN1_rho, QN2_rho, QN3_rho, ...
    lambda_series1, lambda_series2, lambda_series3, ...
    domRad0, sigma, beta, rho, N)



%==========================================================================
%Compute 2D manifold parameter expansion in omega:
%==========================================================================

tic

[Q_MN1, Q_MN2, Q_MN3] = ...
lorenz_2D_stableBranch_at0_intval_vectorized(...
    QN1, QN2, QN3, QN1_rho, QN2_rho, QN3_rho, ...
    lambda_series1, lambda_series2, ...
    xi_1_series1, xi_2_series1, xi_3_series1, ...
    xi_1_series2, xi_2_series2, xi_3_series2, ...
    M, N, sigma, beta, rho);


time_2D_manifold_new = toc


%==========================================================================
%==========================================================================
%Validate individual tails: p_alpha^M(omega)
%==========================================================================
%==========================================================================


HM_alpha_bounds = ...
    compute_alphaM_bounds_poly(Q_MN1, Q_MN2, Q_MN3, ...
                              lambda_series1, lambda_series2, ...
                              delta_xi1, delta_xi2, p0, tau, intval(1), ...
                              sigma, rho, beta, M, N)                         
                                                   
%Compute Bounds on Iverse Matrices:                          

resonanceBounds=compute_resonanceBounds(...
    lambda_series1, lambda_series2, lambda_series3, ...
    delta_lambda1, delta_lambda2, delta_lambda3, tau, N, M)
                         
                         

%Compute Validated bounds on the coefficient tails:                          
H_alpha_validatedBounds = ...
    validated_coeff_bounds_optimized1(Q_MN1, Q_MN2, Q_MN3, ...
         Q_omega_sigmaNorm, delta_Q, Q_omegaInv_sigmaNorm, delta_Qinv,...
         resonanceBounds, HM_alpha_bounds, ...
         delta_lambda1, delta_lambda2, delta_xi1, delta_xi2, ...
         p0, tau, intval(1), sigma, rho, beta, M, N);                       


         
     
%========================================================================
%Omega Expansion 2D Residual Error:
%========================================================================
 
E_MN_residual_2Dbranch = ...
     residual_stableBranch_at0_intval(Q_MN1, Q_MN2, Q_MN3, ...
     lambda_series1, lambda_series2, sigma, beta, rho, domRad0, tau, M, N)
 
 
 
%========================================================================== 
%Validated A-Posteriori Bounds:
%==========================================================================


s = intval('0.05')

[delta_HN, goodComputation] = validatedBranchError_stableAtOrigin(...
    Q_MN1, Q_MN2, Q_MN3, domRad0, tau, s, E_MN_residual_2Dbranch, ...
    Q_omega_sigmaNorm, delta_Q, Q_omegaInv_sigmaNorm, delta_Qinv,...
    lambda_series1, lambda_series2, delta_lambda1, delta_lambda2,...
    H_alpha_validatedBounds, M, N, sigma, beta, rho);

delta_HN



compTime = toc





