

clear 
format long

%==========Parameter Expansions for the Henon Map==========================


a = 1.4
b = 0.3
N = 140


%Fixed Point (floating point formula)

x_1 = ((b-1)/(2*a)) + (sqrt((1-b)^2 +4*a))/(2*a)
y_1 = b*x_1

x_2 = ((b-1)/(2*a)) - (sqrt((1-b)^2 +4*a))/(2*a)
y_2 = b*x_2



f1 = henonMap(x_1, y_1, a, b)
f2 = henonMap(x_2, y_2, a, b)


error1 = norm(f1 - [x_1; y_1])
error2 = norm(f2 - [x_2; y_2])


%Fixed Point Parameter Expansion:==========================================


x0 = x_1
y0 = b*x0;

x1 = x0/(2*a*x0 - b + 1)
y1 = b*x1 + x0

xn = zeros(1, N+1);
yn = xn;
xn(1) = x0;
yn(1) = y0;
xn(2) = x1;
yn(2) = y1;


coef_1 = 1/(2*a*x0-b+1);
for n = 2:N
   thisSum = 0;
   for k = 1:(n-1)
       thisSum = thisSum + xn(n-k+1)*xn(k+1);
   end
   xn(n+1) = coef_1*(xn(n) - a*thisSum);
   yn(n+1) = b*xn(n+1) + xn(n);
end



%Test Error:

omega = -2.0
p1_omega = [horner(omega, xn, N);
            horner(omega, yn, N)]
        
fp1_omega = henonMap(p1_omega(1), p1_omega(2), a, b+omega)
error_omega1 = norm(p1_omega - fp1_omega)
        

%Plot fixed points:
numPlotPoints = 100
omegaAxis = linspace(-omega, omega, numPlotPoints);
x_points = zeros(1, numPlotPoints);
for k = 1:numPlotPoints
    x_points(k) = horner(omegaAxis(k), xn, N); 
end

plot(omegaAxis, x_points, 'b')


%Eigenvalues: =============================================================


lambda1 = -a*x_1 + sqrt((a*x_1)^2 + b)
lambda2 = -a*x_1 - sqrt((a*x_1)^2 + b)    

%g = det([-2*a*x_1-lambda2, 1; b, -lambda2])
%%[A, B] = eigs([-2*a*x0, 1; b, 0])

lambda11 = (1 - 2*a*x1*lambda1)/(2*lambda1 + 2*a*x0);
lambda21 = (1 - 2*a*x1*lambda2)/(2*lambda2 + 2*a*x0);  


M = 50
lambda1n = zeros(1, M+1);
lambda1n(1) = lambda1;
lambda1n(2) = lambda11;

coef_2 = -1/(2*lambda1n(1) + 2*a*x0)
for n = 2:M
    theSum1 = 0;
    for k = 1:(n-1)
        theSum1 = theSum1 + lambda1n(n-k+1)*lambda1n(k+1);
    end
    
    theSum2 = 0;
    for k = 0:(n-1)
        theSum2 = theSum2 + xn(n-k+1)*lambda1n(k+1);
    end
    lambda1n(n+1) = coef_2*(theSum1 + 2*a*theSum2);
end


omega = 0.2
lambda1_omega = horner(omega, lambda1n, M) 

[U, Sigma] = eigs([-2*a*horner(omega, xn, N), 1; b+omega, 0])

error_lambda = Sigma(2,2) - lambda1_omega

%Plot

numPlotPoints = 100;
omegaAxis = linspace(-omega, omega, numPlotPoints);
lambda_points = zeros(1, numPlotPoints);
for k = 1:numPlotPoints
    lambda_points(k) = horner(omegaAxis(k), lambda1n, M); 
end

plot(omegaAxis, lambda_points, 'g')


%Eigenvectors==============================================================

[U, Sigma] = eigs([-2*a*x_1, 1;
                         b, 0])
%Eigenvector:
xi1_0 = U(:, 2)

%E = [-2*a*x_1-lambda1,        1;
%                   b, -lambda1]*xi1_0

%Derivative of Eigenvector at omega = 0 <==> first coeficient.
xi1_1 = [b, -lambda1n(1);
         2*xi1_0(1), 2*xi1_0(2)]\[lambda1n(2)*xi1_0(2) - xi1_0(1);
                                   0]

%Test that this is the derivative:                               
omega = -3*10^(-1)                              
[U_omega, Sigma_omega] = eigs([-2*a*horner(omega, xn, N), 1; b+omega, 0])               
  

delta_xi1_0 = U_omega(:, 2)

delta_error = norm(delta_xi1_0 - xi1_0 - xi1_1*omega)

          
K = 30
eig_homMatrix = [         b, -lambda1n(1);
                 2*xi1_0(1),  2*xi1_0(2)];
    
xi1n = zeros(2, K+1);
xi1n(:, 1) = xi1_0;
xi1n(:, 2) = xi1_1;
             
for n = 2:K               
    sum1 = 0;
    for k = 0:(n-1)
        sum1 = sum1 + lambda1n(n-k+1)*xi1n(2, k+1);
    end
    sum2 = 0;
    for k = 1:(n-1)
        sum2 = sum2 + xi1n(1, n-k+1)*xi1n(1, k+1) + ...
                      xi1n(2, n-k+1)*xi1n(2, k+1);
    end
    thisRHS = [-xi1n(1, n) + sum1;
               -sum2];
           
    xi1n(:, n+1) = eig_homMatrix\thisRHS;
end
               
               
xi1_omega = [horner(omega, xi1n(1, :), K);
             horner(omega, xi1n(2, :), K)]
         
         
eigVect_error = norm(xi1_omega - delta_xi1_0)         
         
         
         
         
         


