
clear 
format long

%==========Parameter Expansions for the Henon Map==========================


a = 1.4
b = 1.0
N = 140


%Fixed Point (floating point formula)

x_1 = ((b-1)/(2*a)) + (sqrt((1-b)^2 +4*a))/(2*a)
y_1 = b*x_1

x_2 = ((b-1)/(2*a)) - (sqrt((1-b)^2 +4*a))/(2*a)
y_2 = b*x_2



f1 = henonMap(x_1, y_1, a, b)
f2 = henonMap(x_2, y_2, a, b)


error1 = norm(f1 - [x_1; y_1])
error2 = norm(f2 - [x_2; y_2])


%Fixed Point Parameter Expansion:==========================================

[xn1, yn1] = henon_FP_expansion(x_1, y_1, a, b, N)
[xn2, yn2] = henon_FP_expansion(x_2, y_2, a, b, N)



%Test Error:

omega = 2.0
p1_omega = [horner(omega, xn1, N);
            horner(omega, yn1, N)]
        
fp1_omega = henonMap(p1_omega(1), p1_omega(2), a, b+omega)
error_omega1 = norm(p1_omega - fp1_omega)
        

%Plot fixed points:
numPlotPoints = 100
omegaAxis = linspace(-omega, omega, numPlotPoints);
x_points1 = zeros(1, numPlotPoints);
for k = 1:numPlotPoints
    x_points1(k) = horner(omegaAxis(k), xn1, N); 
end


p2_omega = [horner(omega, xn2, N);
            horner(omega, yn2, N)]
        
fp2_omega = henonMap(p2_omega(1), p2_omega(2), a, b+omega)

%Plot fixed points:
numPlotPoints = 100
omegaAxis = linspace(-omega, omega, numPlotPoints);
x_points2 = zeros(1, numPlotPoints);
for k = 1:numPlotPoints
    x_points2(k) = horner(omegaAxis(k), xn2, N); 
end



error_omega2 = norm(p2_omega - fp2_omega)
error_omega1 = norm(p1_omega - fp1_omega)


figure 
hold on
plot(b, x_1, 'k*')
plot(b, x_2, 'k*')
plot(b+omegaAxis, x_points1, 'b')
plot(b+omegaAxis, x_points2, 'g')




%Eigenvalues: =============================================================


lambda11 = -a*x_1 + sqrt((a*x_1)^2 + b)
lambda12 = -a*x_1 - sqrt((a*x_1)^2 + b)    

lambda21 = -a*x_2 + sqrt((a*x_2)^2 + b)
lambda22 = -a*x_2 - sqrt((a*x_2)^2 + b) 

%g = det([-2*a*x_1-lambda2, 1; b, -lambda2])
%%[A, B] = eigs([-2*a*x0, 1; b, 0])

N_lambda1 = 120;
[lambda1n] = henon_eigVal_expansion(xn1, lambda11, a, b, N_lambda1);
N_lambda2 = 120;
[lambda2n] = henon_eigVal_expansion(xn1, lambda12, a, b, N_lambda2);



omega = 0.55
lambda1_omega = horner(omega, lambda1n, N_lambda1) 

[U, Sigma] = eigs([-2*a*horner(omega, xn1, N), 1; b+omega, 0])

error_lambda = Sigma(2,2) - lambda1_omega


%Plot

numPlotPoints = 100;
omegaAxis1 = linspace(-omega, omega, numPlotPoints);
lambda_points = zeros(1, numPlotPoints);
for k = 1:numPlotPoints
    lambda_points1(k) = horner(omegaAxis1(k), lambda1n, N_lambda1); 
end

omega2 = 0.6
lambda2_omega = horner(omega2, lambda2n, N_lambda2) 

[U, Sigma] = eigs([-2*a*horner(omega2, xn1, N), 1; b+omega2, 0])

error_lambda2 = Sigma(1,1) - lambda2_omega

%Plot

numPlotPoints = 100;
omegaAxis2 = linspace(-omega2, omega2, numPlotPoints);
lambda_points2 = zeros(1, numPlotPoints);
for k = 1:numPlotPoints
    lambda_points2(k) = horner(omegaAxis2(k), lambda2n, N_lambda2); 
end


figure
hold on

theta = linspace(0, 2*pi);
% creates an array of points representing the radians
plot(cos(theta), sin(theta), 'g');
plot(0,0, 'ko')
plot(lambda11, 0, 'k*')
plot(lambda12, 0, 'k*')
plot(lambda_points1, zeros(1, numPlotPoints), 'r')
plot(lambda_points2, zeros(1, numPlotPoints), 'b')


figure
hold on
plot(b, lambda12, 'k*')
plot(b, lambda11, 'k*')
plot(b+omegaAxis2, lambda_points2, 'b')
plot(b+omegaAxis1, lambda_points1, 'r')



N_lambda3 = 140;
[lambda3n] = henon_eigVal_expansion(xn2, lambda21, a, b, N_lambda3);
N_lambda4 = 140;
[lambda4n] = henon_eigVal_expansion(xn2, lambda22, a, b, N_lambda4);


omega3 = 1.6
lambda3_omega = horner(omega3, lambda3n, N_lambda3) 

[U, Sigma] = eigs([-2*a*horner(omega3, xn2, N), 1; b+omega3, 0])

error_lambda3 = Sigma(1,1) - lambda3_omega


%Plot

numPlotPoints = 100;
omegaAxis3 = linspace(-omega3, omega3, numPlotPoints);
lambda_points3 = zeros(1, numPlotPoints);
for k = 1:numPlotPoints
    lambda_points3(k) = horner(omegaAxis(k), lambda3n, N_lambda3); 
end

omega4 = 1.65
lambda4_omega = horner(omega4, lambda4n, N_lambda4) 

[U, Sigma] = eigs([-2*a*horner(omega4, xn2, N), 1; b+omega4, 0])

error_lambda4 = Sigma(2,2) - lambda4_omega



%Plot

numPlotPoints = 100;
omegaAxis = linspace(-omega4, omega4, numPlotPoints);
lambda_points4 = zeros(1, numPlotPoints);
for k = 1:numPlotPoints
    lambda_points4(k) = horner(omegaAxis(k), lambda4n, N_lambda4); 
end



figure
hold on

theta = linspace(0, 2*pi);
% creates an array of points representing the radians
plot(cos(theta), sin(theta), 'g');
plot(0,0, 'ko')
plot(lambda21, 0, 'k*')
plot(lambda22, 0, 'k*')
plot(lambda_points3, zeros(1, numPlotPoints), 'r')
plot(lambda_points4, zeros(1, numPlotPoints), 'b')


figure
hold on
plot(b, lambda21, 'k*')
plot(b+omegaAxis3, lambda_points3, 'g')



%Eigenvectors==============================================================

omega = 0.2

[U1, Sigma1] = eigs([-2*a*xn1(1), 1;
                         b, 0])
%Eigenvector:
xi1_0 = U1(:, 2)

%Compute Expansions:
N_xi_11 = 50  
[xi_11_n] = henon_eigVect_expansion(xn1, lambda1n, xi1_0, a, b, N_xi_11)


[U_omega, Sigma_omega] = eigs([-2*a*horner(omega, xn1, N), 1; b+omega, 0])               
  
delta_xi1_0 = U_omega(:, 2)
          
xi1_omega = [horner(omega, xi_11_n(1, :), N_xi_11);
             horner(omega, xi_11_n(2, :), N_xi_11)]
              
eigVect_error = norm(xi1_omega - delta_xi1_0)         
         

omega = 0.6

[U1, Sigma1] = eigs([-2*a*xn1(1), 1;
                         b, 0])
%Eigenvector:
xi1_0 = U1(:, 1)

%Compute Expansions:
N_xi_12 = N_lambda2 
[xi_12_n] = henon_eigVect_expansion(xn1, lambda2n, xi1_0, a, b, N_xi_12)

[U_omega, Sigma_omega] = eigs([-2*a*horner(omega, xn1, N), 1; b+omega, 0])               
  
delta_xi1_0 = U_omega(:, 1)
          
xi1_omega = [horner(omega, xi_12_n(1, :), N_xi_12);
             horner(omega, xi_12_n(2, :), N_xi_12)]
              
eigVect_error = norm(xi1_omega - delta_xi1_0) 



omega = 1.25

[U1, Sigma1] = eigs([-2*a*xn2(1), 1;
                         b, 0])
%Eigenvector:
xi1_0 = U1(:, 1)


%Compute Expansions:
N_xi_21 = N_lambda3 
[xi_21_n] = henon_eigVect_expansion(xn2, lambda3n, xi1_0, a, b, N_xi_21)

[U_omega, Sigma_omega] = eigs([-2*a*horner(omega, xn2, N), 1; b+omega, 0])               
  
delta_xi1_0 = U_omega(:, 1)
          
xi1_omega = [horner(omega, xi_21_n(1, :), N_xi_21);
             horner(omega, xi_21_n(2, :), N_xi_21)]
              
eigVect_error = norm(xi1_omega - delta_xi1_0) 







omega = 0.2

[U1, Sigma1] = eigs([-2*a*xn2(1), 1;
                         b, 0])
%Eigenvector:
xi1_0 = U1(:, 2)


%Compute Expansions:
N_xi_22 = N_lambda4 
[xi_22_n] = henon_eigVect_expansion(xn2, lambda4n, xi1_0, a, b, N_xi_22)

[U_omega, Sigma_omega] = eigs([-2*a*horner(omega, xn2, N), 1; b+omega, 0])               
  
delta_xi1_0 = U_omega(:, 2)
          
xi1_omega = [horner(omega, xi_22_n(1, :), N_xi_22);
             horner(omega, xi_22_n(2, :), N_xi_22)]
              
eigVect_error = norm(xi1_omega - delta_xi1_0) 







