function varargout=henon_rhoPartial_intval(p, lambda, xi, ...
                              P1, P2, a, b, N);

%Differential at the fixed point and it's rho partial.                                  
Dfp = henonDifferential_intval(p(1), p(2), a, b);

%rho partial of differential;


x_1 = p(1)/(2*a*p(1) - b + 1);
y_1 = b*x_1 + p(1);
Dfp_rhoPartial = intval([-2*a*x_1, 0;
                                1, 0]);

%rho partials of first order data.
lambda_1 = ...
    rhoPartial_of_henon_eigVal_intval(p, lambda, a, b);  

xi_rhoPartial =  ...
    rhoPartial_of_henon_eigVect_intval(p, lambda, xi, a, b);

   
    
    
%Initialize coefficient storage.
P1_rho = intval(zeros(1, N+1));
P2_rho = intval(zeros(1, N+1));
  
%Assign first order coefficients:

%The orgin is a fixed point for all rho, then the partial of
%the fixed point with respect to rho vanishes.
P1_rho(1) = x_1;
P2_rho(1) = y_1;


%First order coefficients are rho partials of eigenvectors.
P1_rho(2) = xi_rhoPartial(1);
P2_rho(2) = xi_rhoPartial(2);

Id = intval(eye(2));

%Compute b_mn coefficient for m+n >= 2.
for n = 2:N
    An = Dfp - (lambda^n)*Id;            
    a_n = [P1(n+1);
           P2(n+1)];
                
    sn_rho = compute_henon_sn_rhoPartial_intval(n, P1, P2, ...
                      P1_rho, P2_rho, a, b, N);
              
    b_n = sn_rho - [Dfp_rhoPartial - ...
        (n*lambda^(n-1)*lambda_1)*Id]*a_n;               
                          
    vn = An\b_n;
    P1_rho(n+1) = vn(1);
    P2_rho(n+1) = vn(2);
end

varargout(1,1)={P1_rho};
varargout(2,1)={P2_rho};


