
clear 
format long

intvalinit('SharpIVmult')



%==========================================================================
%
%
%
%
%
%
%==========================================================================


%---------------------------Global Parameters------------------------------
%time scale
L = intval(1.0)
%Parameterization Orders:
N0_s = 20
N1_u = 20
%Parameterization Domain Sizes
domRad0_s = intval('0.5');

domRad1_u = intval('0.5');


%Lorenz Parameters
sigma = intval(10)                          
beta = intval(8/3)
rho = intval(28)


scale0_s1 = 1.5;
scale0_s2 = 1.5;
scale0_u  = 1.0;


scale1_complex = intval(2.5);
scale1_real  = intval(1.0);


%==========================================================================
%==========================Invariant Manifolds=============================
%==========================================================================


%-----------First order data at the origin---------------------------------

p0 = intval([0 0 0]);

Dfp0 = lorenzDifferential_intval(p0, sigma, rho, beta, L);

[U_temp, Sigma_temp] = eigs(Dfp0.mid)      






[lambda1_temp, xi1_temp] = ...
     verifyeig(Dfp0, Sigma_temp(1,1), U_temp(:,1));

[lambda2_temp, xi2_temp] = ...
     verifyeig(Dfp0, Sigma_temp(2,2), U_temp(:,2));

[lambda3_temp, xi3_temp] = ...
     verifyeig(Dfp0, Sigma_temp(3,3), U_temp(:,3));


 
Q0 = [xi1_temp, xi2_temp, xi3_temp];
Q0_inv = inv(Q0)
 

if (inf(Sigma_temp(1,1)) > 0)
    lambda0_s1 = lambda2_temp;
    lambda0_s2 = lambda3_temp;
    lambda0_u  = lambda1_temp; 
    xi0_s1 = scale0_s1*xi2_temp;
    xi0_s2 = scale0_s2*xi3_temp;
    xi0_u  = scale0_u*xi1_temp;
elseif (sup(Sigma_temp(1,1)) < 0)
    lambda0_s1 = lambda1_temp;
    lambda0_s2 = lambda3_temp;
    lambda0_u  = lambda2_temp; 
    xi0_s1 = scale0_s1*xi1_temp;
    xi0_s2 = scale0_s2*xi3_temp;
    xi0_u  = scale0_u*xi2_temp;
end

if ((sup(lambda0_s1) < 0) & (sup(lambda0_s2) < 0) & (inf(lambda0_u) > 1))
    goodLinearAssign_0 = 1
else
    %crash the program
    crashNow = linearProblem
end




%------------------------2D Manifold at Origin-----------------------------

tic

%Compute Coefficients:
[P0_s1, P0_s2, P0_s3] = lorenz_2D_manifold_intval(p0, lambda0_s1, lambda0_s2, ...
                             xi0_s1, xi0_s2, sigma, rho, beta, N0_s, L);



%Check Coefficients:                                    
[goodComputation,delta0_s] = lorenzApostori_intval(N0_s, L, sigma, rho, beta, ...
                              domRad0_s,P0_s1,P0_s2,P0_s3,lambda0_s1,lambda0_s2, Q0, Q0_inv);                             
   

time_0 = toc/60                          
                          
sup(delta0_s)

stopNow = stopHere




%----------------First order data at ``eye''-------------------------------



p1 = [sqrt(beta*(rho-1)), sqrt(beta*(rho-1)), rho-1];




error_p1 = lorenzRHS_intval(p1, sigma, rho, beta, L)


%stopNow = stopHere


Dfp1 = lorenzDifferential_intval(p1, sigma, rho, beta, L)
[U_temp, Sigma_temp] = eigs(Dfp1.mid);      



[lambda1_temp, xi1_temp] = ...
     verifyeig(Dfp1, Sigma_temp(1,1), U_temp(:,1));

[lambda2_temp, xi2_temp] = ...
     verifyeig(Dfp1, Sigma_temp(2,2), U_temp(:,2));

[lambda3_temp, xi3_temp] = ...
     verifyeig(Dfp1, Sigma_temp(3,3), U_temp(:,3));



Q1 = [xi1_temp, xi2_temp, xi3_temp];
Q1_inv = inv(Q0)
 

if(isreal(Sigma_temp(1,1)) == 1)
    if (Sigma_temp(1,1) > 0)
        lambda1_s1 = lambda2_temp;
        lambda1_s2 = lambda3_temp;
        lambda1_u  = lambda1_temp; 
        xi1_s1 = scale1_complex*xi2_temp;
        xi1_s2 = scale1_complex*xi3_temp;
        xi1_u  = scale1_real*xi1_temp;
    elseif (Sigma_temp(1,1) < 0)
        lambda1_u1 = lambda2_temp;
        lambda1_u2 = lambda3_temp;
        lambda1_s  = lambda1_temp; 
        xi1_u1 = scale1_complex*xi2_temp;
        xi1_u2 = scale1_complex*xi3_temp;
        xi1_s  = scale1_real*xi1_temp;
    end
elseif(isreal(Sigma_temp(3,3)) == 1)
   if (Sigma_temp(3,3) > 0)
        lambda1_s1 = lambda1_temp;
        lambda1_s2 = lambda2_temp;
        lambda1_u  = lambda3_temp; 
        xi1_s1 = scale1_s1*xi1_temp;
        xi1_s2 = scale1_s2*xi2_temp;
        xi1_u  = scale1_u*xi3_temp;
    elseif (Sigma_temp(3,3) < 0)
        lambda1_u1 = lambda1_temp;
        lambda1_u2 = lambda2_temp;
        lambda1_s  = lambda3_temp; 
        xi1_u1 = scale1_complex*xi1_temp;
        xi1_u2 = scale1_complex*xi2_temp;
        xi1_s  = scale1_real*xi3_temp;
    end 
end



%For ``eyes'' of Lorenz we expect two dimensional complex
%conjugate unstable and 1D stable.
if ((isreal(lambda1_s) == 1) && (sup(lambda1_s) < 0))
    goodLinearAssign_1 = 1
else
    %crash the program
    crashNow = linearProblem
end




tic

[P1_u1, P1_u2, P1_u3] = lorenz_2D_manifold_intval(p1, lambda1_u1, lambda1_u2, ...
                             xi1_u1, xi1_u2, sigma, rho, beta, N1_u, L);

                         


%Check Coefficients:                                    
[goodComputation,delta_complex] = lorenzApostori_intval(N1_u, L, sigma, rho, beta, ...
                              domRad1_u,P1_u1,P1_u2,P1_u3,lambda1_u1,lambda1_u2, Q1, Q1_inv);    

toc/60
                          
                          
%sup(delta0_s)                          
sup(delta_complex)













  