function [goodComputation,delta]=lorenzApostori_1D_intval(N,L,...
    sigma,rho,beta,domRad,P1,P2,P3,lambda, Q, Qinv);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Compute and Validate Parameterization in Gray-Scott
%
%
%-------------NOTE:--------------------------------------------------------
%The main `knobs' that can be adjusted in the computation are:
%N               -> The parameterization order
%domRad          -> The parameterization domain we try to 
%                   validate the estimates on
%L               -> How long the connecting orbit will be.
%
%If the validation fails, then you can either increase the order, decrease
%scaleEigs, or decrease domRad.  The only constraint is that N > 0, domRad
%needs to be greater than one (if we want exponentially decreasing bound)
%on the coefficients).  The smaller scaleEigs is, the smaller the image
%of the parameterizaiotn function will be, i.e. the smaller the local
%stable manifold will be, and the bigger L has to be. If you want to 
%decrease L, you can either increase N, increase domRad, or increase 
%scaleEigs.  However, any of these can destroy the estimates.  So there is
%still something of an art to choosing the right constants.  However,
%once you set them, the program will automatically tell you if they 
%are acceptable.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Ws Parameterization Control:

% s determines the distance between rho and rho'
s = intval('0.0001');

%==========================================================================
%A-Posteriori Validation of parameterization of stable manifold:
%==========================================================================

%==========================================================================
%A-Posteriori Calculation:
%
%Set up for validation of stable manifold of p = [1 0 0 0] in Gray-Scott.


%Code will have to be modified (slightly) to do unstable manifold or
%to do manifolds at other fixed points.
%==========================================================================

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Composition of the vector field with the Parameterization (as powerseries)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%The Hex vector field is 
%               | P_2                                                           |
%               | L^2 [-gamma/4 P_1 + sqrt(2)/4 P_3^2 + 3/8 P_1^3 3 P_1 P_3^2]  |
% g circ P   =  |                                                               |
%               |P_4                                                            |
%               |(^2[-gamma P_3 + sqrt(2)/2 P_1 P_3 + 9 P_3^3 + 3 P_1^2 P_3 ]   |
%


%Shift fixed point to origin:

%The nonlinear terms are evaluated using convolution sums


Pxy = convolveSeries_1D_general_intval(P1,P2);
Pxz = convolveSeries_1D_general_intval(P1,P3);


P1_big = intval(zeros(1, 2*N+1));
P2_big = intval(zeros(1, 2*N+1));
P3_big = intval(zeros(1, 2*N+1));

P1_big(1:(N+1)) = P1;
P2_big(1:(N+1)) = P2;
P3_big(1:(N+1)) = P3;

%Then:
g_of_P1 = L*sigma*(P2_big - P1_big);
g_of_P2 = L*(rho*P1_big - P2_big - Pxz);
g_of_P3 = L*(Pxy - beta*P3_big);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Composition of DP with L(theta):
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

DP_circL_theta_1 = intval(zeros(1, 2*N+1));
DP_circL_theta_2 = intval(zeros(1, 2*N+1));
DP_circL_theta_3 = intval(zeros(1, 2*N+1));


for n = 0:N
      if (n == 0)
        DP_circL_theta_1(1) = intval(0);
        DP_circL_theta_2(1) = intval(0);
        DP_circL_theta_3(1) = intval(0);
      elseif (n == 1) 
        DP_circL_theta_1(2) = lambda*P1(2);
        DP_circL_theta_2(2) = lambda*P2(2);
        DP_circL_theta_3(2) = lambda*P3(2);
      else
        DP_circL_theta_1(n+1) = (n*lambda)*P1(n+1);
        DP_circL_theta_2(n+1) = (n*lambda)*P2(n+1);
        DP_circL_theta_3(n+1) = (n*lambda)*P3(n+1);
      end
end

%Computation of difference = power series of error function:
Error1 = g_of_P1 - DP_circL_theta_1;
Error2 = g_of_P2 - DP_circL_theta_2;
Error3 = g_of_P3 - DP_circL_theta_3;

%Compute the weighted sum-norm for the error functional:
% Item 1 of Definition 4.1 (Validation Values)
error_weightedNorm_1 = intval(0);
error_weightedNorm_2 = intval(0);
error_weightedNorm_3 = intval(0);

for n = 0:(2*N) 
      norm_an_1 = abs(Error1(n+1));
	  norm_an_2 = abs(Error2(n+1));
	  norm_an_3 = abs(Error3(n+1));
      thisTerm_1 = norm_an_1*(domRad)^n;
	  thisTerm_2 = norm_an_2*(domRad)^n;
	  thisTerm_3 = norm_an_3*(domRad)^n;
      error_weightedNorm_1 = error_weightedNorm_1 + thisTerm_1;
	  error_weightedNorm_2 = error_weightedNorm_2 + thisTerm_2;
 	  error_weightedNorm_3 = error_weightedNorm_3 + thisTerm_3;
end




%Consider also the weighted norm of P_N itself, with the same weight,
%centered around the fixed point (hence the first term is zero):
% Item 2 of Definition 4.1 (Validation Values)


P_weightedNorm = intval(0);
for n = 0:N
    if (n == 0) 
      P_weightedNorm = intval(0);  
    else
      norm_a_n=norm([P1(n+1),P2(n+1),P3(n+1)]);
      thisTerm = norm_a_n*(domRad)^n;
      P_weightedNorm = P_weightedNorm + thisTerm;
    end
end

epsilon_tol = max([sup(error_weightedNorm_1), ...
                   sup(error_weightedNorm_2), ... 
                   sup(error_weightedNorm_3)]) 

P_supNorm_bound = P_weightedNorm

rho_prime = P_supNorm_bound
rho = rho_prime + s

thisSum = 0;
for n = 1:N
    thisCoef = abs(P1(n+1)) + max(sup(abs(P2(n+1))), sup(abs(P3(n+1))));
    thisSum = thisSum + (thisCoef/(lambda*n))*domRad^n;
end



if ((N+1) <= sup(lambda)/inf(lambda))
    crashNow = crashHere
end


C3 = exp(thisSum)
K1 = 1;
N_f = 4;
C4 = norm(Q, inf)*norm(Qinv, inf)
delta_test = 2*C3*epsilon_tol/((N+1)*inf(lambda) - sup(lambda))

cond1 = ((N+1)*lambda - lambda)/(2*3*exp(0)*intval('pi')*N_f*C3*C4*K1)
cond2 = s/exp(1)

if (sup(delta_test) >= min(inf(cond1), inf(cond2)))
    crashNow = crashHere
end

goodComputation = 1;
delta = delta_test;















