function [goodComputation,delta]=lorenzApostori_intval(...
                                 N,L,sigma,rho,beta,domRad, ...
                                 P1,P2,P3,lambda_1,lambda_2, ...
                                 Q, Q_inv);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Validate Parameterization 
%
%
%-------------NOTE:--------------------------------------------------------
%The main `knobs' that can be adjusted in the computation are:
%N               -> The parameterization order
%domRad          -> The parameterization domain we try to 
%                   validate the estimates on
%L               -> How long the connecting orbit will be.
%
%If the validation fails, then you can either increase the order, decrease
%scaleEigs, or decrease domRad.  The only constraint is that N > 0, domRad
%needs to be greater than one (if we want exponentially decreasing bound)
%on the coefficients).  The smaller scaleEigs is, the smaller the image
%of the parameterizaiotn function will be, i.e. the smaller the local
%stable manifold will be, and the bigger L has to be. If you want to 
%decrease L, you can either increase N, increase domRad, or increase 
%scaleEigs.  However, any of these can destroy the estimates.  So there is
%still something of an art to choosing the right constants.  However,
%once you set them, the program will automatically tell you if they 
%are acceptable.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


inValidation = 1

%Ws Parameterization Control:

% s determines the distance between rho and rho'
s = intval('0.01');

%==========================================================================
%A-Posteriori Validation of parameterization of stable manifold:
%==========================================================================


[Q1, Q2, Q3] = convertToPoly_2D(P1, P2, P3, N);


%Q1(0,0)
%Q2(0,0)
%Q3(0,0)

%stopNow = stopHere

%The nonlinear terms are evaluated using convolution sums


%Pxy = convolveSeries_2D_general_intval(P1,P2);
%Pxz = convolveSeries_2D_general_intval(P1,P3);

%atConvolution1 = 0

%Pxy = Q1*Q2;

%doneConvolution1 = 1

%atConvolution2 = 0

%Pxz = Q1*Q3;

%doneConvolution2 = 1

%stopNow = stopHere


%P1_big = intval(zeros(2*N+1, 2*N+1));
%P2_big = intval(zeros(2*N+1, 2*N+1));
%P3_big = intval(zeros(2*N+1, 2*N+1));

%P1_big(1:(N+1), 1:(N+1)) = P1;
%P2_big(1:(N+1), 1:(N+1)) = P2;
%P3_big(1:(N+1), 1:(N+1)) = P3;

%Then:
%g_of_P1 = L*sigma*(P2_big - P1_big);
%g_of_P2 = L*(rho*P1_big - P2_big - Pxz);
%g_of_P3 = L*(Pxy - beta*P3_big);

computingCompositions = 1

g_of_Q1 = L*sigma*(Q2 - Q1);
g_of_Q2 = L*(rho*Q1 - Q2 - Q1*Q3); 
g_of_Q3 = L*(Q1*Q2 - beta*Q3);


%g_of_Q1(0,0) = 0;
%g_of_Q2(0,0) = 0;
%g_of_Q3(0,0) = 0;



%stopNow = stopHere

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Composition of DP with L(theta):
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

DP_circL_theta_1 = intval(zeros(N+1, N+1));
DP_circL_theta_2 = intval(zeros(N+1, N+1));
DP_circL_theta_3 = intval(zeros(N+1, N+1));


computingDifferential = 1

for n = 0:N
   for m = 0:N
      if (n+m <= N)
        if ((n == 0) && (m == 0))
            DP_circL_theta_1(1,1) = intval(0.0);
            DP_circL_theta_2(1,1) = intval(0.0);
            DP_circL_theta_3(1,1) = intval(0.0);
        elseif ((n == 0) && (m == 1))
            DP_circL_theta_1(2,1) = lambda_1*P1(2,1);
            DP_circL_theta_2(2,1) = lambda_1*P2(2,1);
            DP_circL_theta_3(2,1) = lambda_1*P3(2,1);
        elseif ((n == 1) && (m == 0)) 
            DP_circL_theta_1(1,2) = lambda_2*P1(1,2);
            DP_circL_theta_2(1,2) = lambda_2*P2(1,2);
            DP_circL_theta_3(1,2) = lambda_2*P3(1,2);
        else
            DP_circL_theta_1(m+1,n+1) = (m*lambda_1+n*lambda_2)*P1(m+1,n+1);
            DP_circL_theta_2(m+1,n+1) = (m*lambda_1+n*lambda_2)*P2(m+1,n+1);
            DP_circL_theta_3(m+1,n+1) = (m*lambda_1+n*lambda_2)*P3(m+1,n+1);
        end
      end
   end
end




[DQ1_Lambda, DQ2_Lambda, DQ3_Lambda] = convertToPoly_2D(...
                                              DP_circL_theta_1, ...
                                              DP_circL_theta_2, ...
                                              DP_circL_theta_3, N);


%Computation of difference = power series of error function:
%Error1 = g_of_P1 - DP_circL_theta_1;
%Error2 = g_of_P2 - DP_circL_theta_2;
%Error3 = g_of_P3 - DP_circL_theta_3;

computingErrorPoly = 1

Error1 = abs(g_of_Q1 - DQ1_Lambda);
Error2 = abs(g_of_Q2 - DQ2_Lambda);
Error3 = abs(g_of_Q3 - DQ3_Lambda);

%Compute the weighted sum-norm for the error functional:
% Item 1 of Definition 4.1 (Validation Values)


error_1 = intval(0);
error_2 = intval(0);
error_3 = intval(0);


boundingError = 1


error_1 = polyval(Error1, sup(domRad), sup(domRad))
error_2 = polyval(Error2, sup(domRad), sup(domRad))
error_3 = polyval(Error3, sup(domRad), sup(domRad))


%stopNow = stopHere

%for m = 0:(2*N)
%    for n = 0:(2*N)
%       if ((m+n <= 2*N)) 
%          norm_a_mn1 = abs(Error1(m,n));
%			norm_a_mn2 = abs(Error2(m,n));
%			norm_a_mn3 = abs(Error3(m,n));
%          thisTerm1 = norm_a_mn1*(domRad)^(m+n);
%          thisTerm2 = norm_a_mn2*(domRad)^(m+n);
%			thisTerm3 = norm_a_mn3*(domRad)^(m+n);
%			error_1 = error_1 + thisTerm1;
%			error_2 = error_2 + thisTerm2;
%			error_3 = error_3 + thisTerm3;
%       end
%    end
%end


error_weightedNorm = max([sup(error_1), sup(error_2), sup(error_3)])



%stopNow = stopHere


%Consider also the weighted norm of P_N itself, with the same weight,
%centered around the fixed point (hence the first term is zero):
% Item 2 of Definition 4.1 (Validation Values)

P_weightedNorm = intval(0);
for m = 0:N
    for n = 0:N
        if ((n == 0) && (m == 0))
          P_weightedNorm = intval(0);  
        else
          norm_a_mn=norm([P1(m+1,n+1),P2(m+1,n+1),P3(m+1,n+1)]);
          thisTerm = norm_a_mn*(domRad)^(m+n);
          P_weightedNorm = P_weightedNorm + thisTerm;
        end
    end
end

epsilon_tol = error_weightedNorm
P_supNorm_bound = P_weightedNorm

rho_prime = P_supNorm_bound
rho = rho_prime + s

C2 = intval(1)

mu_minus = abs(real(lambda_2))
mu_plus = abs(real(lambda_1))

M1 = 4

linCondNum = (N+1)*mu_minus - mu_plus




theSum = 0;
for n = 0:N
    for m = 0:N
        if ((m+n >= 1) & (m+n <= N))
            K_alpha = abs(P1(m+1, n+1)) + ...
                  max([sup(abs(P2(m+1, n+1))), sup(abs(P3(m+1, n+1)))]);
        
            theDenom = abs(lambda_1*m + lambda_2*n);
            theWeight = domRad^(m+n);
            theSum = theSum + (K_alpha*theWeight)/theDenom;
        end
    end
end

theSum

C3 = norm(Q, inf)*norm(Q_inv, inf)*exp(theSum)
L_bound = C3/linCondNum

deltaNum_1 = 2*epsilon_tol*L_bound
deltaNum_2 = min(inf([1/(2*3*exp(1)*intval('pi')*C2*M1*L_bound), ...
    inf(s/exp(intval(0)))]))


if ((inf(linCondNum) > 0) & (sup(deltaNum_1) < deltaNum_2))
   goodComputation = 1
else
    goodComputation = 0
end

goodComputation = goodComputation;
delta = deltaNum_1;














