clear
format long


%Lorenz Parameters:
sigma = 10                          
beta = 8/3
rho = 13.9265
%rho = 28
omega = 0.1



%Parameterization Controls:
M = 5
N = 30
whichEig = 2;


%Fixed point at origin:
p0 = [0;
      0;
      0];
   
  
Dfp0 = [-sigma, sigma,     0;
           rho,    -1,     0;
             0,     0, -beta]

            
[U, Sigma] = eigs(Dfp0);

Sigma


%==========================================================================
%Expansion of Unstable Eigenvalue:
%==========================================================================


%Pick an eigenvalue to expand around:
lambda = Sigma(whichEig,whichEig)

if (abs(lambda + beta) < 5*10^(-6))
   lambdaProblem = stopHere 
end


lambda_series = zeros(1, M+1);
lambda_series(1) = lambda;
theDenom = 2*lambda + sigma + 1;
lambda_series(2) = sigma/theDenom;
 
for n = 2:M
    theSum = 0;
    for k = 1:(n-1)
        theSum = theSum + lambda_series(n-k+1)*lambda_series(k+1);
    end
    lambda_series(n+1) = -theSum/theDenom;
end


lambda_omega = horner(omega, lambda_series(1, :), M)

%Check:
Dfp_omega = [    -sigma, sigma,     0;
              rho+omega,    -1,     0;
                      0,     0, -beta]

            
[U_omega, Sigma_omega] = eigs(Dfp_omega);

Sigma_omega

%Pick an eigenvalue to expand around:
lambda_checkOmega = Sigma_omega(whichEig,whichEig)
lambda_error = abs(lambda_omega - lambda_checkOmega)



%==========================================================================
%Expansion of Unstable Eigenvector:
%==========================================================================


xi_1_series = zeros(1, M+1);
xi_2_series = zeros(1, M+1);
xi_3_series = zeros(1, M+1);
        
xi_1_series(1) = U(1,whichEig);
xi_2_series(1) = U(2,whichEig);


A = [   -(sigma+lambda),            sigma;
       2*xi_1_series(1), 2*xi_2_series(1)];

b = [lambda_series(2)*xi_1_series(1);
                                   0];
                               
xi1 = A\b;                               
   
xi_1_series(2) = xi1(1);
xi_2_series(2) = xi1(2);


for n = 2:M
     
    theSum1 = 0;
    for k = 0:(n-1)
        theSum1 = theSum1 + lambda_series(n-k+1)*xi_1_series(k+1);
    end
    
    theSum2 = 0;
    for k = 1:(n-1)
        theSum2 = theSum2 + xi_1_series(n-k+1)*xi_1_series(k+1) + ...
                            xi_2_series(n-k+1)*xi_2_series(k+1);
    end
    
    
    b = [ theSum1;
         -theSum2];
    xi_n = A\b;
    
    xi_1_series(n+1) = xi_n(1);
    xi_2_series(n+1) = xi_n(2);
end



xi_omega = [horner(omega, xi_1_series(1, :), M);
            horner(omega, xi_2_series(1, :), M);
            0];

xi_error = norm(xi_omega - U_omega(:,whichEig))






%==========================================================================
%Expansion of Stable Eigenvalues:
%==========================================================================



%Pick an eigenvalue to expand around:
lambda_1 = Sigma(1, 1)
lambda_2 = Sigma(3,3)

lambda1_series = zeros(1, M+1);
lambda2_series = zeros(1, M+1);

lambda1_series(1) = lambda_1;
lambda2_series(1) = lambda_2;

theDenom = 2*lambda_1 + sigma + 1;
lambda1_series(2) = sigma/theDenom;
 
for n = 2:M
    theSum = 0;
    for k = 1:(n-1)
        theSum = theSum + lambda1_series(n-k+1)*lambda1_series(k+1);
    end
    lambda1_series(n+1) = -theSum/theDenom;
end


lambda1_omega = horner(omega, lambda1_series(1, :), M)
lambda2_omega = horner(omega, lambda2_series(1, :), M)


%Check:
Dfp_omega = [    -sigma, sigma,     0;
              rho+omega,    -1,     0;
                      0,     0, -beta]

            
[U_omega, Sigma_omega] = eigs(Dfp_omega);

Sigma_omega

%Pick an eigenvalue to expand around:
lambda1_checkOmega = Sigma_omega(1,1)
lambda1_error = abs(lambda1_omega - lambda1_checkOmega)

lambda2_checkOmega = Sigma_omega(3,3)
lambda2_error = abs(lambda2_omega - lambda2_checkOmega)






%==========================================================================
%Expansion of Stable Eigenvectors:
%==========================================================================




xi1_1_series = zeros(1, M+1);
xi1_2_series = zeros(1, M+1);
xi1_3_series = zeros(1, M+1);
   
xi2_1_series = zeros(1, M+1);
xi2_2_series = zeros(1, M+1);
xi2_3_series = zeros(1, M+1);


xi1_1_series(1) = U(1,1);
xi1_2_series(1) = U(2,1);

xi2_3_series(1) = U(3,3);



A = [   -(sigma+lambda_1),             sigma;
        2*xi1_1_series(1), 2*xi1_2_series(1)];

b = [lambda1_series(2)*xi1_1_series(1);
                                   0];
                               
xi1 = A\b;                               
   
xi1_1_series(2) = xi1(1);
xi1_2_series(2) = xi1(2);


for n = 2:M
     
    theSum1 = 0;
    for k = 0:(n-1)
        theSum1 = theSum1 + lambda1_series(n-k+1)*xi1_1_series(k+1);
    end
    
    theSum2 = 0;
    for k = 1:(n-1)
        theSum2 = theSum2 + xi1_1_series(n-k+1)*xi1_1_series(k+1) + ...
                            xi1_2_series(n-k+1)*xi1_2_series(k+1);
    end
    
    
    b = [ theSum1;
         -theSum2];
    xi_n = A\b;
    
    xi1_1_series(n+1) = xi_n(1);
    xi1_2_series(n+1) = xi_n(2);
end



xi1_omega = [horner(omega, xi1_1_series(1, :), M);
             horner(omega, xi1_2_series(1, :), M);
             horner(omega, xi1_3_series(1, :), M)];

xi1_error = norm(xi1_omega - U_omega(:,1))

xi2_omega = [horner(omega, xi2_1_series(1, :), M);
             horner(omega, xi2_2_series(1, :), M);
             horner(omega, xi2_3_series(1, :), M)];

xi2_error = norm(xi2_omega - U_omega(:,3))


%==========================================================================
%Compute 1D and 2D manifold coefficients at omega = 0:
%==========================================================================


xi_lambda = U(:,whichEig)


lambda_1 = Sigma(1,1);
lambda_2 = Sigma(3,3);

xi_1 = U(:, 1);
xi_2 = U(:, 3);


[PN1, PN2, PN3] = lorenz_1D_manifold(p0, lambda, xi_lambda, ...
                                      sigma, rho, beta, N, 1);

                                  
%[goodComputation, delta0] = lorenzApostori_1D(N, 1, sigma, rho, beta, ...
%                              domRad0,PN1,PN2,PN3, lambda);                             
%delta0_u                    


[QN1, QN2, QN3] = lorenz_2D_manifold(...
                            p0, lambda_1, lambda_2, xi_1, xi_2, ...
                            sigma, rho, beta, N, 1);


                        
                    
                        
%==========================================================================
%Compute rho partial of PN at omega = 0 (i.e. first order term in omega).
%==========================================================================



[PN1_rho, PN2_rho, PN3_rho] = lorenz_1D_rhoPartial_at0(...
                                      p0, lambda, xi_lambda, ...
                                      PN1, PN2, PN3, ...
                                      sigma, rho, beta, N, 1);



[QN1_rho, QN2_rho, QN3_rho] = lorenz_2D_rhoPartial_at0(...
                                   p0, lambda_1, lambda_2, ...
                                   xi_1, xi_2, ...
                                   QN1, QN2, QN3, ...
                                   sigma, rho, beta, N, 1);                                 
                                  
                           
                                
                                  
%==========================================================================
%Compute 1D manifold parameter expansion in omega:
%==========================================================================


PMN1 = zeros(M+1, N+1);
PMN2 = zeros(M+1, N+1);
PMN3 = zeros(M+1, N+1);

PMN1(1, :) = PN1;
PMN2(1, :) = PN2;
PMN3(1, :) = PN3;

PMN1(2, :) = PN1_rho;
PMN2(2, :) = PN2_rho;
PMN3(2, :) = PN3_rho;

PMN1(3:M+1, 2) = xi_1_series(3:M+1)';
PMN2(3:M+1, 2) = xi_2_series(3:M+1)';
PMN3(3:M+1, 2) = xi_3_series(3:M+1)';

Id = eye(3);

for n = 2:N
    for m = 2:M
        A = Dfp0 - n*lambda*Id;
         
        thisTerm1 = 0;
        thisTerm2 = 0;
        thisTerm3 = 0;
        for l = 0:(m-1)
            thisTerm1 = thisTerm1 + lambda_series(m-l+1)*PMN1(l+1, n+1);
            thisTerm2 = thisTerm2 + lambda_series(m-l+1)*PMN2(l+1, n+1);
            thisTerm3 = thisTerm3 + lambda_series(m-l+1)*PMN3(l+1, n+1);
        end
        
        smn_term = ...
            convolution_smn(m,n,PMN1,PMN2,PMN3, sigma, rho, beta, M, N);
        
        smn1 = n*thisTerm1;
        smn2 = n*thisTerm2 + smn_term(2) - PMN1(m, n+1);
        smn3 = n*thisTerm3 - smn_term(3);
        
        b = [smn1;
             smn2;
             smn3];
         
        thisCoeff = A\b;
        
        PMN1(m+1, n+1) = thisCoeff(1);
        PMN2(m+1, n+1) = thisCoeff(2);
        PMN3(m+1, n+1) = thisCoeff(3);
      
    end
end




    
%Check error in 1D expansions:
%Dfp_omega = [    -sigma, sigma,     0;
%              rho+omega,    -1,     0;
%                      0,     0, -beta]
%            
%[U_omega, Sigma_omega] = eigs(Dfp_omega);
%
%Sigma_omega
%omega
%
%[PN1_omega, PN2_omega, PN3_omega] = ...
%    lorenz_1D_manifold(p0, Sigma_omega(whichEig, whichEig), ...
%                       U_omega(:, whichEig), ...
%                       sigma, rho+omega, beta, N, 1);
%
%n = 8   
%                   
%an_omega = horner(omega, PMN1(:, n), M)                  
%PN1_omega(n)
%                   
%a_error = abs(an_omega - PN1_omega(n))

                                
%==========================================================================
%Compute 2D manifold parameter expansion in omega:
%==========================================================================


QMN1 = zeros(M+1, N+1, N+1);
QMN2 = zeros(M+1, N+1, N+1);
QMN3 = zeros(M+1, N+1, N+1);

QMN1(1, :, :) = QN1(:, :);
QMN2(1, :, :) = QN2(:, :);
QMN3(1, :, :) = QN3(:, :);

QMN1(2, :, :) = QN1_rho(:, :);
QMN2(2, :, :) = QN2_rho(:, :);
QMN3(2, :, :) = QN3_rho(:, :);

QMN1(3:M+1, 2, 1) = xi1_1_series(3:M+1);
QMN2(3:M+1, 2, 1) = xi1_2_series(3:M+1);
QMN3(3:M+1, 2, 1) = xi1_3_series(3:M+1);

QMN1(3:M+1, 1, 2) = xi2_1_series(3:M+1);
QMN2(3:M+1, 1, 2) = xi2_2_series(3:M+1);
QMN3(3:M+1, 1, 2) = xi2_3_series(3:M+1);

Id = eye(3);

for n1 = 0:N
    for n2 = 0:N
        for m = 2:M
            if ((n1+n2 >= 2) & (n1+n2 <= N))
                A = Dfp0 - (n1*lambda_1 + n2*lambda_2)*Id;
         
                oneTerm1 = 0;
                oneTerm2 = 0;
                oneTerm3 = 0;
        
                for k = 0:(m-1)
                    lambda_coeff = n1*lambda1_series(m-k+1) + ...
                                   n2*lambda2_series(m-k+1);
                    oneTerm1 = oneTerm1 + ...
                        lambda_coeff*QMN1(k+1, n1+1, n2+1);
                    oneTerm2 = oneTerm2 + ...
                        lambda_coeff*QMN2(k+1, n1+1, n2+1);
                    oneTerm3 = oneTerm3 + ...
                        lambda_coeff*QMN3(k+1, n1+1, n2+1);
                end
        
                smn1n2_term = ...
                    convolution_smn1n2(...
                        m,n1,n2,QMN1,QMN2,QMN3, ...
                        sigma, rho, beta, M, N);
        
                smn1n2_1 = oneTerm1;
                smn1n2_2 = oneTerm2 + ...
                           smn1n2_term(2) - QMN1(m, n1+1, n2+1);
                smn1n2_3 = oneTerm3 - ...
                           smn1n2_term(3);
        
                b = [smn1n2_1;
                     smn1n2_2;
                     smn1n2_3];
         
                thisCoeff = A\b;
        
                QMN1(m+1, n1+1, n2+1) = thisCoeff(1);
                QMN2(m+1, n1+1, n2+1) = thisCoeff(2);
                QMN3(m+1, n1+1, n2+1) = thisCoeff(3);
        
            end
        end
    end
end

                 


%Check:

Dfp_omega = [    -sigma, sigma,     0;
             rho+omega,    -1,     0;
                     0,     0, -beta]
           
[U_omega, Sigma_omega] = eigs(Dfp_omega);

Sigma_omega
omega

[QN1_omega, QN2_omega, QN3_omega] = lorenz_2D_manifold(...
                            p0, Sigma_omega(1,1), Sigma_omega(3,3), ...
                            U_omega(:,1), U_omega(:, 3), ...
                            sigma, rho+omega, beta, N, 1);
 
                        
n1 = 2;
n2 = 3;

an1n2_omega = horner(omega, QMN3(:, n1+1, n2+1), M)     
QN3_omega(n1+1, n2+1)
                  
an1n2_error = abs(an1n2_omega - QN3_omega(n1+1, n2+1))















