%==========================================================================
%REU Tutorial:  (Example Four)
%
%ode_example.m
%
%Solving Ordinary Differential equations.  Plotting the results
%
%Dependency: This program depends on the file 'lorenzField.m'
%==========================================================================



clear
format long


sigma = 10                          
beta = 8/3
rho = 99.96;

%Integraton: Integrate out the Transients
%numSteps = 500
%tspan = linspace(0, 50, numSteps);  %Time axis
%y0 = [0.5; 1.0; 0.2]                %initial conditions
%options=odeset('RelTol',1e-12,'AbsTol',1e-8);
%[t,y] = ode45('lorenzField', tspan, y0, options, flag, sigma, rho, beta);

%Point on the attractor:
%y0 = y(500, :)

%Or hard code the initial condition:
y0 = [4.353321509483964  -5.958696195762751  88.162194001623832]
numSteps = 5000   %Set this to be what ever you want for num of 
                  %sample points.
T_guess = 1.0995                  
tspan = linspace(0, T_guess, numSteps);

options=odeset('RelTol',1e-12,'AbsTol',1e-8);
[t,y] = ode113('lorenzField', tspan, y0, options, flag, sigma, rho, beta);



%Save Data:
%save('lorenz.mat');

%To recover data type
%>> load lorenz.mat


%State Variable Plots: 
figure
hold on
plot(t(:), y(:,1), 'r')
plot(t(:), y(:,2), 'b')
plot(t(:), y(:,3), 'g')
xlabel('time axis','FontSize',8)
ylabel('state variables','FontSize',8)
title('time series for the Lorenz Systems','FontSize',10)
h = legend('x', 'y', 'z', 3)

%Phase Space Plot
figure
hold on
plot3(y(:,1), y(: ,2), y(:,3)) 
plot3(y(1,1), y(1,2), y(1,3), 'g*')
plot3(y(numSteps,1), y(numSteps,2), y(numSteps,3), 'ko')
xlabel('x-axis','FontSize',8)
ylabel('y-axis','FontSize',8)
zlabel('z-axis','FontSize',8)
title('State Space of the Lorenz Systems','FontSize',10)



