function varargout=lorenz_1D_rhoPartial_at0_intval(p, lambda, xi, ...
                                      P1, P2, P3, sigma, rho, beta, N, L);

%Differential at the fixed point and it's rho partial.                                  
Dfp = lorenzDifferential(p, sigma, rho, beta, L);

Dfp_rhoPartial = L*intval([0, 0, 0;
                           1, 0, 0;
                           0, 0, 0]);

%rho partials of first order data.
lambda_rhoPartial = rhoPartial_of_eigVal_intval(lambda, sigma, rho, beta, L);  

xi_rhoPartial =  rhoPartial_of_eigVect_intval(p, lambda, xi, sigma, rho, beta, L);

%Initialize coefficient storage.
P1_rho = intval(zeros(1, N+1));
P2_rho = intval(zeros(1, N+1));
P3_rho = intval(zeros(1, N+1));
  
%Assign first order coefficients:

%The orgin is a fixed point for all rho, then the partial of
%the fixed point with respect to rho vanishes.
P1_rho(1,1) = intval(0);
P2_rho(1,1) = intval(0);
P3_rho(1,1) = intval(0);

%First order coefficients are rho partials of eigenvectors.
P1_rho(1,2) = xi_rhoPartial(1);
P2_rho(1,2) = xi_rhoPartial(2);
P3_rho(1,2) = xi_rhoPartial(3);

Id = intval(eye(3));

%Compute b_mn coefficient for m+n >= 2.
for n = 2:N
    An = Dfp - n*lambda*Id;            
    a_n = [P1(1, n+1);
           P2(1, n+1);
           P3(1, n+1)];
                
    sn_rho = L*compute_sn_rhoPartial_intval(n, P1, P2, P3, ...
                      P1_rho, P2_rho, P3_rho, sigma, rho, beta, N);
              
    b_n = -[Dfp_rhoPartial - n*lambda_rhoPartial*Id]*a_n - sn_rho;               
                          
    vn = An\b_n;
    P1_rho(1, n+1) = vn(1);
    P2_rho(1, n+1) = vn(2);
    P3_rho(1, n+1) = vn(3);
end

varargout(1,1)={P1_rho};
varargout(2,1)={P2_rho};
varargout(3,1)={P3_rho};


