function varargout=lorenz_2D_rhoPartial_at0(p, lambda_1, lambda_2, xi_1, xi_2, ...
                                      P1, P2, P3, sigma, rho, beta, N, L);

%Differential at the fixed point and it's rho partial.                                  
Dfp = lorenzDifferential(p, sigma, rho, beta, L);

Dfp_rhoPartial = L*[0, 0, 0;
                    1, 0, 0;
                    0, 0, 0];

%rho partials of first order data.
lambda_1rhoPartial = rhoPartial_of_eigVal(lambda_1, sigma, rho, beta, L);  
lambda_2rhoPartial = rhoPartial_of_eigVal(lambda_2, sigma, rho, beta, L); 

xi_1rhoPartial =  rhoPartial_of_eigVect(p, lambda_1, xi_1, sigma, rho, beta, L);
xi_2rhoPartial =  rhoPartial_of_eigVect(p, lambda_2, xi_2, sigma, rho, beta, L);

%Initialize coefficient storage.
P1_rho = zeros(N+1, N+1);
P2_rho = zeros(N+1, N+1);
P3_rho = zeros(N+1, N+1);
  
%Assign first order coefficients:

%The orgin is a fixed point for all rho, then the partial of
%the fixed point with respect to rho vanishes.
P1_rho(1,1) = 0;
P2_rho(1,1) = 0;
P3_rho(1,1) = 0;

%First order coefficients are rho partials of eigenvectors.
P1_rho(2,1) = xi_1rhoPartial(1);
P2_rho(2,1) = xi_1rhoPartial(2);
P3_rho(2,1) = xi_1rhoPartial(3);

P1_rho(1,2) = xi_2rhoPartial(1);
P2_rho(1,2) = xi_2rhoPartial(2);
P3_rho(1,2) = xi_2rhoPartial(3);

Id = eye(3);

%Compute b_mn coefficient for m+n >= 2.
for n = 0:N
    for m = 0:N
        if ((m+n >= 2) & (m+n <= N))
            Amn = Dfp - (m*lambda_1 + n*lambda_2)*Id;            
            a_mn = [P1(m+1, n+1);
                    P2(m+1, n+1);
                    P3(m+1, n+1)];
                
            smn_rho = L*compute_smn_rhoPartial(m, n, P1, P2, P3, ...
                              P1_rho, P2_rho, P3_rho, sigma, rho, beta, N);

            LambdaRho_dot_mn = m*lambda_1rhoPartial + n*lambda_2rhoPartial;              
            b_mn = -[Dfp_rhoPartial - LambdaRho_dot_mn*Id]*a_mn - smn_rho;               
                          
            vmn = Amn\b_mn;
            P1_rho(m+1, n+1) = vmn(1);
            P2_rho(m+1, n+1) = vmn(2);
            P3_rho(m+1, n+1) = vmn(3);
        end
    end
end

varargout(1,1)={P1_rho};
varargout(2,1)={P2_rho};
varargout(3,1)={P3_rho};


