function varargout=lorenz_2D_rhoPartial_at0_intval_vectorized(...
                                      p, lambda_1, lambda_2, xi_1, xi_2, ...
                                      P1, P2, P3, sigma, rho, beta, N, L);

%Differential at the fixed point and it's rho partial.                                  
Dfp = lorenzDifferential_intval(p, sigma, rho, beta, L);

Dfp_rhoPartial = L*intval([0, 0, 0;
                           1, 0, 0;
                           0, 0, 0]);

%rho partials of first order data.
lambda_1rhoPartial = rhoPartial_of_eigVal_intval(...
                                   lambda_1, sigma, rho, beta, L);  
lambda_2rhoPartial = 0; 


xi_1rhoPartial =  rhoPartial_of_eigVect_intval(...
                                   p, lambda_1, xi_1, sigma, rho, beta, L);
xi_2rhoPartial =  intval(zeros(3,1));

%Initialize coefficient storage.
P1_rho = intval(zeros(N+1, N+1));
P2_rho = intval(zeros(N+1, N+1));
P3_rho = intval(zeros(N+1, N+1));
  
%Assign first order coefficients:

%The orgin is a fixed point for all rho, then the partial of
%the fixed point with respect to rho vanishes.
P1_rho(1,1) = intval(0);
P2_rho(1,1) = intval(0);
P3_rho(1,1) = intval(0);

%First order coefficients are rho partials of eigenvectors.
P1_rho(2,1) = xi_1rhoPartial(1);
P2_rho(2,1) = xi_1rhoPartial(2);
P3_rho(2,1) = xi_1rhoPartial(3);

P1_rho(1,2) = xi_2rhoPartial(1);
P2_rho(1,2) = xi_2rhoPartial(2);
P3_rho(1,2) = xi_2rhoPartial(3);

Id = intval(eye(3));

%Compute b_mn coefficient for m+n >= 2.
for n1 = 2:N
    for m1 = 0:n1
        
       n  = n1 - m1;
       m = m1;
       Amn = Dfp - (m*lambda_1 + n*lambda_2)*Id;            
       a_mn = [P1(m+1, n+1);
               P2(m+1, n+1);
               P3(m+1, n+1)];
                                        
        %Compute s_mn: (vectorized version)
        A1 = reshape(P1(1:m+1, 1:n+1), (n+1)*(m+1), 1);
        A1 = A1(2:((n+1)*(m+1)-1));
        
        A2 = reshape(P2(1:m+1, 1:n+1), (n+1)*(m+1), 1);
        A2 = A2(2:((n+1)*(m+1)-1));
        
        A3 = reshape(P3(1:m+1, 1:n+1), (n+1)*(m+1), 1);
        A3 = A3(2:((n+1)*(m+1)-1));
        
        B1 = reshape(P1_rho(1:m+1, 1:n+1), (n+1)*(m+1), 1);
        B1 = B1(2:((n+1)*(m+1)-1));
        
        B2 = reshape(P2_rho(1:m+1, 1:n+1), (n+1)*(m+1), 1);
        B2 = B2(2:((n+1)*(m+1)-1));
        
        B3 = reshape(P3_rho(1:m+1, 1:n+1), (n+1)*(m+1), 1);
        B3 = B3(2:((n+1)*(m+1)-1));
        
        B1 = flipud(B1);
        A1 = flipud(A1);
        
        theValue_13 = dot(B1, A3) + dot(A1, B3);
        theValue_12 = dot(B1, A2) + dot(A1, B2);              
        
        smn_rho = L*[  intval(0);
                    -theValue_13;
                     theValue_12];
        
        LambdaRho_dot_mn = m*lambda_1rhoPartial + n*lambda_2rhoPartial;              
        b_mn = -[Dfp_rhoPartial - LambdaRho_dot_mn*Id]*a_mn - smn_rho;               
                          
        vmn = Amn\b_mn;
        P1_rho(m+1, n+1) = vmn(1);
        P2_rho(m+1, n+1) = vmn(2);
        P3_rho(m+1, n+1) = vmn(3);
    end
end

varargout(1,1)={P1_rho};
varargout(2,1)={P2_rho};
varargout(3,1)={P3_rho};


