function varargout=lorenz_2D_stableBranch_at0_intval_vectorized(...
    QN1, QN2, QN3, QN1_rho, QN2_rho, QN3_rho, ...
    lambda1_series, lambda2_series, ...
    xi1_1_series, xi1_2_series, xi1_3_series, ...
    xi2_1_series, xi2_2_series, xi2_3_series, ...
    M, N, sigma, beta, rho);


QMN1 = intval(zeros(M+1, N+1, N+1));
QMN2 = intval(zeros(M+1, N+1, N+1));
QMN3 = intval(zeros(M+1, N+1, N+1));

QMN1(1, :, :) = QN1;
QMN2(1, :, :) = QN2;
QMN3(1, :, :) = QN3;

QMN1(2, :, :) = QN1_rho;
QMN2(2, :, :) = QN2_rho;
QMN3(2, :, :) = QN3_rho;

QMN1(3:M+1, 2, 1) = xi1_1_series(3:M+1)';
QMN2(3:M+1, 2, 1) = xi1_2_series(3:M+1)';
QMN3(3:M+1, 2, 1) = xi1_3_series(3:M+1)';

QMN1(3:M+1, 1, 2) = xi2_1_series(3:M+1)';
QMN2(3:M+1, 1, 2) = xi2_2_series(3:M+1)';
QMN3(3:M+1, 1, 2) = xi2_3_series(3:M+1)';



Dfp0 = [    -sigma,         sigma,     intval(0);
               rho,    intval(-1),     intval(0);
         intval(0),     intval(0),          -beta];
Id = intval(eye(3));


lambda1 = lambda1_series(1);
lambda2 = lambda2_series(1);

for K1 = 2:N
    K1
	for K2 = 0:K1
        n1 = K1-K2;
        n2 = K2;
        homMat = Dfp0 - (n1*lambda1 + n2*lambda2)*Id;
			for m = 2:M
				
            %Compute lambda term: (vectorized version)
                
            A1 = n1*lambda1_series(2:m+1)' + n2*lambda2_series(2:m+1)';       
            A1 = flipud(A1);
        
            B1 = QMN1(1:m, n1+1, n2+1);
            B2 = QMN2(1:m, n1+1, n2+1);
            B3 = QMN3(1:m, n1+1, n2+1);
       
            thisTerm1 = dot(A1, B1);
            thisTerm2 = dot(A1, B2);
            thisTerm3 = dot(A1, B3);
         
            %vectorized computation of the convolution term: 
        
            A = reshape(QMN1(1:m+1, 1:n1+1, 1:n2+1), (n1+1)*(n2+1)*(m+1), 1);
            A = A(2:((n1+1)*(n2+1)*(m+1)-1));

            B = reshape(QMN3(1:m+1, 1:n1+1, 1:n2+1), (n1+1)*(n2+1)*(m+1), 1);
            B = B(2:((n1+1)*(n2+1)*(m+1)-1));
            B = flipud(B);

            theValue_13 = dot(A, B);

            B = reshape(QMN2(1:m+1, 1:n1+1, 1:n2+1), (n1+1)*(n2+1)*(m+1), 1);
            B = B(2:((n1+1)*(n2+1)*(m+1)-1));
            B = flipud(B);

            theValue_12 = dot(A, B);
            
            %Solution of the homological equatoin:
            b = [ thisTerm1;
                  thisTerm2 + theValue_13 - QMN1(m, n1+1, n2+1);
                  thisTerm3 - theValue_12];
                
            thisCoeff = homMat\b;
        
            QMN1(m+1, n1+1, n2+1) = thisCoeff(1);
            QMN2(m+1, n1+1, n2+1) = thisCoeff(2);
            QMN3(m+1, n1+1, n2+1) = thisCoeff(3);
        end
	end
end
                                  
varargout(1,1)={QMN1};
varargout(2,1)={QMN2};
varargout(3,1)={QMN3};
