clear
format long


%Lorenz Parameters:
sigma = 10                          
beta = 8/3
rho = 13.9265
%rho = 28

%Parameter Domains:
omega = 1.5
domRad0 = 1.0; 


%Parameterization Controls:
M = 15
N = 30
whichEig = 2;


%Fixed point at origin:
p0 = [0;
      0;
      0];
   
  
Dfp0 = [-sigma, sigma,     0;
           rho,    -1,     0;
             0,     0, -beta]

            
[U, Sigma] = eigs(Dfp0);

Sigma


%==========================================================================
%Expansion of Eigenvalue:
%==========================================================================


%Pick an eigenvalue to expand around:
lambda = Sigma(whichEig,whichEig)

lambda_test = (-(1+sigma) - sqrt(sigma^2 - 2*sigma + 1 + 4*sigma*rho))/2
lambda_error = abs(lambda - lambda_test)
%lambda = lambda_test


if (abs(lambda + beta) < 5*10^(-6))
   lambdaProblem = stopHere 
end


lambda_series = zeros(1, M+1);
lambda_series(1) = lambda;
theDenom = 2*lambda + sigma + 1;
lambda_series(2) = sigma/theDenom;
 
for n = 2:M
    theSum = 0;
    for k = 1:(n-1)
        theSum = theSum + lambda_series(n-k+1)*lambda_series(k+1);
    end
    lambda_series(n+1) = -theSum/theDenom;
end


%==========================================================================
%Eigenvalue Residual Error:
%==========================================================================


lambda_squared = convolveSeries_1D_general(lambda_series, lambda_series);


big_lambda = zeros(1, 2*M+1);
Error = zeros(1, 2*M+1);

big_lambda(1, 1:M+1) = lambda_series;

Error = lambda_squared + (1+sigma)*big_lambda;

Error(1) = Error(1) + sigma*(1-rho);
Error(2) = Error(2) - sigma;

sigma_error = 0;
for m = 0:(2*M)
   sigma_error = sigma_error + abs(Error(m+1))*abs(omega)^m; 
end

sigma_error




%==========================================================================
%Expansion of Eigenvector:
%==========================================================================




xi_1_series = zeros(1, M+1);
xi_2_series = zeros(1, M+1);
xi_3_series = zeros(1, M+1);
        
xi_1_series(1) = U(1,whichEig);
xi_2_series(1) = U(2,whichEig);


A = [   -(sigma+lambda),            sigma;
       2*xi_1_series(1), 2*xi_2_series(1)];

b = [lambda_series(2)*xi_1_series(1);
                                   0];
                               
xi1 = A\b;                               
   
xi_1_series(2) = xi1(1);
xi_2_series(2) = xi1(2);


for n = 2:M
     
    theSum1 = 0;
    for k = 0:(n-1)
        theSum1 = theSum1 + lambda_series(n-k+1)*xi_1_series(k+1);
    end
    
    theSum2 = 0;
    for k = 1:(n-1)
        theSum2 = theSum2 + xi_1_series(n-k+1)*xi_1_series(k+1) + ...
                            xi_2_series(n-k+1)*xi_2_series(k+1);
    end
    
    
    b = [ theSum1;
         -theSum2];
    xi_n = A\b;
    
    xi_1_series(n+1) = xi_n(1);
    xi_2_series(n+1) = xi_n(2);
end



%==========================================================================
%Eigenvectior Residual Error:
%==========================================================================



big_xi1 = zeros(1, 2*M+1);
big_xi2 = zeros(1, 2*M+1);

big_xi1(1:M+1) = xi_1_series;
big_xi2(1:M+1) = xi_2_series;

xi1_squared = convolveSeries_1D_general(xi_1_series, xi_1_series);
xi2_squared = convolveSeries_1D_general(xi_2_series, xi_2_series);
xi1_times_lambda = convolveSeries_1D_general(xi_1_series, lambda_series);

Error_1 = -sigma*big_xi1 + sigma*big_xi2 - xi1_times_lambda;
Error_2 = xi1_squared + xi2_squared;

Error_2(1) = Error_2(1) - 1;


sigmaError1 = 0;
sigmaError2 = 0;

for m = 0:2*M
    sigmaError1 = sigmaError1 + abs(Error_1(m+1))*abs(omega)^m;
    sigmaError2 = sigmaError2 + abs(Error_2(m+1))*abs(omega)^m; 
end


sigmaError_eigVect = max(sigmaError1, sigmaError2)



%==========================================================================
%Compute 1D manifold coefficients at omega = 0:
%==========================================================================


xi_lambda = U(:,whichEig)


[PN1, PN2, PN3] = lorenz_1D_manifold(p0, lambda, xi_lambda, ...
                                      sigma, rho, beta, N, 1);

 
                            
                                  
%==========================================================================                                  
%1D Manifold Residual:     
%==========================================================================

                                 
                                  
[goodComputation, delta0] = lorenzApostori_1D(N, 1, sigma, rho, beta, ...
                              domRad0,PN1,PN2,PN3, lambda);                             
delta0  


                       
%==========================================================================
%Compute rho partial of 1D PN at omega = 0 (i.e. first order term in omega).
%==========================================================================



[PN1_rho, PN2_rho, PN3_rho] = lorenz_1D_rhoPartial_at0(...
                                      p0, lambda, xi_lambda, ...
                                      PN1, PN2, PN3, ...
                                      sigma, rho, beta, N, 1);






%==========================================================================
%Omega Partial at omega = 0 Residuial:
%==========================================================================



P3Pomega1 = convolveSeries_1D_general(PN3, PN1_rho);
P1Pomega3 = convolveSeries_1D_general(PN1, PN3_rho);

P2Pomega1 = convolveSeries_1D_general(PN2, PN1_rho);
P1Pomega2 = convolveSeries_1D_general(PN1, PN2_rho);


P1_big = zeros(1, 2*N+1);
P2_big = zeros(1, 2*N+1);
P3_big = zeros(1, 2*N+1);

Pomega1_big = zeros(1, 2*N+1);
Pomega2_big = zeros(1, 2*N+1);
Pomega3_big = zeros(1, 2*N+1);

differentialTerm1 = zeros(1, 2*N+1);
differentialTerm2 = zeros(1, 2*N+1);
differentialTerm3 = zeros(1, 2*N+1);


P1_big(1:N+1) = PN1;
P2_big(1:N+1) = PN2;
P3_big(1:N+1) = PN3;

Pomega1_big(1:N+1) = PN1_rho;
Pomega2_big(1:N+1) = PN2_rho;
Pomega3_big(1:N+1) = PN3_rho;


for n = 0:N
    differentialTerm1(n+1) = n*(lambda_series(2)*PN1(n+1) + ...
                                lambda_series(1)*PN1_rho(n+1));
    differentialTerm2(n+1) = n*(lambda_series(2)*PN2(n+1) + ...
                                lambda_series(1)*PN2_rho(n+1));
    differentialTerm3(n+1) = n*(lambda_series(2)*PN3(n+1) + ...
                                lambda_series(1)*PN3_rho(n+1));
end


Error_rhoPartial_1 = -sigma*Pomega1_big + sigma*Pomega2_big + ...
                     -differentialTerm1;
 
                 
Error_rhoPartial_2 = rho*Pomega1_big - P3Pomega1 - P1Pomega3 - ...
                     Pomega2_big + P1_big - differentialTerm2;

Error_rhoPartial_3 = P2Pomega1 + P1Pomega2 - beta*Pomega3_big - ...
                     differentialTerm3;
                 
thisSum1 = 0;
thisSum2 = 0;
thisSum3 = 0;

for n = 0:2*N
    thisSum1 = thisSum1 + abs(Error_rhoPartial_1(n+1))*domRad0^n;
    thisSum2 = thisSum2 + abs(Error_rhoPartial_2(n+1))*domRad0^n;
    thisSum3 = thisSum3 + abs(Error_rhoPartial_3(n+1))*domRad0^n;
end
                               
rhoPartial_sigmaError1 = max([thisSum1, thisSum2, thisSum3])


%==========================================================================
%Compute 1D manifold parameter expansion in omega:
%==========================================================================


PMN1 = zeros(M+1, N+1);
PMN2 = zeros(M+1, N+1);
PMN3 = zeros(M+1, N+1);

PMN1(1, :) = PN1;
PMN2(1, :) = PN2;
PMN3(1, :) = PN3;

PMN1(2, :) = PN1_rho;
PMN2(2, :) = PN2_rho;
PMN3(2, :) = PN3_rho;

PMN1(3:M+1, 2) = xi_1_series(3:M+1)';
PMN2(3:M+1, 2) = xi_2_series(3:M+1)';
PMN3(3:M+1, 2) = xi_3_series(3:M+1)';

Id = eye(3);

for n = 2:N
    for m = 2:M
        A = Dfp0 - n*lambda*Id;
         
        thisTerm1 = 0;
        thisTerm2 = 0;
        thisTerm3 = 0;
        for l = 0:(m-1)
            thisTerm1 = thisTerm1 + lambda_series(m-l+1)*PMN1(l+1, n+1);
            thisTerm2 = thisTerm2 + lambda_series(m-l+1)*PMN2(l+1, n+1);
            thisTerm3 = thisTerm3 + lambda_series(m-l+1)*PMN3(l+1, n+1);
        end
        
        smn_term = ...
            convolution_smn(m,n,PMN1,PMN2,PMN3, sigma, rho, beta, M, N);
        
        smn1 = n*thisTerm1;
        smn2 = n*thisTerm2 + smn_term(2) - PMN1(m, n+1);
        smn3 = n*thisTerm3 - smn_term(3);
        
        b = [smn1;
             smn2;
             smn3];
         
        thisCoeff = A\b;
        
        PMN1(m+1, n+1) = thisCoeff(1);
        PMN2(m+1, n+1) = thisCoeff(2);
        PMN3(m+1, n+1) = thisCoeff(3);
      
    end
end




%==========================================================================
%Omega Expansion 1D Residual Error:
%==========================================================================


PMN1_big = zeros(2*M+1, 2*N+1); 
PMN2_big = zeros(2*M+1, 2*N+1); 
PMN3_big = zeros(2*M+1, 2*N+1); 

P1omega = zeros(2*M+1, 2*N+1);

differentialTerm1 = zeros(2*M+1, 2*N+1);
differentialTerm2 = zeros(2*M+1, 2*N+1);
differentialTerm3 = zeros(2*M+1, 2*N+1);

PMN1_big(1:M+1, 1:N+1) = PMN1;
PMN2_big(1:M+1, 1:N+1) = PMN2;
PMN3_big(1:M+1, 1:N+1) = PMN3;

PMN13 = convolveSeries_2D_general(PMN1,PMN3);
PMN12 = convolveSeries_2D_general(PMN1,PMN2);


for n = 0:N
    this_amn1 = zeros(1, M+1);
    this_amn1 = n*PMN1(:, n+1)';
    
    this_amn2 = zeros(1, M+1);
    this_amn2 = n*PMN2(:, n+1)';
    
    this_amn3 = zeros(1, M+1);
    this_amn3 = n*PMN3(:, n+1)';
    
    thisDiffTerm1 = convolveSeries_1D_general(this_amn1, lambda_series);
    thisDiffTerm2 = convolveSeries_1D_general(this_amn2, lambda_series);
    thisDiffTerm3 = convolveSeries_1D_general(this_amn3, lambda_series);

    differentialTerm1(:, n+1) = thisDiffTerm1;
    differentialTerm2(:, n+1) = thisDiffTerm2;
    differentialTerm3(:, n+1) = thisDiffTerm3;
end



for n = 0:N
    for m = 1:M
        P1omega(m+1, n+1) = PMN1(m, n+1); 
    end
end



Error_omega_1 = -sigma*PMN1_big + sigma*PMN2_big - differentialTerm1; 

Error_omega_2 = rho*PMN1_big + P1omega - PMN13 - ...
                               PMN2_big - differentialTerm2;

Error_omega_3 = PMN12 - beta*PMN3_big - differentialTerm3;

thisSum1 = 0;
thisSum2 = 0;
thisSum3 = 0;

for m = 0:2*M
    for n = 0:2*N
        thisSum1 = thisSum1 + ...
                   abs(Error_omega_1(m+1, n+1))*(abs(omega)^m)*(domRad0)^n;
        thisSum2 = thisSum2 + ...
                   abs(Error_omega_2(m+1, n+1))*(abs(omega)^m)*(domRad0)^n;
        thisSum3 = thisSum3 + ...
                   abs(Error_omega_3(m+1, n+1))*(abs(omega)^m)*(domRad0)^n;
    end
end

expansion_sigmaError = max([thisSum1, thisSum2, thisSum3])


























