

clear
format long


%Lorenz Parameters:
sigma = 10                          
beta = 8/3
rho = 13.9265
%rho = 28

%Parameter Domains:
omega = 0.1
domRad0 = 0.5; 


%Parameterization Controls:
M = 5
N = 10
whichEig = 1;


%Fixed point at origin:
p0 = [0;
      0;
      0];
   
  
Dfp0 = [-sigma, sigma,     0;
           rho,    -1,     0;
             0,     0, -beta]

            
[U, Sigma] = eigs(Dfp0);

Sigma


%==========================================================================
%Expansion of Eigenvalue One:
%==========================================================================


%Pick an eigenvalue to expand around:
lambda1 = Sigma(whichEig,whichEig)
lambda2 = Sigma(3,3)

lambda_test = (-(1+sigma) - sqrt(sigma^2 - 2*sigma + 1 + 4*sigma*rho))/2
lambda_error = abs(lambda1 - lambda_test)
%lambda = lambda_test


if (abs(lambda1 + beta) < 5*10^(-6))
   lambdaProblem = stopHere 
end


lambda1_series = zeros(1, M+1);
lambda1_series(1) = lambda1;

lambda2_series = zeros(1, M+1);
lambda2_series(1) = lambda2;


theDenom = 2*lambda1 + sigma + 1;
lambda1_series(2) = sigma/theDenom;
 
for n = 2:M
    theSum = 0;
    for k = 1:(n-1)
        theSum = theSum + lambda1_series(n-k+1)*lambda1_series(k+1);
    end
    lambda1_series(n+1) = -theSum/theDenom;
end


%==========================================================================
%Eigenvalue Residual Error:
%==========================================================================


lambda_squared = convolveSeries_1D_general(lambda1_series, lambda1_series);


big_lambda = zeros(1, 2*M+1);
Error = zeros(1, 2*M+1);

big_lambda(1, 1:M+1) = lambda1_series;

Error = lambda_squared + (1+sigma)*big_lambda;

Error(1) = Error(1) + sigma*(1-rho);
Error(2) = Error(2) - sigma;

sigma_error = 0;
for m = 0:(2*M)
   sigma_error = sigma_error + abs(Error(m+1))*abs(omega)^m; 
end

sigma_error

%==========================================================================
%Expansion of Eigenvector:
%==========================================================================

xi1_1_series = zeros(1, M+1);
xi1_2_series = zeros(1, M+1);
xi1_3_series = zeros(1, M+1);
        
xi1_1_series(1) = U(1,whichEig);
xi1_2_series(1) = U(2,whichEig);

xi2_1_series = zeros(1, M+1);
xi2_2_series = zeros(1, M+1);
xi2_3_series = zeros(1, M+1);
       
xi2_3_series(1) = U(3,3);

A = [   -(sigma+lambda1),            sigma;
        2*xi1_1_series(1), 2*xi1_2_series(1)];

b = [lambda1_series(2)*xi1_1_series(1);
                                    0];
                               
xi1 = A\b;                               
   
xi1_1_series(2) = xi1(1);
xi1_2_series(2) = xi1(2);

for n = 2:M
     
    theSum1 = 0;
    for k = 0:(n-1)
        theSum1 = theSum1 + lambda1_series(n-k+1)*xi1_1_series(k+1);
    end
    
    theSum2 = 0;
    for k = 1:(n-1)
        theSum2 = theSum2 + xi1_1_series(n-k+1)*xi1_1_series(k+1) + ...
                            xi1_2_series(n-k+1)*xi1_2_series(k+1);
    end
        
    b = [ theSum1;
         -theSum2];
    xi_n = A\b;
    
    xi1_1_series(n+1) = xi_n(1);
    xi1_2_series(n+1) = xi_n(2);
end

%==========================================================================
%Eigenvectior Residual Error:
%==========================================================================

big_xi1 = zeros(1, 2*M+1);
big_xi2 = zeros(1, 2*M+1);

big_xi1(1:M+1) = xi1_1_series;
big_xi2(1:M+1) = xi1_2_series;

xi1_squared = convolveSeries_1D_general(xi1_1_series, xi1_1_series);
xi2_squared = convolveSeries_1D_general(xi1_2_series, xi1_2_series);
xi1_times_lambda = convolveSeries_1D_general(xi1_1_series, lambda1_series);

Error_1 = -sigma*big_xi1 + sigma*big_xi2 - xi1_times_lambda;
Error_2 = xi1_squared + xi2_squared;

Error_2(1) = Error_2(1) - 1;

sigmaError1 = 0;
sigmaError2 = 0;

for m = 0:2*M
    sigmaError1 = sigmaError1 + abs(Error_1(m+1))*abs(omega)^m;
    sigmaError2 = sigmaError2 + abs(Error_2(m+1))*abs(omega)^m; 
end

sigmaError_eigVect = max(sigmaError1, sigmaError2)



%==========================================================================
%Compute 2D manifold coefficients at omega = 0:
%==========================================================================

lambda2 = Sigma(3,3);

xi1 = U(:,1);
xi2 = U(:,3);


[QN1, QN2, QN3] = lorenz_2D_manifold(p0, lambda1, lambda2, xi1, xi2, ...
                                      sigma, rho, beta, N, 1);

                                
%==========================================================================                                  
%2D Manifold Residual:     
%==========================================================================
                      
                              
   

[goodComputation, delta0] = lorenzApostori(N, 1, sigma, rho, beta, domRad0, ...
                                           QN1, QN2, QN3,lambda1,lambda2);                             

delta0  



                     
%==========================================================================
%Compute rho partial of 2D PN at omega = 0 (i.e. first order term in omega).
%==========================================================================

[QN1_rho, QN2_rho, QN3_rho] = lorenz_2D_rhoPartial_at0(p0, lambda1, lambda2, ...
                                                       xi1, xi2, QN1, QN2, QN3, ...
                                                       sigma, rho, beta, N, 1);


%==========================================================================
%Omega Partial at omega = 0; 2D Residuial:
%==========================================================================


Q3Qomega1 = convolveSeries_2D_general(QN3, QN1_rho);
Q1Qomega3 = convolveSeries_2D_general(QN1, QN3_rho);

Q2Qomega1 = convolveSeries_2D_general(QN2, QN1_rho);
Q1Qomega2 = convolveSeries_2D_general(QN1, QN2_rho);


Q1_big = zeros(2*N+1, 2*N+1);
Q2_big = zeros(2*N+1, 2*N+1);
Q3_big = zeros(2*N+1, 2*N+1);

Qomega1_big = zeros(2*N+1, 2*N+1);
Qomega2_big = zeros(2*N+1, 2*N+1);
Qomega3_big = zeros(2*N+1, 2*N+1);

differentialTerm1 = zeros(2*N+1, 2*N+1);
differentialTerm2 = zeros(2*N+1, 2*N+1);
differentialTerm3 = zeros(2*N+1, 2*N+1);


Q1_big(1:N+1, 1:N+1) = QN1;
Q2_big(1:N+1, 1:N+1) = QN2;
Q3_big(1:N+1, 1:N+1) = QN3;

Qomega1_big(1:N+1, 1:N+1) = QN1_rho;
Qomega2_big(1:N+1, 1:N+1) = QN2_rho;
Qomega3_big(1:N+1, 1:N+1) = QN3_rho;

for n = 0:N
	for m = 0:N
		term1 = m*lambda1_series(2)*QN1(m+1, n+1) + ...
              m*lambda1_series(1)*QN1_rho(m+1, n+1) + ...
              n*lambda2_series(2)*QN1(m+1, n+1) + ...
              n*lambda2_series(1)*QN1_rho(m+1, n+1);
		term2 = m*lambda1_series(2)*QN2(m+1, n+1) + ...
              m*lambda1_series(1)*QN2_rho(m+1, n+1) + ...
              n*lambda2_series(2)*QN2(m+1, n+1) + ...
              n*lambda2_series(1)*QN2_rho(m+1, n+1);
		term3 = m*lambda1_series(2)*QN3(m+1, n+1) + ...
              m*lambda1_series(1)*QN3_rho(m+1, n+1) + ...
              n*lambda2_series(2)*QN3(m+1, n+1) + ...
              n*lambda2_series(1)*QN3_rho(m+1, n+1);
		differentialTerm1(m+1, n+1) = term1;
		differentialTerm2(m+1, n+1) = term2;
		differentialTerm3(m+1, n+1) = term3;
	end
end



Error_rhoPartial_1 = -sigma*Qomega1_big + sigma*Qomega2_big + ...
                     -differentialTerm1;
 
                 
Error_rhoPartial_2 = rho*Qomega1_big - Q3Qomega1 - Q1Qomega3 - ...
                     Qomega2_big + Q1_big - differentialTerm2;

Error_rhoPartial_3 = Q2Qomega1 + Q1Qomega2 - beta*Qomega3_big - ...
                     differentialTerm3;
                 
thisSum1 = 0;
thisSum2 = 0;
thisSum3 = 0;

for n = 0:2*N
	for m = 0:2*N
    thisSum1 = thisSum1 + abs(Error_rhoPartial_1(m+1, n+1))*domRad0^(n+m);
    thisSum2 = thisSum2 + abs(Error_rhoPartial_2(m+1, n+1))*domRad0^(n+m);
    thisSum3 = thisSum3 + abs(Error_rhoPartial_3(m+1, n+1))*domRad0^(n+m);
	end
end   
                 
           
rhoPartial_sigmaError1 = max([thisSum1, thisSum2, thisSum3])


%==========================================================================
%Compute 2D manifold parameter expansion in omega:
%==========================================================================

QMN1 = zeros(M+1, N+1, N+1);
QMN2 = zeros(M+1, N+1, N+1);
QMN3 = zeros(M+1, N+1, N+1);

QMN1(1, :, :) = QN1;
QMN2(1, :, :) = QN2;
QMN3(1, :, :) = QN3;

QMN1(2, :, :) = QN1_rho;
QMN2(2, :, :) = QN2_rho;
QMN3(2, :, :) = QN3_rho;

QMN1(3:M+1, 2, 1) = xi1_1_series(3:M+1)';
QMN2(3:M+1, 2, 1) = xi1_2_series(3:M+1)';
QMN3(3:M+1, 2, 1) = xi1_3_series(3:M+1)';

QMN1(3:M+1, 1, 2) = xi2_1_series(3:M+1)';
QMN2(3:M+1, 1, 2) = xi2_2_series(3:M+1)';
QMN3(3:M+1, 1, 2) = xi2_3_series(3:M+1)';

Id = eye(3);

for n1 = 0:N
	for n2 = 0:N
   		if ((n1+n2 >= 2) & (n1 + n2 <= N))
			A = Dfp0 - (n1*lambda1 + n2*lambda2)*Id;
			for m = 2:M
				thisTerm1 = 0;
        		thisTerm2 = 0;
        		thisTerm3 = 0;
        		for l = 0:(m-1)
					lambdaTerm = n1*lambda1_series(m-l+1) + n2*lambda2_series(m-l+1);
            		thisTerm1 = thisTerm1 + lambdaTerm*QMN1(l+1, n1+1, n2+1);
            		thisTerm2 = thisTerm2 + lambdaTerm*QMN2(l+1, n1+1, n2+1);
            		thisTerm3 = thisTerm3 + lambdaTerm*QMN3(l+1, n1+1, n2+1);
        		end

				smn1n2_term = ...	
					convolution_smn1n2(m,n1,n2,QMN1,QMN2,QMN3, sigma, rho, beta, M, N);

				
				s1 = thisTerm1;
        		s2 = thisTerm2 + smn1n2_term(2) - QMN1(m, n1+1, n2+1);
        		s3 = thisTerm3 - smn1n2_term(3);
        
        		b = [s1;
            		  s2;
             	  s3];
         
        		thisCoeff = A\b;
        
        		QMN1(m+1, n1+1, n2+1) = thisCoeff(1);
        		QMN2(m+1, n1+1, n2+1) = thisCoeff(2);
        		QMN3(m+1, n1+1, n2+1) = thisCoeff(3);
			end
		end
	end
end


%Test Expansion: 
%Dfp0_omega = [   -sigma, sigma,     0;
%              rho+omega,    -1,     0;
%                      0,     0, -beta];
           
%[U_omega, Sigma_omega] = eigs(Dfp0_omega);

%lambda1_omega = Sigma_omega(1,1);
%lambda2_omega = Sigma_omega(3,3);

%xi1_omega = U_omega(:,1);
%xi2_omega = U_omega(:,3);

%[QN1_omega, QN2_omega, QN3_omega] = lorenz_2D_manifold(…
%                       p0, lambda1_omega, lambda2_omega, xi1_omega, xi2_omega, ...
%                       sigma, rho+omega, beta, N, 1);

%n1 = 1
%n2 = 1  
%thisCoef = QN1_omega(n1+1, n2+1)
%thisSum = 0;

%for m = 0:M
%	thisSum = thisSum + QMN1(m+1, n1+1, n2+1)*omega^m;
%end

%coeffError = abs(thisCoef - thisSum)

%==========================================================================
%Omega Expansion 2D Residual Error:
%==========================================================================

QMN1_big = zeros(2*M+1, 2*N+1, 2*N+1); 
QMN2_big = zeros(2*M+1, 2*N+1, 2*N+1); 
QMN3_big = zeros(2*M+1, 2*N+1, 2*N+1); 

Q1omega = zeros(2*M+1, 2*N+1, 2*N+1);

differentialTerm1 = zeros(2*M+1, 2*N+1, 2*N+1);
differentialTerm2 = zeros(2*M+1, 2*N+1, 2*N+1);
differentialTerm3 = zeros(2*M+1, 2*N+1, 2*N+1);

QMN1_big(1:M+1, 1:N+1, 1:N+1) = QMN1;
QMN2_big(1:M+1, 1:N+1, 1:N+1) = QMN2;
QMN3_big(1:M+1, 1:N+1, 1:N+1) = QMN3;

QMN13 = convolveSeries_3D_general(QMN1,QMN3);
QMN12 = convolveSeries_3D_general(QMN1,QMN2);



for n1 = 0:N
	for n2 = 0:N
		if (n1 + n2 <= N)
    		this_amn1 = zeros(1, M+1);
    		this_amn1 = QMN1(:, n1+1, n2+1)';
    
    		this_amn2 = zeros(1, M+1);
    		this_amn2 = QMN2(:, n1+1, n2+1)';
    
    		this_amn3 = zeros(1, M+1);
    		this_amn3 = QMN3(:, n1+1, n2+1)';
    
    		thisDiffTerm1_1 = convolveSeries_1D_general(this_amn1, lambda1_series);
    		thisDiffTerm2_1 = convolveSeries_1D_general(this_amn2, lambda1_series);
    		thisDiffTerm3_1 = convolveSeries_1D_general(this_amn3, lambda1_series);

			thisDiffTerm1_2 = convolveSeries_1D_general(this_amn1, lambda2_series);
    		thisDiffTerm2_2 = convolveSeries_1D_general(this_amn2, lambda2_series);
    		thisDiffTerm3_2 = convolveSeries_1D_general(this_amn3, lambda2_series);

    		differentialTerm1(:, n1+1, n2+1) = n1*thisDiffTerm1_1 + n2*thisDiffTerm1_2;
    		differentialTerm2(:, n1+1, n2+1) = n1*thisDiffTerm2_1 + n2*thisDiffTerm2_2;
    		differentialTerm3(:, n1+1, n2+1) = n1*thisDiffTerm3_1 + n2*thisDiffTerm3_2;
		end
	end
end



for n1 = 0:N
	for n2 = 0:N
		if (n1+n2 <= N)
    		for m = 1:M
        		Q1omega(m+1, n1+1, n2+1) = QMN1(m, n1+1, n2+1); 
    		end
		end
	end
end



Error_omega_1 = -sigma*QMN1_big + sigma*QMN2_big - differentialTerm1; 

Error_omega_2 = rho*QMN1_big + Q1omega - QMN13 - ...
                               QMN2_big - differentialTerm2;

Error_omega_3 = QMN12 - beta*QMN3_big - differentialTerm3;

thisSum1 = 0;
thisSum2 = 0;
thisSum3 = 0;

for n1 = 0:2*N
   for n2 = 0:2*N
		if (n1+n2 <= N)
			for m = 0:2*M
        		thisSum1 = thisSum1 + ...
                   abs(Error_omega_1(m+1, n1+1, n2+1))*(omega^m)*domRad0^(n1+n2);
        		thisSum2 = thisSum2 + ...
                   abs(Error_omega_2(m+1, n1+1, n2+1))*(omega^m)*(domRad0)^(n1+n2);
        		thisSum3 = thisSum3 + ...
                   abs(Error_omega_3(m+1, n1+1, n2+1))*(omega^m)*(domRad0)^(n1+n2);
			end
		end 
	end
end

expansion2D_sigmaError = max([thisSum1, thisSum2, thisSum3])









