clear 
format long


%Lorenz Parameters:
sigma = intval('10')                          
beta = intval('8')/intval('3')
rho = intval('13.9265')
%rho = intval('28')

%Parameter Domains:
omega = intval(1.5)
domRad0 = intval(1.0); 


%Parameterization Controls:
M = 15
N = 30

eig_num = 1
scale_eig = 1

%Fixed point at origin:
p0 = [intval(0);
      intval(0);
      intval(0)];
   
  
Dfp0 = [    -sigma,         sigma,     intval(0);
               rho,    intval(-1),     intval(0);
         intval(0),     intval(0),          -beta]

            
[U_temp, Sigma_temp] = eigs(Dfp0.mid);

Sigma_temp

%Verify eigen data plus:
[lambda, xi] = ...
     verifyeig(Dfp0, Sigma_temp(eig_num, eig_num), U_temp(:,eig_num));

 xi = scale_eig*xi
 
%========================================================================
%Eigenvalue Expansion:
%========================================================================

lambda_test = (-(1+sigma) + sqrt(sigma^2 - 2*sigma + 1 + 4*sigma*rho))/2
lambda_error = abs(lambda - lambda_test)
%lambda = lambda_test


%if (abs(lambda + beta) < 5*10^(-6))
%   lambdaProblem = stopHere 
%end


lambda_series = intval(zeros(1, M+1));
lambda_series(1) = lambda;
theDenom = 2*lambda + sigma + intval(1);
lambda_series(2) = sigma/theDenom;
 
for n = 2:M
    theSum = intval(0);
    for k = 1:(n-1)
        theSum = theSum + lambda_series(n-k+1)*lambda_series(k+1);
    end
    lambda_series(n+1) = -theSum/theDenom;
end



%==========================================================================
%Eigenvalue Residual Error:
%==========================================================================


lambda_squared = ...
    convolveSeries_1D_general_intval(lambda_series, lambda_series);


big_lambda = intval(zeros(1, 2*M+1));
Error = intval(zeros(1, 2*M+1));

big_lambda(1, 1:M+1) = lambda_series;

Error = lambda_squared + (intval(1)+sigma)*big_lambda;

Error(1) = Error(1) + sigma*(intval(1)-rho);
Error(2) = Error(2) - sigma;

sigma_error = intval(0);
for m = 0:(2*M)
   sigma_error = sigma_error + abs(Error(m+1))*abs(omega)^m; 
end

sigma_error



%==========================================================================
%Validate Eigenvalue:
%==========================================================================


%Bound on order > 0 terms:
R_lambda = 0;
for m = 1:(M-1)
    R_lambda = R_lambda + abs(lambda_series(m+1))*omega^m;
end

R_lambda

%Bound on Lambda series:
lambda_bound = abs(lambda) + R_lambda

%Bound on second derivatives of fixed point operator:
M_lambda = 2

%Bound of derivative of the operator:
test_M = 2/abs(lambda +(1+sigma))
if (inf(test_M*M_lambda) >=1)
    crashNow = crashHere
end

K_lambda = R_lambda*test_M 

%NK lambda
epsilon_NK_lambda = K_lambda*sigma_error

if (sup(4*M_lambda*K_lambda*epsilon_NK_lambda) > 1)
   crashNow = crashHere 
end

%Delta/Truncation bound on lambda series:
delta_lambda = 2*epsilon_NK_lambda



%==========================================================================
%Expansion of Eigenvector:
%==========================================================================




xi_1_series = intval(zeros(1, M+1));
xi_2_series = intval(zeros(1, M+1));
xi_3_series = intval(zeros(1, M+1));
        
xi_1_series(1) = xi(1);
xi_2_series(1) = xi(2);


A = [   -(sigma+lambda),            sigma;
       2*xi_1_series(1), 2*xi_2_series(1)];

b = [lambda_series(2)*xi_1_series(1);
                           intval(0)];
                               
xi1 = A\b;                               
   
xi_1_series(2) = xi1(1);
xi_2_series(2) = xi1(2);


for n = 2:M
     
    theSum1 = intval(0);
    for k = 0:(n-1)
        theSum1 = theSum1 + lambda_series(n-k+1)*xi_1_series(k+1);
    end
    
    theSum2 = intval(0);
    for k = 1:(n-1)
        theSum2 = theSum2 + xi_1_series(n-k+1)*xi_1_series(k+1) + ...
                            xi_2_series(n-k+1)*xi_2_series(k+1);
    end
    
    
    b = [ theSum1;
         -theSum2];
    xi_n = A\b;
    
    xi_1_series(n+1) = xi_n(1);
    xi_2_series(n+1) = xi_n(2);
end



%==========================================================================
%Eigenvectior Residual Error:
%==========================================================================


big_xi1 = intval(zeros(1, 2*M+1));
big_xi2 = intval(zeros(1, 2*M+1));

big_xi1(1:M+1) = xi_1_series;
big_xi2(1:M+1) = xi_2_series;

xi1_squared = convolveSeries_1D_general_intval(xi_1_series, xi_1_series);
xi2_squared = convolveSeries_1D_general_intval(xi_2_series, xi_2_series);
xi1_times_lambda = ...
    convolveSeries_1D_general_intval(xi_1_series, lambda_series);

Error_1 = -sigma*big_xi1 + sigma*big_xi2 - xi1_times_lambda;
Error_2 = xi1_squared + xi2_squared;

Error_2(1) = Error_2(1) - ((scale_eig)^2)*intval(1);


sigmaError1 = intval(0);
sigmaError2 = intval(0);


for m = 0:2*M
    sigmaError1 = sigmaError1 + abs(Error_1(m+1))*abs(omega)^m;
    sigmaError2 = sigmaError2 + abs(Error_2(m+1))*abs(omega)^m; 
end


sigmaError_eigVect = max([sup(sigmaError1), sup(sigmaError2)])



%==========================================================================
%A-posteriori validation Eigenvect:
%==========================================================================


%Bound on order > 0 terms:
R_xi = 0;
for m = 1:(M-1)
    R_xi = R_xi + norm([xi_1_series(m+1), xi_2_series(m+1)])*omega^m;
end

R_xi

%Bound on Lambda series:
xi_bound = norm(xi) + R_lambda

%Bound on second derivatives of fixed point operator:
M_xi = 4


%Bound of derivative of the operator:
A_0 = [-sigma-lambda, sigma; 2*xi(1), 2*xi(2)]
A_0inv = inv(A_0)
test_M = 0;
for m = 1:M
   A_m = [-lambda_series(m+1),                   0; ...
           2*xi_1_series(m+1), 2*xi_2_series(m+1)];
   this_norm = norm(A_m, inf);
   test_M = test_M + this_norm*omega^m;
   
end

test_M = norm(A_0inv, inf)*(test_M + delta_lambda)


if (inf(test_M) >=1)
    crashNow = crashHere
end

K_xi = norm(A_0inv,inf)/(1-test_M)


%NK lambda
epsilon_NK_xi = K_xi*(sigmaError_eigVect + delta_lambda*xi_bound)

if (sup(4*M_xi*K_xi*epsilon_NK_xi) > 1)
   crashNow = crashHere 
end

%Delta/Truncation bound on lambda series:
delta_xi = 2*epsilon_NK_xi
















