function varargout = triangulate_2D_stable(Ds_fd, Ws_fd, R, ...
    As, Bs, Cs, N, alpha, tau, a, b, c, ...
    startIteration, num_triIterates, p, phaseSpaceTol,...
    numRadii, numAngles, areaMultiplier, triTol);


%Pov Ray Graphics:======================================================== 
   
   %Throw out triangles if their area is bigger than:  
     %areaMultiplier = 5.0
   
     %triAreaThreshold = 0.01;
     %define adapively below
   %Delauny over connects as it assumes simply connected.
   %correct by throwing away triagles with sides too 
   %long
     %triTol = 0.05
     %triTol = 0.5


%==========================================================================
%Produce the local stable parameter set, and their images.
%==========================================================================

m = numRadii;  %number of radi.
n = numAngles; %number of angles.

totalPossiblePoints = m*n;
Ws_loc = zeros(totalPossiblePoints, 3);
Ds_parms = zeros(totalPossiblePoints, 2);

r_step = R/m;
theta_step = 2*pi/n;

u_pointNum = 1;
s_pointNum = 1;

the_rs = (0.0:r_step:R);
thetas = (0.0:theta_step:(2*pi));
 

for i = 1:m
   rad_number = i
    for j = 1:n
       thisIndex = i*(n-1) + j;
       thisRay = the_rs(i)*[cos(thetas(j)), sin(thetas(j))]; 
       Ds_parms(thisIndex, 1:2) = [thisRay(1), thisRay(2)];    
    end
end    

Ds_parms = unique(Ds_parms, 'rows');

%==========================================================================
%Triangulate the parameter domain:
%==========================================================================

x = Ds_parms(:,1);
y = Ds_parms(:,2);

TRI_s = delaunay(x, y);

%figure
%hold on
%triplot(TRI_s, Ds(:,1),  Ds(:,2), 'red');

%==========================================================================
%Compute the phase space triangulation of the local manifold,
%in PovRay ready format.
%==========================================================================


tempSizeVect = size(TRI_s)
sizeTri = tempSizeVect(1,1)

for (k = 1:sizeTri)
  x1 = x(TRI_s(k, 1)); 
  y1 = y(TRI_s(k, 1));
  
  x2 = x(TRI_s(k, 2)); 
  y2 = y(TRI_s(k, 2));
  
  x3 = x(TRI_s(k, 3)); 
  y3 = y(TRI_s(k, 3));
  
  p1 = evaluate_complexCase(x1, y1, As, Bs, Cs, N)';
  p2 = evaluate_complexCase(x2, y2, As, Bs, Cs, N)';
  p3 = evaluate_complexCase(x3, y3, As, Bs, Cs, N)';
  
  vertices_s(k, 1:3) = p1; 
  vertices_s(k, 4:6) = p2;
  vertices_s(k, 7:9) = p3;

end

Ws_triang = vertices_s;

%==========================================================================
%Triangulate Fundamental Domain:
%==========================================================================

cloudPoints = Ds_fd;
cloudPoints = unique(cloudPoints, 'rows');

x = cloudPoints(:,1);
y = cloudPoints(:,2);

%figure
%hold on
TRI_sFD = delaunay(x, y);
delaunyDone = 1

%Build Triangulation of Ws_loc:
tempSizeVect = size(TRI_sFD);
sizeTri = tempSizeVect(1,1);
goodTriangles = 0;
TRI_coor = zeros(sizeTri, 3);

%Remove Triangules that are too big.  (Delaunay assumes domain is simply
%connected.  Remove mistaken edges `by hand'.

for (k = 1:sizeTri)
    v1 = [x(TRI_sFD(k,1)), y(TRI_sFD(k,1))];
    v2 = [x(TRI_sFD(k,2)), y(TRI_sFD(k,2))];
    v3 = [x(TRI_sFD(k,3)), y(TRI_sFD(k,3))];
    dist1 = norm(v1-v2);
    dist2 = norm(v1-v3);
    dist3 = norm(v2-v3);
    
    theDistances = [dist1, dist2, dist3];
    maxDistance = max(theDistances);
    if (maxDistance < triTol);
        goodTriangles = goodTriangles + 1;
        TRI_coor(goodTriangles, 1:3) = TRI_sFD(k, 1:3);
    end    
end    

fixDomainDone = 1

TRI_sFD = TRI_coor(1:goodTriangles, 1:3);

figure
hold on
triplot(TRI_sFD, cloudPoints(:,1),  cloudPoints(:,2), 'red');

tempSizeVect = size(TRI_sFD);
sizeTri = tempSizeVect(1,1)

Ds_fd_points = cloudPoints;
Ds_fd_TRI = TRI_sFD;

%==========================================================================
%Lift the triangulation of the Fundamental Domain into phase space:
%==========================================================================

%Stable Triangulation:

vertices_sFD = zeros(sizeTri, 9);

for (k = 1:sizeTri)
  %k
  %sizeTri
  x1 = x(TRI_sFD(k, 1)); 
  y1 = y(TRI_sFD(k, 1));
  
  x2 = x(TRI_sFD(k, 2)); 
  y2 = y(TRI_sFD(k, 2));
  
  x3 = x(TRI_sFD(k, 3)); 
  y3 = y(TRI_sFD(k, 3));
  
  p1 = evaluate_complexCase(x1, y1, As, Bs, Cs, N)';
  p2 = evaluate_complexCase(x2, y2, As, Bs, Cs, N)';
  p3 = evaluate_complexCase(x3, y3, As, Bs, Cs, N)';
  
  vertices_sFD(k, 1:3) = p1; 
  vertices_sFD(k, 4:6) = p2;
  vertices_sFD(k, 7:9) = p3;

end    

Ws_fd_triang = vertices_sFD;
stableVerticesDone = 1

figure
hold on
plot3(vertices_sFD(:,1), vertices_sFD(:,2), vertices_sFD(:,3), 'red.');

%==========================================================================
%Globalize the fundamental domain with serpenski subdivision.
%==========================================================================

%Set Tri Area Threshold:

p1 = vertices_sFD(1, 1:3); 
p2 = vertices_sFD(1, 4:6);
p3 =  vertices_sFD(1, 7:9);
triArea = 0.5*norm(cross(p2-p1, p3-p1))
triAreaThreshold = areaMultiplier*triArea;

%Check area of triangle in parameter space:
q1 = [cloudPoints(TRI_sFD(1, 1), 1:2), 0];
q2 = [cloudPoints(TRI_sFD(1,2), 1:2), 0];
q3 = [cloudPoints(TRI_sFD(1,3), 1:2), 0];

parameterTriAres = 0.5*norm(cross(q2-q1, q3-q1));

%Save Fundamental Domain Data For Unstable Computation:
 %hold_TRI = TRI_sFD;
 %hold_cloud = cloudPoints;

%-------------------------------------------------------------------------- 
%Iterate Stable Triangles:
%--------------------------------------------------------------------------

memoryReserve = 5*10^5

%current size of fundamental domain
tempSize = size(TRI_sFD);
numTriangles = tempSize(1,1);  

%current number of vertices
tempSize = size(vertices_sFD);
numVertices = tempSize(1,1);

%current number of cloud points
tempSize = size(cloudPoints);
numCloudPoints = tempSize(1,1);

%Initialize temp stable manifods container
Ws_temp = zeros(2*10^6, 9);
Ws_temp(1:numVertices, 1:9) = vertices_sFD;

numWsPoints = numVertices;
goodPoints = 0;

%Initialize (TRI_sFD) and vertictices_sFD
TRI_temp = zeros(memoryReserve, 3);
vertices_temp = zeros(memoryReserve, 9);
cloud_temp = zeros(memoryReserve, 2);

cloud_temp(1:numCloudPoints, 1:2) = cloudPoints;
TRI_temp(1:numTriangles, 1:3) = TRI_sFD;
vertices_temp(1:numVertices, 1:9) = vertices_sFD;

lastBarycenter = 1;
%Globalize Stable Manifold:
for (n = startIteration:(startIteration + num_triIterates))
    n
    thisIndex = 1;
    
    while (thisIndex <= numTriangles)
        goToNextIndex = 0;
        subdivisions = 0;
 
        if (mod(thisIndex, 1000) == 1)
             n_s = n
             thisIndex_s = thisIndex
             numTriangles_s = numTriangles
        end
        
        while ((goToNextIndex ~= 1) && (thisIndex <=numTriangles))
            %thisIndex
            %numTriangles
            %Check Condition number of simplex and compute image:
            [fp1, fp2, fp3, conditionNumber, newTriArea] = ...
                stableImageTriangle_and_conditionsII(...
                thisIndex, n, vertices_temp, triAreaThreshold, ...
                phaseSpaceTol, alpha, tau, a, b, c);
   
          %conditionNumber  
          if (conditionNumber == 1)
           %Simplex Image Was Fine:  
            thisIndex = thisIndex + 1;
            numWsPoints = numWsPoints + 1;
            Ws_temp(numWsPoints, 1:9) = [fp1, fp2, fp3];
            goToNextIndex = 1;
          elseif (conditionNumber == 0)
            %Need to subdivide and repeat this index:
            %       (Fix Fundamental domain and images)
               [p1, p2, p3, T1, T2, T3, T4] =...
                   serpinskiSubdivision(thisIndex, TRI_temp, ...
                                        cloud_temp, numCloudPoints);
               TRI_temp(thisIndex, 1:3) = T1;
               TRI_temp(numTriangles + 1, 1:3) = T2;
               TRI_temp(numTriangles + 2, 1:3) = T3;
               TRI_temp(numTriangles + 3, 1:3) = T4;
               cloud_temp(numCloudPoints + 1, 1:2) = p1;
               cloud_temp(numCloudPoints + 2, 1:2) = p2;
               cloud_temp(numCloudPoints + 3, 1:2) = p3;
               numCloudPoints = numCloudPoints + 3;
               numTriangles = numTriangles + 3;
    
               outSubdivide = 1;
          
               [vertices_1, vertices_2, vertices_3, vertices_4] = ...
                  upDate_liftVerticesII(thisIndex, numTriangles, N, ...
                  TRI_temp, cloud_temp, As, Bs, Cs);          
  
               vertices_temp(thisIndex, 1:9) = vertices_1; 
               vertices_temp(numVertices + 1, 1:9) = vertices_2;
               vertices_temp(numVertices + 2, 1:9) = vertices_3; 
               vertices_temp(numVertices + 3, 1:9) = vertices_4;
               numVertices = numVertices + 3;
                   
          elseif (conditionNumber == -1)
            %Simplex image was out of bounds. 
            thisIndex = thisIndex + 1;
            goToNextIndex = 1;
          else
            threwUnexpectedConditionNumber = stopNow;
          end
        end  
    end
end

Ws_fd_triang_global = Ws_temp(1:numWsPoints, 1:9);

figure
hold on
plot3(Ws_fd_triang_global(:,1), ... 
      Ws_fd_triang_global(:,2), ...
      Ws_fd_triang_global(:,3), 'red.');



varargout(1,1)={Ds_fd_points};
varargout(2,1)={Ds_fd_TRI};
varargout(3,1)={Ws_triang};
varargout(4,1)={Ws_fd_triang,};
varargout(5,1)={Ws_fd_triang_global};



















