function varargout = triangulate_2D_unstable(Du_fd, Wu_fd, R, ...
    Au, Bu, Cu, N, alpha, tau, a, b, c, ...
    startIteration, num_triIterates, p, phaseSpaceTol,...
    numRadii, numAngles, areaMultiplier, triTol);



%Pov Ray Graphics:======================================================== 
   
   %Throw out triangles if their area is bigger than:  
     %areaMultiplier = 5.0
   
     %triAreaThreshold = 0.01;
     %define adapively below
   %Delauny over connects as it assumes simply connected.
   %correct by throwing away triagles with sides too 
   %long
     %triTol = 0.05

%==========================================================================
%Produce the local stable parameter set, and their images.
%==========================================================================

m = numRadii;  %number of radi.
n = numAngles; %number of angles.

totalPossiblePoints = m*n;
Wu_loc = zeros(totalPossiblePoints, 3);
Du_parms = zeros(totalPossiblePoints, 2);


r_step = R/m;
theta_step = 2*pi/n;

u_pointNum = 1;
s_pointNum = 1;

the_rs = (0.0:r_step:R);
thetas = (0.0:theta_step:(2*pi));
 

for i = 1:m
   rad_number = i
    for j = 1:n
       thisIndex = i*(n-1) + j;
       thisRay = the_rs(i)*[cos(thetas(j)), sin(thetas(j))]; 
       Du_parms(thisIndex, 1:2) = [thisRay(1), thisRay(2)];    
    end
end    

Du_parms = unique(Du_parms, 'rows');

%==========================================================================
%Triangulate the parameter domain:
%==========================================================================


x = Du_parms(:,1);
y = Du_parms(:,2);

TRI_u = delaunay(x, y);

%figure
%hold on
%triplot(TRI_u, Du(:,1),  Du(:,2), 'red');

%==========================================================================
%Compute the phase space triangulation of the local manifold,
%in PovRay ready format.
%==========================================================================


tempSizeVect = size(TRI_u)
sizeTri = tempSizeVect(1,1)

for (k = 1:sizeTri)
  x1 = x(TRI_u(k, 1)); 
  y1 = y(TRI_u(k, 1));
  
  x2 = x(TRI_u(k, 2)); 
  y2 = y(TRI_u(k, 2));
  
  x3 = x(TRI_u(k, 3)); 
  y3 = y(TRI_u(k, 3));
  
  p1 = evaluate_complexCase(x1, y1, Au, Bu, Cu, N)';
  p2 = evaluate_complexCase(x2, y2, Au, Bu, Cu, N)';
  p3 = evaluate_complexCase(x3, y3, Au, Bu, Cu, N)';
  
  vertices_u(k, 1:3) = p1; 
  vertices_u(k, 4:6) = p2;
  vertices_u(k, 7:9) = p3;

end

Wu_triang = vertices_u;

%==========================================================================
%Triangulate Fundamental Domain:
%==========================================================================

cloudPoints = Du_fd;
cloudPoints = unique(cloudPoints, 'rows');

x = cloudPoints(:,1);
y = cloudPoints(:,2);


%figure
%hold on
TRI_uFD = delaunay(x, y);
delaunyDone = 1

%Build Triangulation of Ws_loc:
tempSizeVect = size(TRI_uFD);
sizeTri = tempSizeVect(1,1);
goodTriangles = 0;
TRI_coor = zeros(sizeTri, 3);

%Remove Triangules that are too big.  (Delaunay assumes domain is simply
%connected.  Remove mistaken edges `by hand'.

for (k = 1:sizeTri)
    v1 = [x(TRI_uFD(k,1)), y(TRI_uFD(k,1))];
    v2 = [x(TRI_uFD(k,2)), y(TRI_uFD(k,2))];
    v3 = [x(TRI_uFD(k,3)), y(TRI_uFD(k,3))];
    dist1 = norm(v1-v2);
    dist2 = norm(v1-v3);
    dist3 = norm(v2-v3);
    
    theDistances = [dist1, dist2, dist3];
    maxDistance = max(theDistances);
    if (maxDistance < triTol);
        goodTriangles = goodTriangles + 1;
        TRI_coor(goodTriangles, 1:3) = TRI_uFD(k, 1:3);
    end    
end    

fixDomainDone = 1

TRI_uFD = TRI_coor(1:goodTriangles, 1:3);

figure
hold on
triplot(TRI_uFD, cloudPoints(:,1),  cloudPoints(:,2), 'blue');


tempSizeVect = size(TRI_uFD);
sizeTri = tempSizeVect(1,1)

Du_fd_points = cloudPoints;
Du_fd_TRI = TRI_uFD;




%==========================================================================
%Lift the triangulation of the Fundamental Domain into phase space:
%==========================================================================

%Stable Triangulation:

vertices_uFD = zeros(sizeTri, 9);

for (k = 1:sizeTri)
  %k
  %sizeTri
  x1 = x(TRI_uFD(k, 1)); 
  y1 = y(TRI_uFD(k, 1));
  
  x2 = x(TRI_uFD(k, 2)); 
  y2 = y(TRI_uFD(k, 2));
  
  x3 = x(TRI_uFD(k, 3)); 
  y3 = y(TRI_uFD(k, 3));
  
  p1 = evaluate_complexCase(x1, y1, Au, Bu, Cu, N)';
  p2 = evaluate_complexCase(x2, y2, Au, Bu, Cu, N)';
  p3 = evaluate_complexCase(x3, y3, Au, Bu, Cu, N)';
  
  vertices_uFD(k, 1:3) = p1; 
  vertices_uFD(k, 4:6) = p2;
  vertices_uFD(k, 7:9) = p3;

end    

Wu_fd_triang = vertices_uFD;
stableVerticesDone = 1

figure
hold on
plot3(vertices_uFD(:,1), vertices_uFD(:,2), vertices_uFD(:,3), 'blue.');
plot3(vertices_uFD(:,4), vertices_uFD(:,5), vertices_uFD(:,6), 'blue.');
plot3(vertices_uFD(:,7), vertices_uFD(:,8), vertices_uFD(:,9), 'blue.');
%plot3(vertices_uFD(:,1), vertices_uFD(:,2), vertices_uFD(:,3), 'blue.');



%==========================================================================
%Globalize the fundamental domain with serpenski subdivision.
%==========================================================================


%Set Tri Area Threshold:

p1 = vertices_uFD(1, 1:3); 
p2 = vertices_uFD(1, 4:6);
p3 =  vertices_uFD(1, 7:9);
triArea = 0.5*norm(cross(p2-p1, p3-p1))
triAreaThreshold = areaMultiplier*triArea;

%Check area of triangle in parameter space:
q1 = [cloudPoints(TRI_uFD(1, 1), 1:2), 0];
q2 = [cloudPoints(TRI_uFD(1,2), 1:2), 0];
q3 = [cloudPoints(TRI_uFD(1,3), 1:2), 0];

parameterTriAres = 0.5*norm(cross(q2-q1, q3-q1));


%Save Fundamental Domain Data For Unstable Computation:
 %hold_TRI = TRI_sFD;
 %hold_cloud = cloudPoints;

%-------------------------------------------------------------------------- 
%Iterate Stable Triangles:
%--------------------------------------------------------------------------


memoryReserve = 5*10^5

%current size of fundamental domain
tempSize = size(TRI_uFD);
numTriangles = tempSize(1,1);  

%current number of vertices
tempSize = size(vertices_uFD);
numVertices = tempSize(1,1);

%current number of cloud points
tempSize = size(cloudPoints);
numCloudPoints = tempSize(1,1);

%Initialize temp stable manifods container
Wu_temp = zeros(2*10^6, 9);
Wu_temp(1:numVertices, 1:9) = vertices_uFD;

numWuPoints = numVertices;
goodPoints = 0;


%Initialize (TRI_sFD) and vertictices_sFD
TRI_temp = zeros(memoryReserve, 3);
vertices_temp = zeros(memoryReserve, 9);
cloud_temp = zeros(memoryReserve, 2);

cloud_temp(1:numCloudPoints, 1:2) = cloudPoints;
TRI_temp(1:numTriangles, 1:3) = TRI_uFD;
vertices_temp(1:numVertices, 1:9) = vertices_uFD;



lastBarycenter = 1;
%Globalize Stable Manifold:
for (n = startIteration:(startIteration + num_triIterates))
    n
    thisIndex = 1;
    
     while (thisIndex <= numTriangles)
        goToNextIndex = 0;
        subdivisions = 0;
 
        if (mod(thisIndex, 1000) == 1)
             n_u = n
             thisIndex_u = thisIndex
             numTriangles_u = numTriangles
        end
        
        while ((goToNextIndex ~= 1) && (thisIndex <=numTriangles))
            %thisIndex
            %numTriangles
            %Check Condition number of simplex and compute image:
            [fp1, fp2, fp3, conditionNumber, newTriArea] = ...
                unstableImageTriangle_and_conditionsII(...
                thisIndex, n, vertices_temp, triAreaThreshold, ...
                phaseSpaceTol, alpha, tau, a, b, c);
   
          %conditionNumber  
          if (conditionNumber == 1)
           %Simplex Image Was Fine:  
            thisIndex = thisIndex + 1;
            numWuPoints = numWuPoints + 1;
            Wu_temp(numWuPoints, 1:9) = [fp1, fp2, fp3];
            goToNextIndex = 1;
          elseif (conditionNumber == 0)
            %Need to subdivide and repeat this index:
            %       (Fix Fundamental domain and images)
               [p1, p2, p3, T1, T2, T3, T4] = serpinskiSubdivision(...
                   thisIndex, TRI_temp, cloud_temp, numCloudPoints);
               TRI_temp(thisIndex, 1:3) = T1;
               TRI_temp(numTriangles + 1, 1:3) = T2;
               TRI_temp(numTriangles + 2, 1:3) = T3;
               TRI_temp(numTriangles + 3, 1:3) = T4;
               cloud_temp(numCloudPoints + 1, 1:2) = p1;
               cloud_temp(numCloudPoints + 2, 1:2) = p2;
               cloud_temp(numCloudPoints + 3, 1:2) = p3;
               numCloudPoints = numCloudPoints + 3;
               numTriangles = numTriangles + 3;
    
               outSubdivide = 1;
          
               [vertices_1, vertices_2, vertices_3, vertices_4] = ...
                  upDate_liftVerticesII(thisIndex, numTriangles, N, ...
                  TRI_temp, cloud_temp, Au, Bu, Cu);          
  
               vertices_temp(thisIndex, 1:9) = vertices_1; 
               vertices_temp(numVertices + 1, 1:9) = vertices_2;
               vertices_temp(numVertices + 2, 1:9) = vertices_3; 
               vertices_temp(numVertices + 3, 1:9) = vertices_4;
               numVertices = numVertices + 3;
                   
          elseif (conditionNumber == -1)
            %Simplex image was out of bounds. 
            thisIndex = thisIndex + 1;
            goToNextIndex = 1;
          else
            threwUnexpectedConditionNumber = stopNow;
          end
        end  
    end
end



Wu_fd_triang_global = Wu_temp(1:numWuPoints, 1:9);

figure
hold on
plot3(Wu_fd_triang_global(:,1), ... 
      Wu_fd_triang_global(:,2), ...
      Wu_fd_triang_global(:,3), 'blue.');


varargout(1,1)={Du_fd_points};
varargout(2,1)={Du_fd_TRI};
varargout(3,1)={Wu_triang};
varargout(4,1)={Wu_fd_triang,};
varargout(5,1)={Wu_fd_triang_global};








