function varargout=triangulate_ball(center, radius, ...
    num_angle_divisions, num_radial_divisions)


%=======================================================
%
%This function computes a triangulation of the planar
%ball with the given center, radius, number or
%angular divisions, and number of radial divisions.
%
%=======================================================

m = num_radial_divisions;
n = num_angle_divisions;

totalPossiblePoints = m*n;
D = zeros(totalPossiblePoints, 2);

r_step = radius/m;
theta_step = 2*pi/n;

u_pointNum = 1;

the_rs = (0.0:r_step:radius);
thetas = (0.0:theta_step:(2*pi));

for i = 1:m
   rad_number = i;
   %this_r = i*r_step;
    for j = 1:n
       D((i-1)*n + j, 1:2) = the_rs(i)*[cos(thetas(j)), sin(thetas(j))]; 
    end
end    

D(:,1) = D(:,1) + center(1);
D(:,2) = D(:,2) + center(2);
D = unique(D, 'rows');

x = D(:,1);
y = D(:,2);
TRI = delaunay(x, y);

varargout(1,1) = {D};
varargout(1,2) = {TRI};

return