function varargout = validatedBranchError_stableAtOrigin(...
    Q_MN1, Q_MN2, Q_MN3, nu, tau, s, E_MN, ...
    Q_omega_sigmaNorm, delta_Q, Q_omegaInv_sigmaNorm, delta_Qinv,...
    lambda_series1, lambda_series2, delta_lambda1, delta_lambda2,...
    H_alpha_validatedBounds, M, N, sigma, beta, rho);



[Qbar_MN1, Qbar_MN2, Qbar_MN3] = ...
                     convertToPoly_3D(Q_MN1, Q_MN2, Q_MN3, M, N);


%Bound on image of Q_MN:                 
rhoBar = norm([polyval(Qbar_MN1, tau, nu, nu);
               polyval(Qbar_MN2, tau, nu, nu);
               polyval(Qbar_MN3, tau, nu, nu)], inf);
        
rhoPrime = rhoBar + s           
           

%Bound DF on image of Q:

DF_bound = norm([2*abs(sigma), ...
                 1+abs(rho)+tau+2*rhoBar, ...
                 2*rhoBar+abs(beta)], inf)

K1 = 2
Nf = 4
             
partial_1_Qbar_MN1 = abs(pderiv(Qbar_MN1, 'x'));
partial_1_Qbar_MN2 = abs(pderiv(Qbar_MN2, 'x'));
partial_1_Qbar_MN3 = abs(pderiv(Qbar_MN3, 'x'));

partial_2_Qbar_MN1 = abs(pderiv(Qbar_MN1, 'y'));
partial_2_Qbar_MN2 = abs(pderiv(Qbar_MN2, 'y'));
partial_2_Qbar_MN3 = abs(pderiv(Qbar_MN3, 'y'));




boundMatrix = ...
[polyval(partial_1_Qbar_MN1, tau, nu, nu), ...
    polyval(partial_2_Qbar_MN1, tau, nu, nu);
polyval(partial_1_Qbar_MN2, tau, nu, nu), ...
    polyval(partial_2_Qbar_MN2, tau, nu, nu);
polyval(partial_1_Qbar_MN3, tau, nu, nu), ...
    polyval(partial_2_Qbar_MN3, tau, nu, nu)]


%Bound on D_theta Q_MN:
D_theta_QMN = norm(boundMatrix, inf)


%eigenvalus Bounds:
mu_superStar = intval(0);

for m = 0:M
   mu_superStar = mu_superStar + abs(lambda_series1(m+1))*tau^m; 
end

mu_superStar = mu_superStar + delta_lambda1

testNorm1 = intval(0);

for m = 1:M
   testNorm1 = testNorm1 + abs(lambda_series1(m+1))*tau^m; 
end

testNorm1 = abs(lambda_series1(1)) - testNorm1 - delta_lambda1
testNorm2 = abs(lambda_series2(1))

testDelta = norm([1/testNorm1, 1/testNorm2], inf);
mu_subStar = 1/testDelta



%Delta_HN Bound:
delta_HN = intval(0);

for n1 = 0:N
    for n2 = 0:N
        if (n1 + n2 <= N)
            delta_HN = delta_HN + ...
                H_alpha_validatedBounds(n1+1, n2+1)*nu^(n1+n2);                
        end
    end
end

delta_H_norm = delta_HN
delta_DHN = intval(0);

for n1 = 0:N
    for n2 = 0:N
        if (n1 + n2 <= N)
            delta_DHN = delta_DHN + ...
                (n1 + n2)*H_alpha_validatedBounds(n1+1, n2+1)*nu^(n1+n2);
        end
    end
end

delta_DHN = mu_superStar*delta_DHN

%++++++++++++++++++++++++++:
%The Complets residual term (accounting also for truncation errors) is:
%++++++++++++++++++++++++++:

sigmaNorm_E = E_MN + DF_bound*delta_HN + ...
              delta_HN^2 + 2*D_theta_QMN*delta_lambda1*nu + delta_DHN             


%Now compute the validated a-posteriori error.
%First the bounds on the inverse linear operator:

C1 = (Q_omega_sigmaNorm + delta_Q)*(Q_omegaInv_sigmaNorm + delta_Qinv)

C2 = intval(0);

for n1 = 0:N
    for n2 = 0:N
        if ((n1+n2 <= N) & (n1 + n2 >= 1))
            M_alpha = intval(0);
            for m = 0:M
                A_mAlpha = ...
                 [0,                      0,       0;
                  Q_MN3(m+1, n1+1, n2+1), 0,       Q_MN1(m+1, n1+1, n2+1);
                  Q_MN2(m+1, n1+1, n2+1), Q_MN1(m+1, n1+1, n2+1), 0];
                M_alpha = M_alpha + norm(A_mAlpha, inf)*tau^m;
            end
            M_alpha = M_alpha + 2*H_alpha_validatedBounds(n1+1, n2+1);
            C2 = C2 + (M_alpha*nu^(n1+n2))/(mu_subStar*(n1+n2));
        end
    end
end

goodComputation = 0;
conditionNumber = inf((N+1)*mu_subStar - mu_superStar)
if (conditionNumber > 0)
    goodComputation = 1
end

L_bound = C1*exp(C2)/((N+1)*mu_subStar - mu_superStar)


delta_HN = 2*L_bound*sigmaNorm_E

conditionVector = ...
    [1/(2*3*exp(intval(1))*intval('pi')*Nf*K1*L_bound);
    s/exp(intval(1))]
if (sup(delta_HN) >= inf(norm(conditionVector, inf)))
    goodComputation = 0
end

delta_H_norm

varargout(1,1)={delta_HN};
varargout(2,1)={goodComputation};


